/*
(c)2002 Ben Woodhouse

ben@elf-stone.com
www.elf-stone.com

This code may not be used in a commercial product without my permission.
If you redistribute it, this message must remain intact. 
*/
// AIPlayer.cpp: implementation of the AIPlayer class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "AIPlayer.h"
#include "dauntless.h"
#include "wizard.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AIPlayer::AIPlayer()
{
	decisionTimer=0;
	numHighPoints=3;
	destPriority=0;

	highPoint[0][0]=46;
	highPoint[0][1]=88;

	highPoint[1][0]=118;
	highPoint[1][1]=52;

	highPoint[2][0]=-9;
	highPoint[2][1]=4;

	destNorm[0]=1; destNorm[1]=0;
	destAngle=0;
	destDist=0;
	sensorLength=20;
	numHazards=0;
	travelTime=0;
	oldPos[0]=0;
	oldPos[1]=0;
	active=true;

	timeBetweenAttacks=100;
	attackTimer=100;
	decisionTimer=0;
	skill=1;
	mood=MOOD_WANDERING;
}

AIPlayer::~AIPlayer()
{

}

void AIPlayer::setSkill(float skill_in)
{
	if (skill_in<0.1) skill_in=0.1;
	skill=skill_in;


	timeBetweenAttacks=100/(skill); 
	
	//glPrintf("tba %f skill%f",timeBetweenAttacks,skill);
}


void AIPlayer::sendMessage(int message, float pos[3])
{
	float * wpos=wiz->phys->pos;
	float vect[3];
//	float dist;
	float vect2d[2];
	float dist2d;


	if (!active) return;


	if (message!=AIS_FLOOD)
	{
		float oneOverDist;
		vect[0]=pos[0]-wpos[0];
		vect[1]=pos[1]-wpos[1];
		vect[2]=pos[2]-wpos[2];

		vect2d[0]=vect[0];
		vect2d[1]=vect[2];

		float dist=lengthOf(vect);

		//normalise the vector
		if (dist>0.01)
		{
			oneOverDist=1.0/dist;
			vect[0]*=oneOverDist;
			vect[1]*=oneOverDist;
			vect[2]*=oneOverDist;
		}
		else
		{
			vect[0]=1;
			vect[1]=0;
			vect[2]=0;
		}

		dist2d=sqrt(vect2d[0]*vect2d[0]+vect2d[1]*vect2d[1]);
		oneOverDist=1.0/dist2d;

		if (dist2d>0.01)
		{
			vect2d[0]*=oneOverDist;
			vect2d[1]*=oneOverDist;
		}
	}

	
	switch (message)
	{
		case AIS_FLOOD:
			//if not there already
			//move to high ground, as a matter of priority
			gotoNearestHighpoint(1);
		break;
		case AIS_ENEMYPOS:
			enemyPos[0]=pos[0];
			enemyPos[1]=pos[1];
			enemyPos[2]=pos[2];
		break;
		case AIS_FIREBALL:
			//move somewhere else
		break;
		case AIS_TORNADO:
			if (dist2d<100)
			{
				setDest(wpos[0]-(vect2d[0]*50),wpos[2]-(vect2d[1]*50),10,0.8);
			}
			else
			{
				addHazard(pos,110,1);
			}
		break;
		case AIS_QUICKSAND:
			if ((random(10)<skill*10))
			{
				//move somewhere else quickly
				if (dist2d<25)
				{
					setDest(wpos[0]-(vect2d[0]*30),wpos[2]-(vect2d[1]*30),10,0.9);
				}			
				else if (dist2d<25)
				{
					setDest(wpos[0]-(vect2d[0]*30),wpos[2]-(vect2d[1]*30),10,0.7);
				}			
				else
				{
					addHazard(pos,25,1);
				}
			}
		break;
	}
}


void AIPlayer::doStuff()
{
	if (wiz->getEnemy()->isAlive() && wiz->isAlive()) //reactivate
	{
		active=true;
	}

	if (!active) return;

	//check if either wizard is dead
	if (!wiz->getEnemy()->isAlive() || !wiz->isAlive())
	{
		active=false;
		wiz->stop();
		return;
	}

	changeMoodTimer-=speedmod;
	if (changeMoodTimer<0)
	{
		if (random(8)<6) mood=0;
		else mood=random(2)+1;
		changeMoodTimer=random(1000);
	}
	attackTimer-=speedmod;
	bool attacked=false;
	if (destPriority>0)
	{
		gotoDestination();
		hazardSensor();
	}
	else
	{
		bool makeDecision=false;

		//hesitation
		decisionTimer-=speedmod;
		if (decisionTimer<=0)
		{
			if (random(10)<skill*10) decisionTimer=500.0*(0.3+0.7*(1.0-skill));
			else decisionTimer=0;
			//glCls();
			//glPrintf("decisionTimer %f",decisionTimer);
			makeDecision=true;
		}
	
		if (attackTimer<0)
		{
			attacked=attack();
		}
		if (makeDecision)
		{
			if (!attacked)
			{
				switch (mood)
				{
					case MOOD_AGGRESSIVE:
						//or head for the enemy
						setDest(wiz->getEnemy()->phys->pos[0],
								wiz->getEnemy()->phys->pos[2],10,0.5);
					break;
					case MOOD_DEFENSIVE:
						gotoNearestHighpoint(0.5);
					break;
					case MOOD_WANDERING:
						//go to a random destination
						setDest(random(300)-150,random(300)-150,20,0.3);
					break;
				}
			}
		}
		gotoDestination();
		hazardSensor();	
	}
	clearHazards();
}


void AIPlayer::setDest(float x, float y, float radius, float priority)
{
	if (priority>destPriority)
	{
		travelTime=0;
		dest[0]=x;
		dest[1]=y;
		destRad=radius;
		destPriority=priority;
	}
}

void AIPlayer::gotoDestination()
{
	float vect[2];
	vect[0]=dest[0]-wiz->phys->pos[0];
	vect[1]=dest[1]-wiz->phys->pos[2];

	float v[3];
	v[0]=vect[0];
	v[1]=0;
	v[2]=vect[1];
	float angle[3];

	travelTime+=speedmod;
	if (travelTime>100) //if going somewhere for more than a second, 
						//reassess priority
	{
		destPriority-=0.0005*speedmod;
		if (destPriority<0) destPriority=0;
	}


	checkStuck();
	oldPos[0]=wiz->phys->pos[0];
	oldPos[1]=wiz->phys->pos[2];

	coordsToAngles(v,&destDist,angle);

	if (destDist>0.01)
	{
		destNorm[0]=vect[0]/destDist;
		destNorm[1]=vect[0]/destDist;
	}
	else
	{
		destNorm[0]=1;
		destNorm[1]=0;
	}

	destAngle=angle[1];



	if (destDist<=destRad)
	{
		travelTime=0;
		wiz->stop();
		destPriority=0;
	}
	else
	{
		wiz->turn(destAngle);
		wiz->walk();
	}
}

void AIPlayer::hazardSensor()
{

	float priority;
	priority=testVect(destNorm);
	//take evasive action
	if (priority>0)
	{
		//turn 90 degrees;
		float newVect[2];
		float yrad;
		bool foundSafeAngle=false;

		//test different angles until a better safer one can be found
		//find an ok vector
		for (int a=1;a<=3;++a)
		{
			yrad=(destAngle+45*a)*dera;
			newVect[0]=cos(yrad);
			newVect[1]=sin(yrad);
			if (testVect(newVect)==0)
			{
				foundSafeAngle=true;
				break;
			}

			yrad=(destAngle-45*a)*dera;
			newVect[0]=cos(yrad);
			newVect[1]=sin(yrad);
			if (testVect(newVect)==0)
			{
				foundSafeAngle=true;
				break;
			}
		}

		//if no safe angle is found, check behind
		if (!foundSafeAngle)
		{
			yrad=(destAngle+180)*dera;
			newVect[0]=cos(yrad);
			newVect[1]=sin(yrad);

			if (testVect(newVect)==0)
			{
				foundSafeAngle=true;
			}
			priority=1.1;
		}
//		newVect[0]=(sin(yrad)*destNorm[1] + cos(yrad)*destNorm[0])*30;
//		newVect[1]=(cos(yrad)*destNorm[1] - sin(yrad)*destNorm[0])*30;
		if (foundSafeAngle)
		{
			setDest(wiz->phys->pos[0]+newVect[0]*30,
					wiz->phys->pos[2]+newVect[1]*30, 10,priority);
		}
		else
		{
			//emergency strategy: RUN TO THE HILLS!
			//(useful if you've just been dumped in lots of water)
			gotoNearestHighpoint(1.2);
		}
	}
	else
	{
		//glCls();
		//glPrintf("no hazard detected");
	}
}

float AIPlayer::testVect(float vect[2])
{
	float sensorPos[2];
	float sLength=sensorLength;

	if (sLength>=destDist)
	{
		sLength=destDist;		
	}
	sensorPos[0]=wiz->phys->pos[0]+vect[0]*sLength;
	sensorPos[1]=wiz->phys->pos[2]+vect[1]*sLength;

	//detect the sea
	float seaHeight=mainGM.getSea()->getYAt(sensorPos[0],sensorPos[1]);
	float landHeight=mainGM.getLand()->getYAt(sensorPos[0],sensorPos[1]);

	if (seaHeight>landHeight)
	{
		return 1;
	}

	//check for other hazards
	for (int a=0;a<numHazards;++a)
	{
		float v[2];
		v[0]=sensorPos[0]-hazards[a].pos[0];
		v[1]=sensorPos[1]-hazards[a].pos[2];

		float dist=sqrt(v[0]*v[0]+v[1]*v[1]);

		if (dist<hazards[a].radius)
		{
			return hazards[a].danger;
		}
	}
	return 0;
}

void AIPlayer::addHazard(float * pos, float radius, float danger)
{
	if (numHazards>=MAX_HAZARDS-1) return;
	Hazard &haz=hazards[numHazards];

	haz.radius=radius;
	haz.pos[0]=pos[0];
	haz.pos[1]=pos[1];
	haz.pos[2]=pos[2];
	haz.danger=danger;
	++numHazards;
}

void AIPlayer::clearHazards()
{
	numHazards=0;
}



void AIPlayer::checkStuck()
{
	float v[2];
	if (destDist>0.01)
	{
		v[0]=oldPos[0]-wiz->phys->pos[0];
		v[1]=oldPos[1]-wiz->phys->pos[2];

		if (fabs(v[0])<0.02 && fabs(v[1])<0.02)
		{
			//not getting anywhere; abandon the destination
			destPriority=0;
		}
	}
}

void AIPlayer::gotoNearestHighpoint(float priority)
{
	if (priority<=destPriority)
	{
		return;
	}

	float dist;
	int nearest=getClosestHighpoint(wiz->phys->pos,dist);

	setDest(highPoint[nearest][0],highPoint[nearest][1],20,priority);
}

int AIPlayer::getClosestHighpoint(float * location, float &distOut)
{
	float hDist;
	float hv[2];

	distOut=1000000;
	int nearest=0;

	for (int a=0;a<3;++a)
	{
		hv[0]=highPoint[a][0]-location[0];
		hv[1]=highPoint[a][1]-location[2];
		hDist=sqrt(hv[0]*hv[0]+hv[1]*hv[1]);

		if (hDist<distOut)
		{
			distOut=hDist;
			nearest=a;
		}
	}
	return nearest;
}


bool AIPlayer::attack()
{
	float epos[3];
	float * wpos;
	float ve[2];
	int spell=-1;

	bool avail[4];
	for (int a=0;a<4;++a)
	{
		if (wiz->getSpellPower(a)==1)
		{
			avail[a]=true;
		}
		else
		{
			avail[a]=false;
		}
	}


	wpos=wiz->phys->pos;
	epos[0]=wiz->getEnemy()->phys->pos[0];
	epos[1]=wiz->getEnemy()->phys->pos[1];
	epos[2]=wiz->getEnemy()->phys->pos[2];

	//lower skill level innacuracy
	if (random(10)>skill*10)
	{
		epos[0]+=(1.0-skill)*(random(50)-25);
		//epos[1]+=skill*(random(50)-25);
		epos[2]+=(1.0-skill)*(random(50)-25);
	}	
	ve[0]=epos[0]-wpos[0];
	ve[1]=epos[0]-wpos[0];

	float enemyDist=sqrt(ve[0]*ve[0]+ve[1]*ve[1]);
	float enemyDistFromHills;
	float distFromHills;

	if (avail[WATER])
	{
		gotoNearestHighpoint(0.8);
		(void)getClosestHighpoint(epos,enemyDistFromHills);
		(void)getClosestHighpoint(wpos,distFromHills);

		if (enemyDistFromHills>100 && distFromHills<100)
		{
			spell=WATER;
		}
	}
	if (spell==-1)
	{
		if (avail[AIR] && enemyDist>160 && enemyDist<250)
		{
			spell=AIR;
		}
		else if (avail[EARTH] && enemyDist>150 && enemyDist<200)
		{
			spell=EARTH;
		}
		else if (avail[FIRE] && enemyDist<200)
		{
			spell=FIRE;
		}
	}
	
	if (spell==-1) 
	{
		return false;
	}
	attackTimer=timeBetweenAttacks;
	wiz->setCurrentSpell(spell);
	wiz->castSpell(epos);
	return true;
}
