/*
(c)2002 Ben Woodhouse

ben@elf-stone.com
www.elf-stone.com

This code may not be used in a commercial product without my permission.
If you redistribute it, this message must remain intact. 
*/
// ActionManager.cpp: implementation of the ActionManager class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ActionManager.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ActionManager::ActionManager()
{
	currentFrame=0;
	numActions=0;
	setAction(0);
	setDestAction(0);
	framesPerSec=15;
	frameTime=1.0/framesPerSec;
	doubleFrameRate=false;

	timer.start();
}

ActionManager::~ActionManager()
{
}

void ActionManager::reset()
{
	currentFrame=0;
	numActions=0;
	setAction(0);
	setDestAction(0);
	doubleFrameRate=false;
	timer.start();
	currentAction=&actions[currentActionNo];
}

void ActionManager::setFPS(float framesPerSec)
{
	this->framesPerSec=framesPerSec;
	frameTime=1.0/framesPerSec;
}

void ActionManager::speedToAction(int actionID)
{
	setDestAction(actionID);

	if (!doubleFrameRate)
	{

		speedAction=actionID;
		oldFPS=framesPerSec;
		framesPerSec*=3;
		doubleFrameRate=true;
	}
}

bool ActionManager::assignAction(char * name, int startFrame, int endFrame, int schematicX, int schematicY)
{
	if (numActions>MAX_ACTIONS) return false;
	
	actions[numActions].startFrame=startFrame;
	actions[numActions].endFrame=endFrame;
	actions[numActions].endFrame=endFrame;
	actions[numActions].numFrames=endFrame-startFrame+1;
	actions[numActions].index=numActions; 
	actions[numActions].schematicX=schematicX;
	actions[numActions].schematicY=schematicY;
	sprintf(actions[numActions].name,"%s",name);
	++numActions;
	return true;
}

void ActionManager::linkAction(int actionNuma, int actionNumb)
{
	actions[actionNuma].links.push_back(actionNumb);
}

void ActionManager::init()
{
	numFrames=b3d->getAnim(0)->numFrames;
	currentAction=&actions[currentActionNo];
}

/*
void ActionManager::buildActionMaps()
{
	bool * actionsLinked=new bool[numActions];

	for (int a=0;a<numActions;++a) actionsLinked[a]=false;

	Action * action=&actions[0];

	for (a=0;a<numActions;++a)
	{
		buildActionMap(a,actionsLinked);
	}

	//find how to get to each action
	actionsLinked=0;

	delete [] actionsLinked;

//	buildActionMap();
}
*/

/*
void ActionManager::buildActionMap(int aNum, bool * actionsLinked)
{
	actionsLinked[aNum]=true;

	Action * action=&actions[aNum];
	int lNum;

	//for each link, if it isn't already linked, link it
	for (int a=0;a<action->links.size();++a)
	{
		lNum=action->links.at(a);
		if (actionsLinked[lNum])
		{
		}
		else
		{
			//link a
			for (int b=0;b<numActions;++b)
			{
			}
			buildActionMap(lNum, actionsLinked);
		}
	}

	
}
*/
void ActionManager::draw()
{
	/*
	if (b3d->drawb)
	{
		for (int m=0;m<numAnims;++m)
		{
			if (anim[m].drawb)
				anim[m].draw(currentAction->startFrame+currentFrame);
		}
		for (m=0;m<numMeshes;++m)
		{
			if (mesh[m].drawb)
				mesh[m].draw();
		}
	}
*/
//	timer+=speedmod;
	double time=timer.getElapsed();

	currentFrame=((float)time/(float)frameTime);
	if (currentFrame>=currentAction->numFrames) 
	{
		
		gotoNextAction();
		if (doubleFrameRate && currentActionNo==speedAction) 
		{
			setFPS(oldFPS);
			doubleFrameRate=false;
		}

		//currentFrame=currentAction->startFrame;
		currentFrame=0;

		timer.start();
	}

	b3d->draw(currentAction->startFrame+currentFrame); 

//	glCls();
//	glPrintf("action %d = '%s'",currentActionNo,currentAction->name);
	
/*	
	while (timer>5)
	{
		timer-=5;
		++currentFrame;

		//if the anim is at the end then play the top one on the stack
		//and delete it
		//if the stack is empty, keep current anim the same
		if (currentFrame>=anim[0].numFrames) 
		{
			if (animStack.head!=NULL)
			{
				currentAnim=animStack.head->dat;
				animStack.deleteElement(0);
			}
			currentFrame=0;
		} 
		glCls();
		glPrintf("frame: %d",currentFrame);
	}
	}*/

}

void ActionManager::setAction(int actionID)
{
	currentActionNo=actionID;
	currentAction=&actions[actionID];
}

void ActionManager::setDestAction(int actionID)
{
	destActionNo=actionID;
	destAction=&actions[actionID];
}


void ActionManager::gotoNextAction() 
{
	if (currentAction->links.size()==0)
	{
		return;
	}
	if (currentAction->links.size()==1)
	{  
		setAction(currentAction->links.at(0));
		return;
	}

	//get the schematic vector from the current frame to the dest frame
	/*
	int vectx=destAction->schematicX-currentAction->schematicX;
	int vecty=destAction->schematicY-currentAction->schematicY;
	*/

	//get the distance to from each link action to the dest action
	int vx,vy;
	float shortestDist=10000000;
	float dist;
	int targetActionID=currentActionNo;
	int linkID;

	for (int a=0;a<currentAction->links.size();++a)
	{
		linkID=currentAction->links.at(a);
		vx=destAction->schematicX - actions[linkID].schematicX;
		vy=destAction->schematicY - actions[linkID].schematicY;

		dist=getSchematicDist(vx,vy);
		if (dist<shortestDist)
		{
			shortestDist=dist;

			targetActionID=linkID;
		}
	}

	setAction(targetActionID);
}

float ActionManager::getSchematicDist(int x, int y)
{
	float fx=x;
	float fy=y;

	return sqrt(fx*fx+fy*fy);
}

