/*
(c)2002 Ben Woodhouse

ben@elf-stone.com
www.elf-stone.com

This code may not be used in a commercial product without my permission.
If you redistribute it, this message must remain intact. 
*/
// AnimMesh.cpp: implementation of the AnimMesh class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "AnimMesh.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------
//AnimMesh Class
//---------------------------------------------------------------------------

void AnimMesh::CreateFrames(int num)
{
    frame=new CFrame [num];
    numFrames=num;
}


void AnimMesh::draw(int frameNo)
{


	float black[4]={0,0,0,0};
	float white[4]={1,1,1,1};
	float col[4]={0,0,0,1};
	CSubMesh * subp;

	CVertex * v=frame[frameNo].ver;
	float (* vNorm)[3]=frame[frameNo].vNorm;




	int a,s;

	if (doubleSided)
	{
		glDisable(GL_CULL_FACE);
		glLightModelf(GL_LIGHT_MODEL_TWO_SIDE,1.0);
	}

	for (s=0;s<numSubs;++s)
	{
		subp=&sub[s];
		if (subp->texNum!=-1) 
		{
		 	parentB3D->getTex(subp->texNum)->bind();
		}
				
		else
		{
			glPushAttrib(GL_LIGHTING_BIT);
			glPushAttrib(GL_ENABLE_BIT);
			//glDisable(GL_TEXTURE_2D);
			setTextureLevel(0);
//			glDisableTexturing();
			glEnable(GL_COLOR_MATERIAL);

			if (doubleSided) 
			{
				glColorMaterial(GL_FRONT,GL_DIFFUSE);

				for (int a=0;a<3;++a)
				{
					col[a]=color[a]*0.8;
				}
				glMaterialfv(GL_BACK,GL_DIFFUSE,col);
				glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,black);
				glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,black);

			}

			else
			{
				glColorMaterial(GL_FRONT,GL_DIFFUSE);

				glMaterialfv(GL_FRONT,GL_AMBIENT,black);
				glMaterialfv(GL_FRONT,GL_SPECULAR,black);
			}
		}
		
		glBegin(GL_TRIANGLES);
			for (a=0;a<subp->numTris;++a)
			{
				for (int b=0;b<3;++b)
				{
					if (subp->texNum==-1)
						glColor3fv(color);
					//glNormal3fv(	subp->tri[a].verNorm[b]   );
//					glNormal3fv( vNorm[ subp->tri[a].ver[b] ] );
					glNormal3fv( vNorm[ subp->tri[a].ver[b] ] );

					glTexCoord2f(	subp->tri[a].texCoord[b][0], 
										subp->tri[a].texCoord[b][1]  );

					glVertex3fv(	v[ subp->tri[a].ver[b] ].pos  );
				}
			}
		glEnd();

		if (subp->texNum==-1) 
		{
			glPopAttrib();
			glPopAttrib();
		}
		
	}//subMesh loop
	if (doubleSided)
	{
		glEnable(GL_CULL_FACE);
		glLightModelf(GL_LIGHT_MODEL_TWO_SIDE,0.0);
	}
	
}



bool AnimMesh::writeToStream(FILE * str)
{
  //write the static mesh data
    _putw(strlen(name),str);
    fputs(name,str);
    _putw(numVers,str);
    _putw(numSubs,str);

    fwrite(color,sizeof(float),3,str);
    fwrite(&doubleSided,sizeof(bool),1,str);

    //write the vertices
    //if (numVers>0) fwrite(ver,sizeof(CVertex),numVers,str);

    //write the sub meshes
    for (int a=0;a<numSubs;++a)
    {
        _putw(sub[a].numTris,str);
        _putw(sub[a].texNum,str);
        fwrite(sub[a].tri,sizeof(CTriangle),sub[a].numTris,str);
    }

    //other frames
    _putw(numFrames,str);

    for (a=0;a<numFrames;++a)
    {
        fwrite(frame[a].ver,sizeof(CVertex),numVers,str);
    }
	return true;

}


bool AnimMesh::readFromStream(FILE * str)
{
//	messageBox("anim");

    int len=_getw(str);
    fgets(name,len+1,str);

    numVers=_getw(str);
    numSubs=_getw(str);

//	messageBox("numVers %d, numSubs %d",numVers,numSubs);

    fread(color,sizeof(float),3,str);
    fread(&doubleSided,sizeof(bool),1,str);

    //allocate the vertices and submeshes
 /*   if (numVers>0) ver=new CVertex[numVers];
	 else ver=NULL;*/

    sub=new CSubMesh[numSubs];

    //read the vertices
    //if (numVers>0) fread(ver,sizeof(CVertex),numVers,str);

    //read the sub meshes
    for (int a=0;a<numSubs;++a)
    {
        sub[a].numTris=_getw(str);
        sub[a].texNum=_getw(str);
        sub[a].tri=new CTriangle[sub[a].numTris];
        fread(sub[a].tri,sizeof(CTriangle),sub[a].numTris,str);
    }

	numFrames=_getw(str);
//	messageBox("numFrames %d",numFrames);

	//allocate the frames
	frame=new CFrame[numFrames];

	//read the vertices for each frame
	for (int b=0;b<numFrames;++b)
	{
		//allocate the vertices
		frame[b].ver=new CVertex[numVers];

		int count=fread(frame[b].ver,sizeof(CVertex),numVers,str);
		//messageBox("ReadCount: [%d] = %d",b,count);
			 
	}

	return true; 
}



void AnimMesh::calculateVNorms()
{
	SimpleTri * tri=getSimpleTris();

	for (int a=0;a<numFrames;++a)
	{
		frame[a].calculateVNorms(tri,getNumTris(),numVers);
	}

	delete [] tri;
}




void CFrame::calculateVNorms(SimpleTri * tri,int numTris,int numVers)
{
	vNorm=new float [numVers][3];
	float (* tNorm)[3]=new float [numTris][3];

	calculateTriNorms(tri,numTris,tNorm);
	calculateVerNorms(tri,tNorm,numTris,numVers);
	delete [] tNorm;
}

void CFrame::calculateTriNorms(SimpleTri * tri, int numTris,float (* norms) [3])
{
	for (int a=0;a<numTris;++a)
	{
		/*
		float vertex[3][3];

		for (int b=0;b<3;++b)
		{
			for (int c=0;c<3;++c)
			{
				vertex[2-b][c]=ver[ tri[a].ver[b] ].pos[c];
			}
		}*/

		getTriNormal(norms[a], 
						ver[ tri[a].ver[0] ].pos, 
						ver[ tri[a].ver[1] ].pos, 
						ver[ tri[a].ver[2] ].pos);
	}
} 

void CFrame::calculateVerNorms(SimpleTri * tri, float (*tNorm)[3], int numTris, int numVers)
{
	//set all vnorms to 0,0,0
	for (int a=0;a<numVers;++a)
	{
		vNorm[a][0]=0;
		vNorm[a][1]=0;
		vNorm[a][2]=0;
	}

	//add the triangle norms from the adjacent tris
	for (a=0;a<numTris;++a)
	{
		for (int b=0;b<3;++b)
		{
			int vnum=tri[a].ver[b];
			vNorm[vnum][0]+=tNorm[a][0];
			vNorm[vnum][1]+=tNorm[a][1];
			vNorm[vnum][2]+=tNorm[a][2];
		}
	}

	//normalise the vnorms
	for (a=0;a<numVers;++a)
	{
		normalize(vNorm[a]);
	}
}

float * AnimMesh::getVertexPos(int frameNo,int v)
{
	return frame[frameNo].ver[v].pos;
}


//---------------------------------------------------------------------------
