/*
(c)2002 Ben Woodhouse

ben@elf-stone.com
www.elf-stone.com

This code may not be used in a commercial product without my permission.
If you redistribute it, this message must remain intact. 
*/
// ArtLight.cpp: implementation of the ArtLight class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ArtLight.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ArtLight::ArtLight()
{
	exist=false;
}

ArtLight::ArtLight(const ArtLight &src) //copy constructor (required by STL)
{
	copyFrom(src);
}

ArtLight::~ArtLight()
{

}



const ArtLight& ArtLight::operator= (const ArtLight &src) //(required by STL)
{
	copyFrom(src);
	return *this;
}

void ArtLight::copyFrom(const ArtLight &src)
{
	originx=src.originx;
	originy=src.originy;
	image=src.image;
}



DynTexture::DynTexture()
{
	numLights=0;
//	saveMe=false;
//	composite512.allocate(512,512,3);
//	composite64.setLikelyFormat();
}

DynTexture::~DynTexture()
{
}



void DynTexture::addNewLightImage(CImage &image)
{
	CImage imag[10];
	imag[0]=image;
	imag[0].removeLSB();
	lightImages.push_back(imag[0]);

	//create a sequence of images with decreasing alpha
	for (int a=1;a<10;++a)
	{
		imag[a]=imag[0];
//		imag[a].adjustBrightness(1.0-(float)a*0.2);
		imag[a].adjustChannelsBrightness(CHANNEL_RED, 1.0-(float)a*0.1);
		lightImages.push_back(imag[a]);
	}
}

bool DynTexture::loadNewLightImage(char * filename)
{
	
	CImage imag[5];
	imag[0].loadBMP(filename);
	imag[0].removeLSB();
	lightImages.push_back(imag[0]);

	//make the light images increasingly dark
	for (int a=1;a<5;++a)
	{
		imag[a]=imag[0];
		imag[a].adjustBrightness(1.0-(float)a*0.2);
		imag[a].removeLSB();
		lightImages.push_back(imag[a]);
	}

	return true;
}

void DynTexture::loadImage(char * filename)
{
	image.loadBMP(filename);
	tex=loadTexFromImage(image,TF_L,TF_L);
	composite=image;
}

int DynTexture::drawLight(int x, int y, int imgNo, bool alphaBlend)
{
	ArtLight * alight;

	int remx=x%4;
	int remy=y%4;

	if (remx<=1) x-=remx;
	else x+=(4-remx);
	if (remy<=1) y-=remy;
	else y+=(4-remy);

	if (numLights==MAX_ARTLIGHTS)
	{
		return -1;
	}

	//int lightNo=getNewLightID();
	int lightNo=numLights;
	numLights++;
	alight=&lights[lightNo];

	try
	{
		alight->image=&lightImages.at(imgNo);
	}
	catch (std::out_of_range e)
	{
		return -1;
	}



 
	int width=alight->image->getWidth();
	int height=alight->image->getHeight();

	//if (width!=64 || height!=64) return;

	int halfwidth=width/2;
	int halfheight=height/2;

	int startx=x-halfwidth;
	int starty=y-halfheight;
	int endx=startx+width;
	int endy=starty+width;

	alight->originx=startx;
	alight->originy=starty;
	
	//check bounds
	if (startx<0 || starty<0 || endx>image.getWidth() || endy>image.getHeight()) 
	{
		numLights--;
		return-1;
	}
	
	if (alphaBlend)
	{
		//glCls();

		composite.regionAlphaBlendImg(startx,starty,*alight->image);
	}
		
	else //additive blend
	{
		composite.regionPlusImg(startx,starty,*alight->image);
	}
 
/*
	//create a backup image if required
	if (backupImg!=NULL)
	{
		backupImg->copyRegionFromNI(	image,startx,starty,
										0,0,
										alight->image->getWidth(),
										alight->image->getHeight());
	}
*/
	
	glPixelStoref(GL_UNPACK_ROW_LENGTH,image.getWidth());
	tex->bind();

	int datStart=(startx + starty * image.getWidth())*3;



	glTexSubImage2D(
		GL_TEXTURE_2D,    
		0,          
		startx,        
		starty,        
		width,        
		height,       
		GL_RGB,        
		GL_UNSIGNED_BYTE, 

		//(BYTE *) image.getData()+datStart
		(BYTE *) composite.getData()+datStart
		//(BYTE *) alight->image->getData()
	);
	glPixelStoref(GL_UNPACK_ROW_LENGTH,0);

	//return the created light number
	return lightNo;
}


/*
void DynTexture::clearLight(int lightNo,CImage *backupImg)
{
	//remove the lights
	tex->bind();
	glPixelStoref(GL_UNPACK_ROW_LENGTH,image.getWidth());

	if (lights[lightNo].exist) removeLight(lights[lightNo],backupImg);

	glPixelStoref(GL_UNPACK_ROW_LENGTH,0);
}
*/
void DynTexture::clearLights()
{
	//remove the lights
	tex->bind();
	glPixelStoref(GL_UNPACK_ROW_LENGTH,image.getWidth());

	for (int a=0;a<numLights;++a)
	{
		removeLight(lights[a]);
	}

	glPixelStoref(GL_UNPACK_ROW_LENGTH,0);

	numLights=0;
}

/*
int DynTexture::getNewLightID()
{
	for (int a=0;a<MAX_ARTLIGHTS;++a)
	{
		if (!lights[a].exist) 
		{
			lights[a].exist=true;
			++numLights;
			return a;
		}
	}
	return -1;
}
*/


void DynTexture::removeLight(ArtLight &lite)
{
	lite.exist=false;
	int datStart=(lite.originx + lite.originy*image.getWidth())*3;

	composite.copyRegionFrom(
							image,
							lite.originx,
							lite.originy,
							lite.image->getWidth(),
							lite.image->getHeight());

	glTexSubImage2D(
		GL_TEXTURE_2D,    
		0,          
	 	lite.originx,
		lite.originy,
		lite.image->getWidth(),        
		lite.image->getHeight(),        
		GL_RGB,        
		GL_UNSIGNED_BYTE,

		(BYTE *) composite.getData()+datStart
//		(BYTE *) image.getData()+datStart
	);

}
