/*
(c)2002 Ben Woodhouse

ben@elf-stone.com
www.elf-stone.com

This code may not be used in a commercial product without my permission.
If you redistribute it, this message must remain intact. 
*/
// B3D.cpp: implementation of the B3D class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "B3D.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------
//B3D Class
//---------------------------------------------------------------------------

void B3D::setLowestColourComponent(float amt)
{
	for (int a=0;a<numMeshes;++a)
	{
		mesh[a].setLowestColourComponent(amt);
	}
	for (a=0;a<numAnims;++a)
	{
		anim[a].setLowestColourComponent(amt);
	}

}

void B3D::ground()
{
	int a;
	float lowest=1000000;

	for (int m=0;m<numMeshes;++m)
	{
			//find the bottom of the object 
			for (a=0;a<mesh[m].numVers;++a)
			{
				if (mesh[m].ver[a].pos[1]<lowest) lowest=mesh[m].ver[a].pos[1];
			}

	}

	for (m=0;m<numMeshes;++m)
	{
		//move the object 
		for (a=0;a<mesh[m].numVers;++a)
		{
			mesh[m].ver[a].pos[1]-=lowest;
		}
	}
}


void B3D::scale(float scale)
{
	int a,b;
	for (int m=0;m<numMeshes;++m)
	{
		for (a=0;a<mesh[m].numVers;++a)
		{
			for (b=0;b<3;++b)
			{
				mesh[m].ver[a].pos[b]*=scale;
			}
		}
	}
	for (m=0;m<numAnims;++m)
	{
		for (int f=0;f<anim[m].numFrames;++f)
		{
			for (a=0;a<anim[m].numVers;++a)
			{
				for (b=0;b<3;++b)
				{
					anim[m].frame[f].ver[a].pos[b]*=scale;
				}
			}
		}
	}
}

int B3D::AnimStackGetLast()
{
	if (animStack.tail!=NULL)
		return animStack.tail->dat;

	return currentAnim;
}

void B3D::AnimStackAdd(int num)
{
	animStack.insertAtEnd(&num);
}

void B3D::draw(int currentFrame)
{
/*	
	if (drawb)
	{

		//draw the current anim or the static meshes
		if (numAnims>0)
		{
			
			if (anim[currentAnim].drawb)
				anim[currentAnim].draw(anim[currentAnim].frame[currentFrame].ver);
//				anim[0].draw(anim[0].frame[0].ver);

			timer+=speedmod;

			if (timer>4)
			{
				timer-=4;
				++currentFrame;

				//if the anim is at the end then play the top one on the stack
				//and delete it
				//if the stack is empty, keep current anim the same
				if (currentFrame>=anim[currentAnim].numFrames) 
				{
					if (animStack.head!=NULL)
					{
						currentAnim=animStack.head->dat;
						animStack.deleteElement(0);
					}
					currentFrame=0;
				} 
			}

		}
 
		else
		{
		}

	}
*/
	if (drawb)
	{
 		for (int m=0;m<numMeshes;++m)
		{
			if (mesh[m].drawb)
				mesh[m].draw();
		}
		for (m=0;m<numAnims;++m)
		{
			if (anim[m].drawb)
				anim[m].draw(currentFrame);
		}
	}
}


void B3D::swapTextures(int texNuma, int texNumb)
{
	CTexture * temp=tex[texNuma];
	tex[texNuma]=tex[texNumb];
	tex[texNumb]=temp;
}

bool B3D::load(char * fn)
{
    FILE * filein;
    int len; 
    char str[50];
	 int a;

//	 debugLog_c debug("loaddebug.txt");

    char sig[4]="B3D";

    if (! (filein=fopen(fn,"rb")) )
    {
        goto error;
    }

    //read the B3D sig
    for (a=0;a<3;++a)
    {
        if (fgetc(filein)!=sig[a]) 
        {
            messageBox("Cannot read file: \nfile is not of correct format.");
            goto error;
        }
    }

	//debug.write("B3D sig found. Assuming file is of correct format\n");

	 

    //read the B3D version
    for (a=0;a<5;++a)
    {
        version[a]=fgetc(filein);
    }


    numMeshes=_getw(filein);
    numTextures=_getw(filein);
    numAnims=_getw(filein);


	 if (numTextures>0) tex=new CTexture * [numTextures];
	 if (numAnims>0) anim=new AnimMesh [numAnims];
	 if (numMeshes>0) mesh=new Mesh [numMeshes];

/*
 	 debug.write("numMeshes:   %d \n",numMeshes);
 	 debug.write("numTextures: %d \n",numTextures);
 	 debug.write("numAnims:    %d \n",numAnims);
*/
	 //allocate the textures

 	 //debug.write("numAnims:    %d \n",numAnims);
	 //debug.update();


    for (a=0;a<numTextures;++a)
    {
		  //debug.write("Texture load    %d \n",a);
		  //debug.update();
        len=_getw(filein);

		  //debug.write("filename length:%d \n",len);
		  //debug.update();

        if (len<50) fgets(str,len+1,filein);
		  else
		  {
			  //debug.write("error: embedded texture filename too long.. Exiting");
			  goto error;
		  }

		  //debug.write(" filename:    %s \n",str);
		  //debug.update();
        //textures.createAtEnd();
        //tp=textures.tail;
		  long length=_getw(filein);
		  //debug.write(" length:    %d \n",length);
		  //debug.update();
		  long pos=ftell(filein);
  
//		  tex[a]=loadNewTexture(str,TF_LML,TF_LML,UNKNOWN,NULL,0,0,filein);
		  tex[a]=loadNewTexture(str,TF_LML,TF_LML,UNKNOWN,LOADTEX_NORMAL,filein);
		  tex[a]->setParams(GL_REPEAT,GL_REPEAT,-1,-1,-1);

		  //make sure the file pointer is exactly at the end of the embedded file
		  fseek(filein,pos+length,SEEK_SET);

		  if (tex[a]==NULL) 
		  {
			  //debug.write("error loading embedded texture.\n");
				goto error;
		  }

		  //debug.write(" Done!\n");
		  //debug.update();

        //tp->dat.data=new BYTE[tp->dat.length];
        //fread(tp->dat.data,sizeof(BYTE),tp->dat.length,filein);

        //textures.tail->dat.name=str;
 
    }

 	 //debug.write("loaded textures ok\n");
	 //debug.update(); 

    for (a=0;a<numMeshes;++a)  
    {
			mesh[a].readFromStream(filein);
			mesh[a].parentB3D=this; 
    }

 	 //debug.write("loaded statics ok\n"); 
	 //bug.update();

    for (a=0;a<numAnims;++a)
    {
	 	 //debug.write("anim %d\n",a);
		 //debug.update();
        //anims.createAtEnd();
        //amp=anims.tail;

		anim[a].readFromStream(filein);
		anim[a].parentB3D=this;
        /*
		anim[a].initFrame.readFromStream(filein);

		  //set the parent so it can find textures
			anim[a].parentB3D=this;
		  anim[a].initFrame.parentB3D=this;

        //copy the name from the initframe mesh to its parent anim mesh
        strcpy(anim[a].name, anim[a].initFrame.name);

        //other frames
		  anim[a].numVers=_getw(filein);
        anim[a].numFrames=_getw(filein);

        //allocate the frames
        anim[a].frame=new CFrame[anim[a].numFrames];

        //read the vertices for each frame
        for (int b=0;b<anim[a].numFrames;++b)
        {
            //allocate the vertices
            anim[a].frame[b].ver=new CVertex[anim[a].numVers];

            fread(anim[a].frame[b].ver,sizeof(CVertex),anim[a].numVers,filein);
			 	 
				debug.write("frame %d loaded\n",b);
				 debug.update();

        }
*/
        //amp=amp->next;

    }
 	 //debug.write("loaded anims ok\n");


    fclose (filein);
 
	calculateAnimNorms();
    return true;
error:

    
    return false;
}

void B3D::calculateAnimNorms()
{
	for (int a=0;a<numAnims;++a)
	{
		anim[a].calculateVNorms();
	}
}

AnimMesh *  B3D::findAnimMesh(const char * name)
{
	for (int a=0;a<numAnims;++a)
	{
		if (strcmp(name,anim[a].name)==0)
			return &anim[a];
	}
	return NULL;
}

Mesh *  B3D::findMesh(const char * name)
{
	for (int a=0;a<numMeshes;++a)
	{
		if (strcmp(name,mesh[a].name)==0)
			return &mesh[a];
	}
	return NULL;
}


//---------------------------------------------------------------------------
