// B3D.h: interface for the B3D class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_B3D_H__8DB0EE05_AB41_4AC5_860F_2BEE6D030D95__INCLUDED_)
#define AFX_B3D_H__8DB0EE05_AB41_4AC5_860F_2BEE6D030D95__INCLUDED_

#include "Mesh.h"
#include "AnimMesh.h"

//class for storing embedded texture files
struct CTextureFile
{
    CTextureFile()
    {
        data=NULL;
        length=0;
    }
    ~CTextureFile()
    {
        if (data!=NULL)
            delete data;
    }
	 std::string name; 
    BYTE * data;
    int length;
};

//split this into 2 
//or derive 2 types of b3d. Just do something, FFS
//B3D should only contain a model type so it can be reused when there
//is more than one instance of a particular model. It should not 
//be used for instance-only things like animation. Therefore, don't
//split it, just provide separate animation sprite (?) and static sprite (?)
//classes
//(and maybe split it as well)

//in other words:
//b3d currently contains all the animated and non-animated meshes in a scene. It should
//not be used as an animated mesh (action) network. Perhaps a class derived from b3D should
//do this. 
class B3D
{
    public:
        B3D()
        {
				timer=0;
				numMeshes=0;
				numTextures=0;
				numAnims=0;
				mesh=NULL;
				anim=NULL;
				tex=NULL;
				drawb=true;
        }
		~B3D()
		{
			if (mesh!=NULL) delete [] mesh;
			if (tex!=NULL) delete [] tex;
			if (anim!=NULL) delete [] anim;
		}
        bool load(char * fn);
		  
		//scaling and moving
		void ground();
		void scale(float scale);

		virtual void draw(int currentFrame=0);

		//erk??
		void AnimStackAdd(int num);
		int AnimStackGetLast(void);

        //will be new-allocated arrays in game version
        //will include draw func and texture objects in game version

        char version[5];

		int getNumMeshes(){return numMeshes;}
		int getNumTextures(){return numTextures;}
		int getNumAnims(){return numAnims;}
		void swapTextures(int texNuma, int texNumb);
		void calculateAnimNorms();

		Mesh * getMesh(int num){return &mesh[num];}
		AnimMesh * getAnim(int num){return &anim[num];}

		CTexture * getTex(int num){return tex[num];}

		bool drawb; 

		int currentAnim;
		CLinkedList <int> animStack;

		bool isDoneb;

		Mesh * findMesh(const char * name);
		AnimMesh *  findAnimMesh(const char * name);
		void setLowestColourComponent(float amt);




	protected:

		float timer;

		int numMeshes;
		Mesh * mesh;

		int numAnims;
		AnimMesh * anim;

		int numTextures;
		CTexture ** tex;
};


#endif // !defined(AFX_B3D_H__8DB0EE05_AB41_4AC5_860F_2BEE6D030D95__INCLUDED_)
