/*
(c)2002 Ben Woodhouse

ben@elf-stone.com
www.elf-stone.com

This code may not be used in a commercial product without my permission.
If you redistribute it, this message must remain intact. 
*/
// BoundingSphere.cpp: implementation of the BoundingSphere class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BoundingSphere.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


BoundingSphere::BoundingSphere()
{
	radius=10;
	numChildren=0;

	pos[0]=0;
	pos[1]=0;
	pos[2]=0;

	children=NULL;
}

BoundingSphere::~BoundingSphere()
{
	if (children!=NULL) delete [] children;
}

void BoundingSphere::createChildren(int numChildren)
{
	children=new BoundingSphere[numChildren];
	this->numChildren=numChildren;
}

void BoundingSphere::checkFrustum(ViewFrustum &frustum)
{
	int in=0;
	float d;

	float (*frust)[4]=frustum.frustum;

	for(int a=0;a<6;++a)
	{
		d=	frust[a][0]*pos[0]+
			frust[a][1]*pos[1]+
			frust[a][2]*pos[2]+
			frust[a][3];

		if(d<=-radius) 
		{
			vis=COL_OUT;
			return;
		}
		if(d>radius) ++in;
	}

//	glPrintf("%f %d",d,in);
	if (in==6) 
	{
		vis=COL_IN;
		return;
	}
	
	vis=COL_PART;
	//check the children
	if (children!=NULL)
	{
		for (a=0;a<numChildren;++a)
		{
			children[a].checkFrustum(frustum);
		}
	}
}


void BoundingSphere::setPos(float x,float y,float z)
{
	//get the move vector
	float v[3];
	
	v[0]=x-pos[0];
	v[1]=y-pos[1];
	v[2]=z-pos[2];

	//move all children (and their children) by it
	for (int a=0;a<numChildren;++a)
	{
		children[a].moveBy(v);
	}

	//set the pos
	pos[0]=x; pos[1]=y; pos[2]=z;
}

void BoundingSphere::moveBy(float v[3])
{
	pos[0]+=v[0];
	pos[1]+=v[1];
	pos[2]+=v[2];

	for (int a=0;a<numChildren;++a)
	{
		children[a].moveBy(v);
	}
}

void BoundingSphere::setDPos(int dim, float amt)
{
	//get the move vector
	float v=amt-pos[dim];

	//move all children (and their children) by it
	
	for (int a=0;a<numChildren;++a)
	{
		children[a].moveDBy(dim,v);
	}

	//set the pos
	pos[dim]=amt;
}

void BoundingSphere::moveDBy(int dim, float amt)
{
	pos[dim]+=amt;

	for (int a=0;a<numChildren;++a)
	{
		children[a].moveDBy(dim,amt);
	}
}

