/*
dauntless.h

(c)2000 Ben Woodhouse
*/

#define DIAGNOSTIC 1
#define PI 3.1415927

#define EARTH 0
#define AIR 1
#define FIRE 2
#define WATER 3

#define GM_START 0
#define GM_NORMAL 1
#define GM_BLANK 2

#define MODE_GAME 0
#define MODE_INTRO 1
#define MODE_ENDGAME 2
//********************************************************************
//CLASSES

#include "menu.h"
#include "heightmap.h"
#include "land.h"
#include "sea.h"
#include "ActionManager.h"
#include "wizard.h"
#include "WDPlayer.h"
#include "userPlayer.h"
#include "AIPlayer.h"
#include "NetPlayer.h"
#include "particleManager.h"
#include "particleFire.h"
#include "particleExplosion.h"
#include "particleTornado.h"
#include "particleRain.h"
#include "LightManager.h"
#include "SpellManager.h"
#include "Intro.h"

#include "GameInfoDisplay.h"

extern int frameNo;


//GAMEMODES

class mainGM_c:public gameMode
{
	public:

		mainGM_c();
		~mainGM_c();
		void mainLoop(void);
		void draw(void);
		void init(void); 
		void shutdown(void);
		void setSeaDepth(float depth){if (sea!=NULL) sea->setDepth(depth);}
		void newGame(bool multiplayer, int userWizardNo, float AISkill=1);

		void initParticles();

		CLand * getLand(){return &landscape;}
		CSea * getSea(){return sea;}

		void input();
		void cameraInput();

		void createPhysicsObjects();
		void displayEnergyBars();
		void createTreePhysOb(float x,float y,float z);
		void initSounds();
		void setWind(float windLvl){desiredWind=windLvl;}
		int ID;

		B3D wizB3D;
		Wizard wizard[2];
		WDPlayer *player[2];
		B3D trees;

		PhysicsSystem physics;
		void ambientSound();

		Sound * sounds[MAX_SOUNDS];
		int numSounds;
		LightManager lights;
		SpellManager * getSpellManager(){return &spellManager;}

		//relays messages to AIs
		void AIMessage(int message, float pos[3]);
		void setMode(int nextMode){this->nextMode=nextMode;}
		OrbitalCamera * getCam(){return orbitalCam;}
		void changeToMode(int mode);
		Tune * getThemeTune(){return &themeTune;}

		void wizardDied(int wizNum);

		void checkForRespawn();
		void respawn();
		void setInvertMousex(int val);
		void setInvertMousey(int val);
		void setDisplayHelp(bool val){displayHelp=val;}
		int getMode(){return mode;}
		void setClearTimer(double clearTimer){this->clearTimer=clearTimer;}


	protected:
		void doClearScreen();

		bool multiplayerGame;
		HelpMenu help;
		bool displayHelp;
		bool invertMousex;
		bool invertMousey;
		int score[2];
		void colCameraToLand();
		float camYoffset;

		float wind;

		float desiredWind;

		float timer;

		double soundTimer;
		SpellManager spellManager;

		CdebugPoint dp[4];

		CLand landscape;
		CSea * sea; 
		ViewFrustum viewFrustum;
		OrbitalCamera * orbitalCam;



		bool debugPhysics;

		GameInfoDisplay infoDisplay;
		Tune themeTune;

		int userWizardNo;

		float wizStartPos[2][3];

		Intro intro;
		int mode;
		int nextMode;

		double respawnTimer;
		bool respawnb;

		double clearTimer;
		bool clearTimerb;
		int oldNumChars;
};

extern mainGM_c mainGM;

extern timer_c timEr;

extern bool showCorners;

extern bool sw_borders;

//DEBUG ls
//extern CdebugPoint mergeDBP[2];







//********************************************************************
//FUNCTIONS

void loadTextures(void);
void loadObjects(void);
void landscaper_initialize(void);
void landscaper_shutdown(void);

