#ifndef FASTMATHS_H
#define FASTMATHS_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define FASTSIN_RES 1024
#define FASTSIN_GRAN 0.006135923152 // 2pi/1024
#define FASTSIN_MAX 7.0
#define FASTSIN_MIN 0
#define ONEOVER1024 0.0009765625
#define FASTSIN_ONEOVERGRAN 162.9746617

extern float fastSinLookup[FASTSIN_RES];
extern float fastCosLookup[FASTSIN_RES];


void initFastMaths();
void initFastSin();
void initFastCos();


inline float fastSin(float val)
{
	int index=(int)val*FASTSIN_ONEOVERGRAN;

	if (index<0) 
	{
		index=-index;
		if (index>=1024) index=index%1024;
		return -fastSinLookup[index];
	}
	if (index>=1024) index=index%1024;
	return fastSinLookup[index];
}

inline float fastCos(float val)
{
	int index=(int)val*FASTSIN_ONEOVERGRAN;

	if (index<0) 
	{
		index=-index;
	}
	
	if (index>1024) index=index%1024;
	return fastCosLookup[index];
}

inline float fastSin_noCheck(float val)
{
	return fastSinLookup[(int)(val*FASTSIN_ONEOVERGRAN)];
}
inline float fastCos_noCheck(float val)
{
	return fastCosLookup[(int)(val*FASTSIN_ONEOVERGRAN)];
}



#endif
