/*
(c)2002 Ben Woodhouse

ben@elf-stone.com
www.elf-stone.com

This code may not be used in a commercial product without my permission.
If you redistribute it, this message must remain intact. 
*/
// FireballSpell.cpp: implementation of the FireballSpell class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "FireballSpell.h"
#include "Wizard.h"
#include "dauntless.h"
#include "LightManager.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

FireballSpell::FireballSpell()
{
}

FireballSpell::FireballSpell(float target[3],float launchpos[3], Wizard * caster, Wizard * enemy)
{
	float lp[3];
	for (int a=0;a<3;++a)
	{
		lp[a]=launchpos[a];
	}

	light=mainGM.lights.createLight(launchpos,1,0.4,0.3); //create an orange light

	fireball=new ParticleFire(lp[0],lp[1],lp[2],550,true);
	phys=mainGM.physics.createObject(lp[0],lp[1],lp[2],1);

	//glPrintf("%f",vect[0]);
	for (a=0;a<3;++a)
	{
		vect[a]=target[a]-lp[a];
	}
	mainGM.sounds[4]->getDuplicate(&whooshSound);
	mainGM.sounds[5]->getDuplicate(&explosionSound);
	whooshSound.play();
//	mainGM.sounds[4]->play();

	normalize(vect);
	exist=true;
	this->enemy=enemy;
	this->caster=caster;
}

FireballSpell::~FireballSpell()
{

	mainGM.lights.destroyLight(light);
	phys.free();
//	explosionSound.stop();
	//explosionSound.release();

	//fireball doesn't need deleting. This is handled by the particlemanager

}
void FireballSpell::doStuff()
{
	
	for (int a=0;a<3;++a)
	{
		pos[a]=phys->realPos[a];

		phys->pos[a]+=vect[a]*speedmod*1.5;
	}
	fireball->setPos(pos);
	light->setPos(pos);

	float * pos=phys->getRealPos();
	float landheight=mainGM.getLand()->getYAt(pos[0],pos[2]);
	if (pos[1]<landheight) //ground collision
	{
		ParticleExplosion * explosion=new ParticleExplosion(pos[0],pos[1],pos[2],2,0.2,250,mainGM.getLand());
		exist=false;
//		ParticleFire * newFire=new ParticleFire(pos[0],landheight,pos[2],90,true);
//		newFire->setAcceleration(0.015);
//		newFire->setExtinguishTime(400);

		//mainGM.sounds[5]->play();
		explosionSound.play();

		fireball->stop();
	}
	else
	{
		if (phys->getCollision())
		{
			ParticleExplosion * explosion=new ParticleExplosion(pos[0],pos[1],pos[2],2,0.2,250,mainGM.getLand());
			exist=false;
			fireball->postMove();
			fireball->stop();
			
			explosionSound.play();
//			mainGM.sounds[5]->play();

			int colID=phys->getCollisionID();
			if (enemy->getPhysID()==colID)
			{
				enemy->hit(0.21,false);
				enemy->catchFire();
				enemy->phys->vel[0]+=vect[0]*0.6;
				enemy->phys->vel[1]+=vect[1]*0.6;
				enemy->phys->vel[2]+=vect[2]*0.6;
			}
		}
		else
		{
			//out of bounds
			if (fabs(phys->pos[0])>400 || fabs(phys->pos[1])>400 || fabs(phys->pos[2])>400)
			{
				exist=false;
				fireball->postMove();
				fireball->stop();
			}
		}
	}

//	glPrintf("hi");fdsfds
	mainGM.getLand()->drawLight(pos[0],pos[1],pos[2],1);
}

void FireballSpell::draw()
{
	//float * pos=phys->getRealPos();
	glPushMatrix();
	glTranslatef(pos[0],pos[1],pos[2]);
	drawSphere(1,10,10);
	glPopMatrix();
}