/*
(c)2002 Ben Woodhouse

ben@elf-stone.com
www.elf-stone.com

This code may not be used in a commercial product without my permission.
If you redistribute it, this message must remain intact. 
*/
// Heightmap.cpp: implementation of the CHeightmap class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Heightmap.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHeightmap::CHeightmap()
{
	height=0;
	width=0;
	scalexz=6.25;
	scaley=0.235294117;
	seaVal=0;
	oneOverScaley=1.0/scaley;

	//size of unit=0.24
	points=NULL; 
	indexStrip=NULL;
	normTable=NULL;
}

 
CHeightmap::~CHeightmap()
{
	if (points!=NULL)
	{ 
		
		for (int y=0;y<height;++y)
		{
			delete [] points[y];
		}
		
		delete [] points;
	}

	if (indexStrip!=NULL)
		delete [] indexStrip;

	if (normTable!=NULL)
		delete [] normTable;

	if (array!=NULL)
		delete array;
}

void CHeightmap::setScale(float scalexz, float scaley)
{
	this->scalexz=scalexz;
	this->scaley=scaley;
	oneOverScaley=1.0/scaley;
}

void CHeightmap::set(int dx, int dy, BYTE heightVal)
{
	height=dy;
	width=dx;
	points=new BYTE * [dy];

	for (int y=0;y<dy;++y)
	{
		points[y]=new BYTE[dx];

		for (int x=0;x<dx;++x)
		{
			points[y][x]=height;
		}
	}

	genArrays(false,true,true);

}



bool CHeightmap::loadFromImage(CImage & image)
{
	if (image.getNumChannels()!=1) return false;

	height=image.getHeight();
	width=image.getWidth();

	points=new BYTE * [height];

	BYTE * data=image.getData();
	
	int dpos=0;
	for (int y=height-1;y>=0;--y)
	{
		points[y]=new BYTE[width];

		for (int x=0;x<width;++x)
		{
			points[y][x]=data[dpos];
			++dpos;
		}
	} 
	genArrays(true,false,false);

	return true;
}

void CHeightmap::drawHM()
{
	array->enable();

	tex->bind();

	for (int a=0;a<height-1;++a)
	{
			glDrawElements(
				GL_TRIANGLE_STRIP,
				indexStrip[a].numIndices,
				GL_UNSIGNED_INT,
				&indexStrip[a].indices[0]
				);
	}

	array->disable();
}

void CHeightmap::loadTexImage(CImage & image)
{
	//messageBox("Logical exmpl"); 
//	image.duplicateChannel(3);
	tex=loadTexFromImage(image,TF_LML,TF_LML);
}

void CHeightmap::genArrays(bool texCoords, bool normals, bool colours)
{
	array=new VertexArray(width*height,texCoords,normals,colours);

	int tci=0;
	int posi=0; 
	int normi=0;
	int coli=0;
	
	float startz=-(height-1)*scalexz*0.5;
	float startx=-(width-1)*scalexz*0.5;
	float xpos;
	float zpos=startz;

	//float norm[1000][1000][3];


	for (int y=0;y<height;++y)
	{
		xpos=startx;
		for (int x=0;x<width;++x)
		{
			array->pos[posi++]=xpos;
			array->pos[posi++]=points[y][x]*scaley;
			array->pos[posi++]=zpos;

			if (texCoords)
			{
				array->texCoord[tci++]=(float)x/64.0;
				array->texCoord[tci++]=(float)-y/64.0;
			}

			if (normals)
			{
				array->normal[normi++]=0;
				array->normal[normi++]=1;
				array->normal[normi++]=0;
			}
			if (colours)
			{
				array->colour[coli++]=1;
				array->colour[coli++]=1;
				array->colour[coli++]=1;
				array->colour[coli++]=0.7;
			}
			xpos+=scalexz;
		}
		zpos+=scalexz;
	}

	indexStrip=new VertexIndices[height-1];

	int indi=0;
	for (y=0;y<height-1;++y)
	{
		
		//pattern
		// 0  2  4
		// |--/---
		// | /| /
        // |/_|/_
		// 1  3  5...
		indexStrip[y].allocate(width*2);
		int arrayPos=y*width;

		indi=0;

		for (int x=0;x<width;++x)
		{	
			
			indexStrip[y].indices[indi]=arrayPos;
			++indi;
			indexStrip[y].indices[indi]=arrayPos+width;
			++indi;
			++arrayPos;
			
		}
		
	}
	

	
/*
	glVertexPointer(3, GL_FLOAT, 0, numArrayVers, &vertexArray);         // filling the arrays    
	glTexCoordPointer(2, GL_FLOAT, 0, numArrayVers, &texCoordArray);

	
	glLockArraysEXT(0, count);                // extension GL_EXT_compiled_vertex_array

	do rendering ... using glArrayElement, glDrawElemets or glDrawArrays

	glUnlockArraysEXT;                       // extension GL_EXT_compiled_vertex_array
*/
}

void CHeightmap::generateNormTable()
{
	normTable=new float[128*128*3];
	float pointa[3]={0,0,0};
	float pointb[3];
	float pointc[3];

	tableUnit=0.05;
	pointb[0]=0;
	pointb[2]=scalexz;

	pointc[0]=scalexz;
	pointc[2]=0;
	int np=0;


	for (int a=0;a<128;++a)
	{
		for (int b=0;b<128;++b)
		{
			//generate the triangle
			pointb[1]=(float)(a-64)*tableUnit;
			pointc[1]=(float)(b-64)*tableUnit;

			getTriNormal(&normTable[np],pointa,pointb,pointc);
			np+=3;
		}

	}
}

void CHeightmap::setTex(CTexture * tex)
{
	this->tex=tex;
}


float CHeightmap::getYAt(float x_in, float z_in)
{
	float xpos=(x_in/scalexz)+(width-1)/2;
	float zpos=(z_in/scalexz)+(width-1)/2;
	//int xPos=(x_in/scalexz)+(width-1)/2;
	//int zPos=(z_in/scalexz)+(height-1)/2;

	if (xpos>(float)width-1.01) xpos=(float)width-1.01;
	if (xpos<0) xpos=0;
	if (zpos>(float)height-1.01) zpos=(float)height-1.01;
	if (zpos<0) zpos=0;

	float cornerPos[4][2];
	float cornerHeights[4];
	float cornerDist[4];

	cornerPos[0][0]=(int)xpos;
	cornerPos[0][1]=(int)zpos;
	cornerPos[1][0]=(int)xpos+1;
	cornerPos[1][1]=(int)zpos; 
	cornerPos[2][0]=(int)xpos;
	cornerPos[2][1]=(int)zpos+1;
	cornerPos[3][0]=(int)xpos+1;
	cornerPos[3][1]=(int)zpos+1;

	for (int a=0;a<4;++a)
	{
		//cornerHeights[a]=points[ cornerPos[a][1] ][ cornerPos[a][0] ]*scaley;
		int offset=cornerPos[a][1]*width+cornerPos[a][0];
		cornerHeights[a]=array->pos[ 1+offset*3 ];
	}

	float dx,dz;
	for (a=0;a<4;++a)
	{
		dx=xpos-(float)cornerPos[a][0];
		dz=zpos-(float)cornerPos[a][1];

		cornerDist[a]=sqrt(dx*dx + dz*dz);
	}

	float totalHeight=0;
	float cornerWeight[4];
	float totalWeight=0;
	for (a=0;a<4;++a)
	{
		if (cornerDist[a]<=1)
		{
			totalHeight+=cornerHeights[a];
			cornerWeight[a]=1.0-cornerDist[a];
			totalWeight+=cornerWeight[a];
		}
	}

	//get the weighted average
	float weight;
	float heightOut=0;
	float oneOverTotalWeight=1.0/totalWeight;
	for (a=0;a<4;++a)
	{
		if (cornerDist[a]<=1)
		{
			weight=cornerWeight[a]*oneOverTotalWeight;
			heightOut+=weight*cornerHeights[a];
		}
	}
	return heightOut;
}



float CHeightmap::getYAt_simple(float x_in, float z_in, HMYOutParamStruct &out)
{
	float xpos=(x_in/scalexz)+(width-1)/2;
	float zpos=(z_in/scalexz)+(width-1)/2;

	if (xpos>(float)width-1.01) xpos=(float)width-1.01;
	if (xpos<0) xpos=0;
	if (zpos>(float)height-1.01) zpos=(float)height-1.01;
	if (zpos<0) zpos=0;

	//int cornerPos[4][2];

	out.cornerPos[0][0]=(int)xpos;
	out.cornerPos[0][1]=(int)zpos;
	out.cornerPos[1][0]=(int)xpos+1;
	out.cornerPos[1][1]=(int)zpos; 
	out.cornerPos[2][0]=(int)xpos;
	out.cornerPos[2][1]=(int)zpos+1;
	out.cornerPos[3][0]=(int)xpos+1;
	out.cornerPos[3][1]=(int)zpos+1;

	float height,heightOut=-1000;
	for (int a=0;a<4;++a)
	{
		int offset=out.cornerPos[a][1]*width+out.cornerPos[a][0];
		height=out.cornerHeight[a]=array->pos[ 1+offset*3 ];
		if (height>heightOut)
		{
			heightOut=height;
		}
	}
	out.pos[0]=xpos;
	out.pos[1]=zpos;

	return heightOut;
}

float CHeightmap::getYAt_2ndPart(HMYOutParamStruct &in)
{
	float xpos=in.pos[0];
	float zpos=in.pos[1];

	float dx,dz;
	float cornerDist[4];

	for (int a=0;a<4;++a)
	{
		dx=xpos-(float)in.cornerPos[a][0];
		dz=zpos-(float)in.cornerPos[a][1];

		cornerDist[a]=sqrt(dx*dx + dz*dz);
	}

	float totalHeight=0;
	float cornerWeight[4];
	float totalWeight=0;
	for (a=0;a<4;++a)
	{
		if (cornerDist[a]<=1)
		{
			totalHeight+=in.cornerHeight[a];
			cornerWeight[a]=1.0-cornerDist[a];
			totalWeight+=cornerWeight[a];
		}
	}

	//get the weighted average
	float weight;
	float heightOut=0;
	float oneOverTotalWeight=1.0/totalWeight;
	for (a=0;a<4;++a)
	{
		if (cornerDist[a]<=1)
		{
			weight=cornerWeight[a]*oneOverTotalWeight;
			heightOut+=weight*in.cornerHeight[a];
		}
	}
	return heightOut;
}