/*
__________________________________________________________________________________________

  Name			:	Image.h
  Version		:	1.01
  Description	:	Image storage and management. Uses intel JPEG library to load jpeg 
					files. Supports embedded files (except JPEG currently). 

					Several operations included for combining and multiplying images. 
					
					This module is required by Texture v2.0

  Last Update	:	20/03/02

  (c)2002 Ben Woodhouse
__________________________________________________________________________________________

*/
 
/*
#ifndef __gl_h_

#define BYTE  unsigned char
typedef unsigned int GLenum;
typedef unsigned char GLboolean;
typedef unsigned int GLbitfield;
typedef signed char GLbyte;
typedef short GLshort;
typedef int GLint;
typedef int GLsizei;
typedef unsigned char GLubyte;
typedef unsigned short GLushort;
typedef unsigned int GLuint;
typedef float GLfloat;
typedef float GLclampf;
typedef double GLdouble;
typedef double GLclampd;
typedef void GLvoid;

 PixelFormat 
#define GL_RED                            0x1903
#define GL_GREEN                          0x1904
#define GL_BLUE                           0x1905
#define GL_ALPHA                          0x1906
#define GL_RGB                            0x1907
#define GL_RGBA                           0x1908

#endif 
*/
#if !defined(AFX_IMAGE_H__CD05ED8B_98F5_4D04_93FC_6AACAECD24CB__INCLUDED_)
#define AFX_IMAGE_H__CD05ED8B_98F5_4D04_93FC_6AACAECD24CB__INCLUDED_
#include "Image_BMPHeader.h"




#define CHANNEL_RED 1
#define CHANNEL_GREEN 2
#define CHANNEL_BLUE 4
#define CHANNEL_ALPHA 8
#define CHANNEL_ALL 15

class SaveImage;

class CImage  
{
	friend CImage;
	friend SaveImage;
public:
	CImage();
    CImage(const CImage &src); //copy constructor (required by STL)
	const CImage& operator= (const CImage &src); //(required by STL)
	
	//raw files must have equal width and height and either 1 or 3 channels
	bool loadRAW(const char * fn);
	bool loadRAWEmbedded(FILE * filePtr);

	bool loadBMP(const char * fn);
	bool loadBMPEmbedded(FILE * filePtr);

	bool loadJPG(const char * fn);
	bool loadJPGEmbedded(FILE * filePtr); 

	bool resample(int newWidth, int newHeight);
	bool resize(int newWidth, int newHeight);


	//bool loadJPG(const char * fn);
	bool duplicateChannel(int numChannels_in); //duplicate a single channel to other
											   //channels



	BYTE * getData();
	int getNumChannels();
	int getMemSize(){return memSize;}
	int getWidth();
	int getHeight();

	//multiplies the specified channels by a single value
	void multiply(int channelFlags,float amount);

	//this can only be done to 3 channel (RGB) images
	//the alphaImage must have only one channel
	bool addAlphaChannel(CImage & alphaImage);
	
	bool setFormat(unsigned int format_in);

	void allocate(); //allocates memory for the set image size
	void allocate(int width_in, int height_in, int channels_in);

	bool NM_normalize(); //treats each pixel as a vector and normalises accordingly 
						 //making each length 127

	bool convertToNormalMap(bool wrap, float scale=2.55); //converts the image to a normal map

	unsigned int getFormat();
	virtual void copyFrom(const CImage & src);
	

	//BLENDS
	//the both images must have 3 channels
	void regionPlusImg(int originx,int originy,CImage &imageb); 

	//the dest image must have 4 channels. The src (this) image must have 3
	//the dest image must be in ARGB format
	void regionAlphaBlendImg(int originx,int originy,CImage &imageb); 


	//COPY REGION : 
	//					NOTE: REGION WIDTHS MUST BE MULTIPLES OF 4
	//both images must be identical
	void copyRegionFrom(CImage & src,
						int originx, 
						int originy, 
						int rwidth, 
						int rheight);

	//dest image must be at least as big as the region to copy
	void copyRegionFromNI(	CImage & src,
							int srcX,
							int srcY,
							int dstX,
							int dstY,
							int rwidth,
							int rheight);

	void removeLSB();

	void adjustBrightness(float brightFactor);
	void adjustChannelsBrightness(int channels, float brightFactor);

	bool RGBAToARGB();

	~CImage();

protected:
	void setLikelyFormat(); 
	bool isPowerOf2(int num);
	BYTE getPixelAt_nearest(int channel, float x, float y);
	BYTE getPixelAt_linear(int channel, float x, float y);

	BYTE * data;
	int numChannels;
	int width;
	int height;
	int memSize;
	unsigned int format;
};

#endif // !defined(AFX_IMAGE_H__CD05ED8B_98F5_4D04_93FC_6AACAECD24CB__INCLUDED_)
