/*

Mesh.cpp

Version 2.0g

Game version.


(c)2002 Ben Woodhouse

ben@elf-stone.com
www.elf-stone.com

This code may not be used in a commercial product without my permission.
If you redistribute it, this message must remain intact. 

*/

#include "stdafx.h"
//#include "mesh.h"


 



//---------------------------------------------------------------------------
//Mesh Class
//---------------------------------------------------------------------------

bool Mesh::writeToStream(FILE * str)
{
  //write the static mesh data
    _putw(strlen(name),str);
    fputs(name,str);
    _putw(numVers,str);
    _putw(numSubs,str);

    fwrite(color,sizeof(float),3,str);
    fwrite(&doubleSided,sizeof(bool),1,str);

    //write the vertices
    if (numVers>0) fwrite(ver,sizeof(CVertex),numVers,str);

    //write the sub meshes
    for (int a=0;a<numSubs;++a)
    {
        _putw(sub[a].numTris,str);
        _putw(sub[a].texNum,str);
        fwrite(sub[a].tri,sizeof(CTriangle),sub[a].numTris,str);
    }
	 return true;

}

bool Mesh::readFromStream(FILE * str)
{

//	messageBox("static");
    int len=_getw(str);
    fgets(name,len+1,str);

    numVers=_getw(str);
    numSubs=_getw(str);

    fread(color,sizeof(float),3,str);
    fread(&doubleSided,sizeof(bool),1,str);

    //allocate the vertices and submeshes
    if (numVers>0) ver=new CVertex[numVers];
	 else ver=NULL;

    sub=new CSubMesh[numSubs];

    //read the vertices
    if (numVers>0) fread(ver,sizeof(CVertex),numVers,str);

    //read the sub meshes
    for (int a=0;a<numSubs;++a)
    {
        sub[a].numTris=_getw(str);
        sub[a].texNum=_getw(str);
        sub[a].tri=new CTriangle[sub[a].numTris];
        fread(sub[a].tri,sizeof(CTriangle),sub[a].numTris,str);
    }
	 return true;
}




//only specify altframever for animation frames
void Mesh::draw()
{

	float black[4]={0,0,0,0};
	float white[4]={1,1,1,1};
	float col[4]={0,0,0,1};
	CSubMesh * subp;

	CVertex * v;

	//specify which vertex set to use
	v=ver;


	int a,s;

	if (doubleSided)
	{
		glDisable(GL_CULL_FACE);
		glLightModelf(GL_LIGHT_MODEL_TWO_SIDE,1.0);
	}

	for (s=0;s<numSubs;++s)
	{
		subp=&sub[s];
		if (subp->texNum!=-1) 
		{
			
			glColor4f(1,1,1,1);
			//glCls();
			//glPrintf("sub: %d",subp->texNum);
		 	parentB3D->getTex(subp->texNum)->bind();
		}
				

		else
		{
			glPushAttrib(GL_LIGHTING_BIT);
			glPushAttrib(GL_ENABLE_BIT);
			glDisable(GL_TEXTURE_2D);
			setTextureLevel(0);
		//	glColor4f(1,1,1,1);

			glEnable(GL_COLOR_MATERIAL);

			if (doubleSided) 
			{
				glColorMaterial(GL_FRONT,GL_DIFFUSE);

				for (int a=0;a<3;++a)
				{
					col[a]=color[a]*0.8;
				}
				glMaterialfv(GL_BACK,GL_DIFFUSE,col);
				glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,black);
				glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,black);

			}

			else
			{
				glColorMaterial(GL_FRONT,GL_DIFFUSE);

				glMaterialfv(GL_FRONT,GL_AMBIENT,black);
				glMaterialfv(GL_FRONT,GL_SPECULAR,black);
			}
		}
		
		glBegin(GL_TRIANGLES);
			for (a=0;a<subp->numTris;++a)
			{
				for (int b=0;b<3;++b)
				{
					if (subp->texNum==-1)
						glColor3fv(color);
					glNormal3fv(	subp->tri[a].verNorm[b]   );

					glTexCoord2f(	subp->tri[a].texCoord[b][0], 
										subp->tri[a].texCoord[b][1]  );

					glVertex3fv(	v[ subp->tri[a].ver[b] ].pos  );
				}
			}
		glEnd();

		if (subp->texNum==-1) 
		{
			//glMaterialfv(GL_FRONT,GL_DIFFUSE,white);
			glPopAttrib();
			glPopAttrib();
			//glDisable(GL_COLOR_MATERIAL);
		}
		
	}//subMesh loop
	if (doubleSided)
	{
		glEnable(GL_CULL_FACE);
		glLightModelf(GL_LIGHT_MODEL_TWO_SIDE,0.0);
	}
	
}

//retreives a list of simple triangle defs from the submeshes
SimpleTri * Mesh::getSimpleTris()
{
	//count number of triangles
	int numTris=getNumTris();

	//allocate
	SimpleTri * tri=new SimpleTri[numTris];

	int stNum=0;
	//retreive the triangles from the submeshes
	for (int a=0;a<numSubs;++a)
	{
		for (int t=0;t<sub[a].numTris;++t)
		{
			for (int c=0;c<3;++c)
			{
				tri[stNum].ver[c]=sub[a].tri[t].ver[c];
			}
			++stNum;
		}
	}
	return tri;
}

int Mesh::getNumTris()
{
	int total=0;
	for (int a=0;a<numSubs;++a)
	{
		total+=sub[a].numTris;
	}
	return total;
}

void Mesh::setLowestColourComponent(float amt)
{
	for (int a=0;a<3;++a)
	{
		if (color[a]<amt) color[a]=amt;
	}
}

//---------------------------------------------------------------------------


