/*
(c)2002 Ben Woodhouse

ben@elf-stone.com
www.elf-stone.com

This code may not be used in a commercial product without my permission.
If you redistribute it, this message must remain intact. 
*/
// Music.cpp: implementation of the CMusic class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Music.h"
#include "Tune.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Music music;


////////////////////////////////

Music::Music()
{
	pPerf=NULL;
	pMusicLoader=NULL;
	numSegments=0;
	volume=1.0;
}

Music::~Music()
{
}

//bool Music::init()
bool Music::init(LPDIRECTSOUND *pDSound)
{


	CoCreateInstance(	CLSID_DirectMusicLoader, NULL, 
					   CLSCTX_INPROC, IID_IDirectMusicLoader8,
					   (void**)&pMusicLoader);

/*	CoCreateInstance(	CLSID_DirectMusicLoader, NULL, 
					   CLSCTX_INPROC, IID_IDirectMusicLoader8,
					   (void**)&pSoundLoader);*/

	CoCreateInstance(	CLSID_DirectMusicPerformance, NULL,
					   CLSCTX_INPROC, IID_IDirectMusicPerformance8,
					   (void**)&pPerf);
					   

	pPerf->InitAudio( 
						NULL,      // IDirectMusic interface not needed.
						pDSound,  //IDirectSound interface
						NULL,      // Window handle.
						DMUS_APATH_SHARED_STEREOPLUSREVERB,  // Default audiopath type.
						64,        // Number of performance channels.
						DMUS_AUDIOF_ALL,   // Features on synthesizer.
						NULL       // Audio parameters; use defaults.
					  );

	return true;

}

void Music::uninit()
{
	pPerf->Stop(
						NULL,
						NULL,
						0,  
						0);

	pPerf->CloseDown();
	pMusicLoader->Release(); 
	pPerf->Release();
	releaseSegments();
}

IDirectMusicSegment8 *Music::loadNewMusic(const char * fn)
{
	WCHAR wfn[255];
	strToWstr(fn,wfn);

	IDirectMusicSegment8 *seg;

	if (FAILED(pMusicLoader->LoadObjectFromFile(
											CLSID_DirectMusicSegment, 
											IID_IDirectMusicSegment8, 
											wfn,     
											(LPVOID*) &seg
											)))
	{
		return NULL;
	}

	addNewSegment(seg);

	seg->Download(pPerf);

	return seg;
}

IDirectMusicSegment8 *Music::loadNewSound(const char * fn)
{
	WCHAR wfn[255];
	strToWstr(fn,wfn);
	IDirectMusicSegment8 * seg;


	if (FAILED(pMusicLoader->LoadObjectFromFile(
											CLSID_DirectMusicSegment, 
											IID_IDirectMusicSegment8, 
											wfn,     
											(LPVOID*) &seg
											)))
	{
		messageBox("FAILED");
		return NULL;
	}

	addNewSegment(seg);
	seg->Download(pPerf);

	return seg;
}

void Music::addNewSegment(IDirectMusicSegment8 * seg)
{
	pSegments.push_back(seg);
	++numSegments;
}

void Music::releaseSegments()
{
	for (int a=0;a<numSegments;++a)
	{
		pSegments[a]->Release();
	}
}



void Music::setVolume(float vol)
{
	volume=vol;
	IDirectMusicAudioPath* pAudioPath;

	long val=(1.0-vol)*-5000;


	if (!FAILED(pPerf->GetDefaultAudioPath(&pAudioPath)))
	{
		int er;
		if (FAILED(er=pAudioPath->SetVolume(val,0)))
		{
			if (er==DMUS_E_NOT_INIT)
			{
				messageBox("DMUS_E_NOT_INIT");
			}
		}
	}
	else
	{
		messageBox("failed");
	}
}
