/*
(c)2002 Ben Woodhouse

ben@elf-stone.com
www.elf-stone.com

This code may not be used in a commercial product without my permission.
If you redistribute it, this message must remain intact. 
*/
// Particle1.cpp: implementation of the CParticle class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Particle1.h"
#include "ParticleTypes.h"
#include "FastMaths.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


Particle::Particle()
{
	rotate=false;
	rotateAngle=0;
	rotateRadius=0;
	rotateRadiusGrow=0;
	rotateCenter[0]=0;
	rotateCenter[1]=0;
	for (int a=0;a<3;++a)
	{
		acc[a]=0;
		vel[a]=0;
		pos[a]=0;
		colour[a]=0;
	}
	gravity=0;
	initLife=1;
	life=1;
	decay=0;
	doubleDraw=0;
	grow=0;
	drawb=true;
//	initLife=0;
}

Particle::Particle(	float posx, 
					float posy,
					float posz)
{
	pos[0]=posx;
	pos[1]=posy;
	pos[2]=posz;

	rotate=false;
	rotateAngle=0;
	rotateRadius=0;
	rotateRadiusGrow=0;
	rotateCenter[0]=0;
	rotateCenter[1]=0;
	for (int a=0;a<3;++a)
	{
		acc[a]=0;
		vel[a]=0;
		colour[a]=0;
	}
	gravity=0;
	initLife=1;
	life=1;
	decay=0;
	doubleDraw=0;
	grow=1;
	drawb=true;
}


Particle::~Particle()
{
}

Particle::Particle(const Particle & src)
{
	copyFrom (src);
}
const Particle& Particle::operator=(const Particle& src)
{
	copyFrom(src);
	return *this;
}

void Particle::copyFrom(const Particle &src)
{
	//memcpy((void *)this,&src,sizeof(Particle));
	

	gravity=src.gravity;
	drawb=src.drawb;
	doubleDraw=src.doubleDraw;
	accelerateB=src.accelerateB;	
	for (int a=0;a<3;++a)
	{
		vel[a]=src.vel[a];
		pos[a]=src.pos[a];
		acc[a]=src.acc[a];
		colour[a]=src.colour[a];
	}
	colour[3]=src.colour[3];
	life=src.life;
	initLife=src.initLife;
	decay=src.decay;
	mass=src.mass;
	rotate=src.rotate;
	rotateAngle=src.rotateAngle;
	rotateRadius=src.rotateRadius;
	rotateSpeed=src.rotateSpeed;
	friction=src.friction;
	grow=src.grow;
	rotateRadiusGrow=src.rotateRadiusGrow;
	rotateCenter[0]=src.rotateCenter[0];
	rotateCenter[1]=src.rotateCenter[1];
	radius=src.radius;
	distFromCam=src.distFromCam;
	blendMode=src.blendMode;
}

void Particle::move(float deltaT)
{
	
	if (grow>0)
	{
		//float growAmount=(radius*grow)-radius;
		//radius+=growAmount*deltaT;
		radius+=grow*deltaT;
	}

	int b;

	float meanVel[3];

	meanVel[0]=vel[0];
	meanVel[1]=vel[1];
	meanVel[2]=vel[2];


	bool velChange=false;
	if (accelerateB)
	{
		vel[0]+=deltaT*acc[0];
		vel[1]+=deltaT*acc[1];
		vel[2]+=deltaT*acc[2];

		
		float accDecay=pow(0.999,deltaT);
		acc[0]*=accDecay;
		acc[1]*=accDecay;
		acc[2]*=accDecay;
		
		velChange=true;
	}

	if (gravity)
	{
		vel[1]-=0.01*radius*deltaT;
		velChange=true;
	}

	if (rotate)
	{
		rotateRadius+=(rotateRadiusGrow*deltaT);
		rotateAngle+=deltaT*rotateSpeed;
		rotateAngle=getFixAngle(rotateAngle);

		pos[0]=rotateCenter[0]+(sin(rotateAngle*dera)*rotateRadius);
		pos[2]=rotateCenter[1]+(cos(rotateAngle*dera)*rotateRadius);
		
//		pos[0]=rotateCenter[0]+(fastSin(rotateAngle*dera)*rotateRadius);
//		pos[2]=rotateCenter[1]+(fastCos(rotateAngle*dera)*rotateRadius);
	}

	if (friction!=0)
	{
		float frAmount=pow(friction,deltaT);

		for (b=0;b<3;++b)
		{
			vel[b]*=frAmount;
		}
		velChange=true;
	}

	if (velChange)
	{
		meanVel[0]=(meanVel[0]+vel[0])*0.5;
		meanVel[1]=(meanVel[1]+vel[1])*0.5;
		meanVel[2]=(meanVel[2]+vel[2])*0.5;
	}

	for (b=0;b<3;++b)
	{
		pos[b]+=meanVel[b]*deltaT;
	}
	if (decay>0)
	{
		life-=deltaT*decay;
	}
}

void Particle::draw()
{
	float bright;
	int a,countto,b;

	float camd[3]={0,0,0};

 	bright=(float)life;

	if (doubleDraw)
	{
		bright*=0.4;
		countto=2;
	}
	else countto=1;

	for (a=0;a<countto;++a)
	{
		glColor4f(colour[0]*bright,colour[1]*bright,colour[2]*bright,colour[3]*bright);

		glPushMatrix();
			if (a==1)
			{
				//get the normal vector from the particle to the camera
				float cPos[3];
				currentVP->getCameraPos(cPos);
				for (b=0;b<3;++b)
				{
					camd[b]=cPos[b]-pos[b];
				}
				normalize(camd);

				//multiply by the radius to get the displacement
				for (b=0;b<3;++b)
				{
					camd[b]*=radius;
				}


								

				if (doubleDraw) bright*=0.5;

			}


			glTranslatef(	pos[0]+camd[0],	
							pos[1]+camd[1],
							pos[2]+camd[2]);

			currentVP->rotateByInvRotMat();

			glBegin(GL_QUADS);

				glTexCoord2f(0, 0);
				glVertex2f(-radius, -radius);

				glTexCoord2f(1, 0);
				glVertex2f(radius, -radius);

				glTexCoord2f(1, 1);
				glVertex2f(radius, radius);

				glTexCoord2f(0, 1);
				glVertex2f(-radius, radius);
			glEnd();
	
		glPopMatrix();
	}
}
