/*
(c)2002 Ben Woodhouse

ben@elf-stone.com
www.elf-stone.com

This code may not be used in a commercial product without my permission.
If you redistribute it, this message must remain intact. 
*/
// ParticleExplosion.cpp: implementation of the ParticleExplosion class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ParticleExplosion.h"
#include "ParticleTypes.h"
#include "ParticleManager.h"
#include "dauntless.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ParticleExplosion::ParticleExplosion(	float x, 
					float y, 
					float z, 
					float psize,
					float eforce, 
					int numParts, 
					CLand * landscape)
{
	PT_Sparks ty(eforce,true);  
	Particle part;

	allocate(numParts+2);

	origin[0]=x;
	origin[1]=y;
	origin[2]=z;

	for (int a=0;a<numParts;++a)
	{
		ty.generate(&part);
		part.pos[0]=x;
		part.pos[1]=y;
		part.pos[2]=z;
		createParticle(part,0);
	}

	this->landscape=landscape;
	gParticles.registerSystem(this);
	if (!dark) light=mainGM.lights.createLight(origin,1,0.4,0.3); //create an orange light
}

ParticleExplosion::ParticleExplosion(	float x, 
					float y, 
					float z,  
					float psize,
					float eforce, 
					int numParts, 
					CLand * landscape,
					float colour[4],
					bool dark)
{
	PT_Sparks ty(eforce,true);  
	Particle part;

	origin[0]=x;
	origin[1]=y;
	origin[2]=z;

	for (int a=0;a<numParts;++a)
	{
		ty.generate(&part);
		for (int b=0;b<4;++b)
		{
			part.colour[b]=colour[b];
		}
		part.pos[0]=x;
		part.pos[1]=y;
		part.pos[2]=z;

		if (dark) part.blendMode=PBLEND_DARK;
		createParticle(part,0);
	}

	this->landscape=landscape;
	gParticles.registerSystem(this);
	
	if (!dark) light=mainGM.lights.createLight(origin,1,0.4,0.3); //create an orange light
	this->dark=dark;
}

ParticleExplosion::~ParticleExplosion()
{
//	glPrintf("explosionDeleted");
	if (!dark) mainGM.lights.destroyLight(light); 
}

void ParticleExplosion::postMove()
{
	float y;
	float * pos;
	float rad;
	float partLow;

//	for (it=particles.begin();it!=particles.end();++it)
	for (int a=0;a<numParticles;++a)
	{
		pos=particles[a].pos;
		rad=particles[a].radius;
		HMYOutParamStruct ps;

		partLow=pos[1]-rad;
		
		//do basic landscape collision check
		if (partLow<landscape->getYAt_simple(pos[0],pos[2],ps)) 
		{
			//do more accurate check
			y=landscape->getYAt_2ndPart(ps);
			if (pos[1]-rad<y)
			{
				pos[1]=y+rad;

//				particles[a].friction=0.8;
				particles[a].vel[0]*=0.9;
				particles[a].vel[2]*=0.9;

				particles[a].acc[0]*=0.9;
				particles[a].acc[2]*=0.9;
			}
		}
	}

	//if (numParts>60)
	float bright=(numParticles-200)*0.02;
	if (bright>1) bright=1;
	else if (bright<0) bright=0;
	if (!dark) 
	{
		landscape->drawLight(origin[0],origin[1],origin[2],bright);
		light->setColour(1*bright,0.4*bright,0.3*bright);
	}
	//pos[1]=y;
	//Printf("pos1=%f  y=%f ",pos[1],y);
}


void ParticleExplosion::preDraw()
{
}
