/*
(c)2002 Ben Woodhouse

ben@elf-stone.com
www.elf-stone.com

This code may not be used in a commercial product without my permission.
If you redistribute it, this message must remain intact. 
*/
// ParticleFire.cpp: implementation of the ParticleFire class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ParticleFire.h"
#include "ParticleManager.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ParticleFire::ParticleFire(float x,float y,float z,float partsPerSec,bool rising)
{

	allocate(partsPerSec+2);

	oldPos[0]=x;
	oldPos[1]=y;
	oldPos[2]=z;
	pos[0]=x;
	pos[1]=y;
	pos[2]=z;
	active=true;
	timeOffset=0;
	this->partsPerSec=partsPerSec;
	gParticles.registerSystem(this);
	this->rising=rising;
	selfExtinguish=false;
	acc=-1;
}

void ParticleFire::setExtinguishTime(double time)
{
	extinguishTimer=time;
	selfExtinguish=true;
}


ParticleFire::~ParticleFire()
{

}

void ParticleFire::postMove()
{

	//float numParts
	if (active)
	{
		PT_Fire ty;  
		Particle part;

		double smSec=speedmod/100.0;
		timeOffset=smSec+timeOffset;
		//int launchCount=timeOffset*partsPerSec;
		double launchInterval=1.0/partsPerSec;
		int launchCount=timeOffset/launchInterval;

		double currentTime=0;


		//start from oldpos and work forwards in time
		for (int a=0;a<launchCount;++a)
		{
			ty.generate(&part);
			part.pos[0]=oldPos[0]+(pos[0]-oldPos[0])*(currentTime/timeOffset);
			part.pos[1]=oldPos[1]+(pos[1]-oldPos[1])*(currentTime/timeOffset);
			part.pos[2]=oldPos[2]+(pos[2]-oldPos[2])*(currentTime/timeOffset);

			if (rising)
			{
				if (acc!=-1)
				{
					part.acc[1]=acc/part.radius;
				}
				part.move((timeOffset-currentTime)*100.0);
			}
			else
			{
				part.vel[1]=0;
				part.acc[1]=0;
			}

			createParticle(part,0);//currentTime*100.0);

			currentTime+=launchInterval;
		}

		timeOffset-=currentTime;

		oldPos[0]=pos[0];
		oldPos[1]=pos[1];
		oldPos[2]=pos[2];

		if (selfExtinguish)
		{
			extinguishTimer-=speedmod;
			if (extinguishTimer<0)
			{
				active=false;
			}
		}

	}

}

void ParticleFire::preDraw()
{
}

void ParticleFire::stop()
{
	active=false;
}

bool ParticleFire::isDead()
{
	if (!active && numParticles==0)
	{
		return true;
	}
	return false;
}
