/*
(c)2002 Ben Woodhouse

ben@elf-stone.com
www.elf-stone.com

This code may not be used in a commercial product without my permission.
If you redistribute it, this message must remain intact. 
*/
// PhysicsObject.cpp: implementation of the PhysicsObject class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PhysicsObject.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

PhysicsObject::PhysicsObject()
{
	for (int a=0;a<3;++a)
	{
		pos[a]=0;
		realPos[a]=0;
		vel[a]=0;
		oldPos[a]=0;
		oldvel[a]=0;
	}
	enabled=true;

	ID=0;
	index=0;
	collidedID=-1;
	collision=false;
	child=NULL;
	numChildren=0;
	parent=NULL;

	friction=0.98;

}

void PhysicsObject::reset()
{
	//delete any children 
	if (child!=NULL)
	{
		delete [] child;
		child=NULL;
	}

	for (int a=0;a<3;++a)
	{
		pos[a]=0;
		realPos[a]=0;
		vel[a]=0;
		oldPos[a]=0;
	}
	numChildren=0;
	collidedID=-1;
	collision=false;
	friction=0.98;

	//	enabled=true;

//	ID=0;
//	index=0;
	//child=NULL;
	numChildren=0;
	parent=NULL;

}

PhysicsObject::PhysicsObject(const PhysicsObject &src)
{
	copyFrom(src); 
}//copy constructor (required by STL)


const PhysicsObject& PhysicsObject::operator= (const PhysicsObject& src)
{
	copyFrom(src);
	return *this;
}//(required by STL)

PhysicsObject::~PhysicsObject()
{
	if (child!=NULL) delete [] child;
}


void PhysicsObject::copyFrom(const PhysicsObject &src)
{
	index=src.index;
	ID=src.ID;
	collidedID=src.collidedID;
	collision=src.collision;
	friction=src.friction;

	parent=src.parent;
	for (int a=0;a<3;++a)
	{
		pos[a]=src.pos[a];
		realPos[a]=src.realPos[a];
		vect[a]=src.vect[a];
		oldPos[a]=src.oldPos[a];
		oldvel[a]=src.oldvel[a];
		vel[a]=src.vel[a];
	} 
	radius=src.radius;
	enabled=src.enabled;
	moving=src.moving;

	if (numChildren>0) 
	{
		delete [] child;
		child=NULL;
		numChildren=0;
	}
	if (src.numChildren>0)
	{
		//copy the children
		createChildren(src.numChildren);	
		for (a=0;a<src.numChildren;++a)
		{
			child[a].copyFrom(src.child[a]);
		}
	}
	//infsphere and cr not copied (they are created during physics loop)
}

void PhysicsObject::setRealPos(const float realPos_in[3]) //move the object without collisions
{
	for (int a=0;a<3;++a)
	{
		oldPos[a]=realPos_in[a];
		realPos[a]=realPos_in[a];
		pos[a]=realPos_in[a];
	}
}


void PhysicsObject::moveToRealPos()
{
	for (int a=0;a<3;++a)
	{
		pos[a]=realPos[a];
		vect[a]=0;
	}
}


void PhysicsObject::calcVect()
{
	if (enabled)
	{
		for (int a=0;a<3;++a)
		{
			vect[a]=pos[a]-realPos[a];
		}

	}
	if (vect[0]==0 && vect[1]==0 && vect[2]==0)
	{
		moving=false;

	}
	else 
	{
		moving=true;
	}

	for (int a=0;a<numChildren;++a)
	{
		child[a].calcVect();
	}

}

void PhysicsObject::calcSphereOfInfluence()
{
	if (moving)
	{
		infSphere.pos[0]=realPos[0]+vect[0]*0.5;
		infSphere.pos[1]=realPos[1]+vect[1]*0.5;
		infSphere.pos[2]=realPos[2]+vect[2]*0.5;

		infSphere.radius=lengthOf(vect)*0.5+radius;
	}
	else
	{
		infSphere.pos[0]=realPos[0];
		infSphere.pos[1]=realPos[1];
		infSphere.pos[2]=realPos[2];
		infSphere.radius=radius;
	}
}


void PhysicsObject::createChildren(int num)
{
	child=new PhysicsObject[num];
	numChildren=num;

	for (int a=0;a<num;++a)
	{
		child[a].parent=this;
	}
}

void PhysicsObject::getDimensionsFromChildren()
{
	if (numChildren==0) return;
	//get the axis-aligned bounding sphere
	float hiz=-1000000000;
	float hix=-1000000000;
	float hiy=-1000000000;
	float lox=1000000000;
	float loy=1000000000;
	float loz=1000000000;


	
	for (int a=0;a<numChildren;++a)
	{
		float * opos=child[a].realPos;
		float rad=child[a].radius;

		if (opos[0]+rad>hix) hix=opos[0]+rad;
		if (opos[0]-rad<lox) lox=opos[0]-rad;
		if (opos[1]+rad>hiy) hiy=opos[1]+rad;
		if (opos[1]-rad<loy) loy=opos[1]-rad;
		if (opos[2]+rad>hiz) hiz=opos[2]+rad;
		if (opos[2]-rad<loz) loz=opos[2]-rad;
	}

	//the center of the bounding sphere is the aabox center
	pos[0]=realPos[0]=(hix+lox)*0.5;
	pos[1]=realPos[1]=(hiy+loy)*0.5;
	pos[2]=realPos[2]=(hiz+loz)*0.5;

	float furthestDist=0;
	
	//get the distance from the center to the furthest sphere (including its radius)
	for (a=0;a<numChildren;++a)
	{
		float * opos=child[a].realPos;
		float v[3];
		for (int b=0;b<3;++b)
		{
			v[b]=opos[b]-pos[b];
		}
		float dist=lengthOf(v)+child[a].radius;
		if (dist>furthestDist)
		{
			furthestDist=dist;
		}
	}
	radius=furthestDist;
	//makeChildrenRelative();
	calcChildrenOffsets();
}

void PhysicsObject::calcChildrenOffsets()
{
	for (int a=0;a<numChildren;++a)
	{
		for (int b=0;b<3;++b)
		{
			child[a].parentOffset[b]=child[a].realPos[b]-realPos[b];
		}
	}
}

void PhysicsObject::updateChildrenPos()
{
	for (int a=0;a<numChildren;++a)
	{
		for (int b=0;b<3;++b)
		{
			child[a].realPos[b]=child[a].parentOffset[b]+realPos[b];
			child[a].pos[b]=child[a].parentOffset[b]+pos[b];
		}
	}
}


void PhysicsObject::setup(float posx,float posy, float posz,float radius)
{
	float p[3];
	p[0]=posx;
	p[1]=posy;
	p[2]=posz;
	setRealPos(p);
	this->radius=radius;
}

float PhysicsObject::getLowestPoint()
{

	float lowest=1000000000;
	float pt;

	if (numChildren>0)
	{
		for (int a=0;a<numChildren;++a)
		{
			pt=child[a].getLowestPoint();
			if (pt<lowest)
			{
				lowest=pt;
			}
		}
	}
	else
	{
		pt=pos[1]-radius;
		if (pt<lowest)
		{
			lowest=pt;
		}
	}
	return lowest;
}

void PhysicsObject::calcOldPos()
{
	oldPos[0]=realPos[0];
	oldPos[1]=realPos[1];
	oldPos[2]=realPos[2];
}


double PhysicsObject::getFrameDist()
{
	double v[3];
	v[0]=realPos[0]-oldPos[0];
	v[1]=realPos[1]-oldPos[1];
	v[2]=realPos[2]-oldPos[2];

	return sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]);
}

float PhysicsObject::getBottomOffset()
{
	float bottom=getLowestPoint();
	return bottom-pos[1];
}


void PhysicsObject::doVelocity()
{
	float frictAmount;
	if (speedmod>0.01)
	{
		frictAmount=pow(friction,speedmod);
	}

	else
	{
		frictAmount=1;
	}
	
	float avVel[3];
	
	if (fabs(vel[0])>0 || fabs(vel[1])>0 || fabs(vel[2])>0)
	{
		for (int a=0;a<3;++a)
		{
			vel[a]*=frictAmount;
		}
/*		vectorAdd(vel,oldvel,avVel); //use average velocity
		for (a=0;a<3;++a)
		{
			avVel[a]*=0.5;
			pos[a]+=avVel[a]*speedmod;
			//pos[a]+=vel[a]*speedmod;
		}
		vectorEquals(oldvel,vel);*/
		
		if (fabs(vel[0])<10 && fabs(vel[1])<10 && fabs(vel[2])<10)
		{
			for (a=0;a<3;++a)
			{
				pos[a]+=vel[a]*speedmod;
			}
		}
	}
}

void PhysicsObject::addVelocity(float vel_in[3])
{
	vel[0]+=vel_in[0];
	vel[1]+=vel_in[1];
	vel[2]+=vel_in[2];
}

void PhysicsObject::velChange(float vel_in[3])
{
	
	//vectorAdd(vel,vel_in,vel);
//	vectorEquals(vel,vel_in);
	for (int a=0;a<3;++a)
	{
		vel[a]=vel_in[a];
	}
	/*
	float aWeight=lengthOf(vel);
	float bWeight=lengthOf(vel_in);


	float totalLength=aWeight+bWeight;

	if (totalLength<0.001)
	{
		return;
	}

	aWeight/=totalLength;
	bWeight=1.0-aWeight;

	glCls();
	gl("vel %f %f %f\n",vel[0],vel[1],vel[2]);

	for (int a=0;a<3;++a)
	{
		vel[a]=aWeight*vel[a]+bWeight*vel_in[a];
	}
	glPrintf("new %f %f %f\n",vel[0],vel[1],vel[2]);
	*/
	/*
	float t=speedmod/timeToComplete;
	if (t>1) t=1;
	float oneMinusT=1.0-t;

	for (int a=0;a<3;++a)
	{
		vel[a]=(oneMinusT*vel[a])+(t*vel_in[a]);
	}*/
}


void PhysicsObject::slowDown(float amt)
{
	vel[0]*=amt;
	vel[1]*=amt;
	vel[2]*=amt;
}