// PhysicsObject.h: interface for the PhysicsObject class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_PHYSICSOBJECT_H__2810925F_9A93_4772_A919_DFF356DEE709__INCLUDED_)
#define AFX_PHYSICSOBJECT_H__2810925F_9A93_4772_A919_DFF356DEE709__INCLUDED_

class PhysicsSystem;

class PhysicsObject;

struct CollisionResponse
{
	CollisionResponse(){enabled=false;}
	bool enabled;
	float vect[3];
	PhysicsObject * ob;
};

struct SphereOfInfluence
{
	float radius;
	float pos[3];
};

class PhysicsObject  
{
	friend PhysicsSystem;
public:
	PhysicsObject();
	//PhysicsObject(PhysicsSystem * parent)
	PhysicsObject(const PhysicsObject &src);

	void reset();

	virtual ~PhysicsObject();

	void setRealPos(const float realPos_in[3]); //move the object without collisions
	void calcVect();
	void doVelocity();
	void moveToRealPos();

	const PhysicsObject& operator= (const PhysicsObject &src); 
	void copyFrom(const PhysicsObject &src);
	void calcSphereOfInfluence();

	float * getRealPos(){return realPos;}
	float * getPos(){return pos;}

	bool getCollision(){return collision;}
	void setCollision(int cID_in){collision=true; collidedID=cID_in;}
	void clearCollision(){collision=false;}
	int getCollisionID(){return collidedID;}
	void setID(int ID_in){ID=ID_in;}
	void setIndex(int index_in){index=index_in;}

	int getID(){return ID;}
	int getIndex(){return index;}
	
	PhysicsObject * getChild(int index){return &child[index];}
	void createChildren(int num);
	int getNumChildren(){return numChildren;}
	void getDimensionsFromChildren();

	void calcOldPos();
	void setup(float posx,float posy, float posz,float radius);
	double getFrameDist();

	void updateChildrenPos();
	void calcChildrenOffsets();
	float getLowestPoint();
	float getBottomOffset();

	void addVelocity(float vel_in[3]);
	void velChange(float vel_in[3]); 
	void slowDown(float amt);

	float vel[3];

	float pos[3];
	float realPos[3];
	float radius;
	void setFriction(float frict){friction=frict;}
protected:
	float friction;

	float oldvel[3];

	PhysicsObject * parent;
	float vect[3];
	float parentOffset[3];

	bool enabled;
	bool moving;
	SphereOfInfluence infSphere;

	PhysicsObject * child;
	int numChildren;

	//both index and ID are set to the same value when the object is created, however, 
	//index is updated whenever the object is moved
	int ID; 
	int index;
	int collidedID;
	bool collision;
	float oldPos[3];
	
	CollisionResponse cr;
};

#endif // !defined(AFX_PHYSICSOBJECT_H__2810925F_9A93_4772_A919_DFF356DEE709__INCLUDED_)
