/*
(c)2002 Ben Woodhouse

ben@elf-stone.com
www.elf-stone.com

This code may not be used in a commercial product without my permission.
If you redistribute it, this message must remain intact. 
*/
// PhysicsSystem.cpp: implementation of the PhysicsSystem class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PhysicsSystem.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


PhysicsSystem::PhysicsSystem()
{
	numObjects=0;
	enabled=true;
	currentID=0;
	clearInsides();

	memset((void *)indexMap,-1,sizeof(int)*MAX_PHYSICS_OBJECTS);
}

PhysicsSystem::~PhysicsSystem()
{

}

void PhysicsSystem::reset()
{
	numObjects=0;
	enabled=true;
	currentID=0;
	clearInsides();
	memset((void *)indexMap,-1,sizeof(int)*MAX_PHYSICS_OBJECTS);

	//reset all the objects
	for (int a=0;a<MAX_PHYSICS_OBJECTS;++a)
	{
		objects[a].reset();
	}
}

void PhysicsSystem::clearObjects()
{
	numObjects=0;
}

PhysPtr PhysicsSystem::createObject(float posx,float posy, float posz, float radius)
{
	/*
	PhysicsObject * ob=new PhysicsObject;
	ob->radius=radius;
	float pos[3];

	pos[0]=posx;
	pos[1]=posy;
	pos[2]=posz;

	ob->setRealPos(pos);
	return ob;
	*/
	
	if (numObjects>=MAX_PHYSICS_OBJECTS)
	{
		messageBox("MAX_PHYSICS_OBJECTS exceeded");
		PhysPtr nullPtr;
		return nullPtr;
	}


	PhysicsObject * ob=&objects[numObjects];

	ob->reset();
	ob->setIndex(numObjects);


	
	int newID=genID();
	indexMap[newID]=numObjects;
 	ob->setID(newID); 
	PhysPtr phys(this,newID); //create a fake pointer to the object


	++numObjects;

	ob->radius=radius;
	float pos[3];

	pos[0]=posx;
	pos[1]=posy;
	pos[2]=posz;
 
	ob->setRealPos(pos);



//	PhysicsObject * newOb=new PhysicsObject(ob);
//	return newOb;
	//objects.push_back(ob);
	//messageBox("end of createOb");

	return phys;
}

void PhysicsSystem::deleteObject(PhysicsObject * obj)
{
	//get the current index for the object
	int index=obj->index;
	indexMap[obj->ID]=-1; //clear the indexmap entry for this ID

	//don't swap if it's the last object
	if (index!=numObjects-1)
	{
		//move the endmost object into its place
		*obj=objects[numObjects-1];

		//update the indexmap
		indexMap[obj->ID]=index;

		//update the index
		obj->index=index;
	}

	//effecively delete the last object
	--numObjects;
}


//returns true if there is an intersection
//first intersection point is returned in iPos_out
//Pre: assumes line does not start in the sphere
bool PhysicsSystem::lineSphereIntersection(float linea[3], 
										   float lineb[3], 
										   float spherePos[3], 
										   float radius,
										   float iPos_out[3])
{
	//a = (x2 - x1)^2 + (y2 - y1)^2 + (z2 - z1)^2
	float xdist=lineb[0]-linea[0];
	float ydist=lineb[1]-linea[1];
	float zdist=lineb[2]-linea[2];

	float u=( (spherePos[0]-linea[0])*xdist + (spherePos[1]-linea[1])*ydist + (spherePos[2]-linea[2])*zdist )
		  / (xdist*xdist+ydist*ydist+zdist*zdist);

	//get u pos
	float ux,uy,uz;

	ux=linea[0]+xdist*u;
	uy=linea[1]+ydist*u;
	uz=linea[2]+zdist*u;

	//check if upos is in the sphere
	float udx=spherePos[0]-ux;
	float udy=spherePos[1]-uy;
	float udz=spherePos[2]-uz;

	float udistSquared=udx*udx+udy*udy+udz*udz;
	float radiusSquared=radius*radius;

	if (udistSquared>radiusSquared) return 0;  //upos is not in the sphere. There is no intersection


	float a=xdist*xdist+ydist*ydist+zdist*zdist;
	// b = 2[ (x2 - x1) (x1 - x3) + (y2 - y1) (y1 - y3) + (z2 - z1) (z1 - z3) ] 
	float b=2* ( xdist*(linea[0]-spherePos[0]) + ydist*(linea[1]-spherePos[1]) +
				 zdist*(linea[2]-spherePos[2]) );

	//c = x3^2 + y3^2 + z3^2 + x1^2 + y1^2 + z1^2 - 2[x3 x1 + y3 y1 + z3 z1] - r^2 
	float c=spherePos[0]*spherePos[0] + spherePos[1]*spherePos[1] + spherePos[2]*spherePos[2] +
		    linea[0]*linea[0] + linea[1]*linea[1] + linea[2]*linea[2] 
			-2*(spherePos[0]*linea[0] + spherePos[1]*linea[1] + spherePos[2]*linea[2]) - radiusSquared;

	//b * b - 4 * a * c 
	float result=b*b-4*a*c;
/*
	glPrintf("line a %f %f %f\n",linea[0],linea[1],linea[2]);
	glPrintf("sphere %f %f %f\n",spherePos[0],spherePos[1],spherePos[2]);

	glPrintf("a%f b%f c%f result%f\n",a,b,c,result);
*/

	float du,du_b;
	float sqrtRes;
	if (result<0) //this shouldn't happen, but check for it anyway
	{
		return false;
	}
	if (result==0) //one intersection
	{
		du=-b/(2-a);
	}
	else
	{
		sqrtRes=sqrt(result);

		//2 intersections
		du=  (-b + sqrtRes)/(2*a);

		du_b=(-b - sqrtRes)/(2*a);

		//take the lowest value. Only interested in first intersection point
		if (du_b<du) du=du_b;
	}

	if (du<0 || du>1) return false;
	iPos_out[0]=linea[0]+du*xdist;
	iPos_out[1]=linea[1]+du*ydist;
	iPos_out[2]=linea[2]+du*zdist;

	return true;
}


//returns true if there is an intersection
//returns the distance down the line (U) in u_out
//Pre: assumes line does not start in the sphere
bool PhysicsSystem::lineSphereIntersection_U(  float linea[3], 
											   float lineb[3], 
											   float spherePos[3], 
											   float radius,
											   float & u_out)
{
	//a = (x2 - x1)^2 + (y2 - y1)^2 + (z2 - z1)^2
	float xdist=lineb[0]-linea[0];
	float ydist=lineb[1]-linea[1];
	float zdist=lineb[2]-linea[2];

	float u=( (spherePos[0]-linea[0])*xdist + (spherePos[1]-linea[1])*ydist + (spherePos[2]-linea[2])*zdist )
		  / (xdist*xdist+ydist*ydist+zdist*zdist);

	//get u pos
	float ux,uy,uz;

	ux=linea[0]+xdist*u;
	uy=linea[1]+ydist*u;
	uz=linea[2]+zdist*u;

	//check if upos is in the sphere
	float udx=spherePos[0]-ux;
	float udy=spherePos[1]-uy;
	float udz=spherePos[2]-uz;

	float udistSquared=udx*udx+udy*udy+udz*udz;
	float radiusSquared=radius*radius;

	if (udistSquared>radiusSquared) return 0;  //upos is not in the sphere. There is no intersection


	float a=xdist*xdist+ydist*ydist+zdist*zdist;
	// b = 2[ (x2 - x1) (x1 - x3) + (y2 - y1) (y1 - y3) + (z2 - z1) (z1 - z3) ] 
	float b=2* ( xdist*(linea[0]-spherePos[0]) + ydist*(linea[1]-spherePos[1]) +
				 zdist*(linea[2]-spherePos[2]) );

	//c = x3^2 + y3^2 + z3^2 + x1^2 + y1^2 + z1^2 - 2[x3 x1 + y3 y1 + z3 z1] - r^2 
	float c=spherePos[0]*spherePos[0] + spherePos[1]*spherePos[1] + spherePos[2]*spherePos[2] +
		    linea[0]*linea[0] + linea[1]*linea[1] + linea[2]*linea[2] 
			-2*(spherePos[0]*linea[0] + spherePos[1]*linea[1] + spherePos[2]*linea[2]) - radiusSquared;

	//b * b - 4 * a * c 
	float result=b*b-4*a*c;
/*
	glPrintf("line a %f %f %f\n",linea[0],linea[1],linea[2]);
	glPrintf("sphere %f %f %f\n",spherePos[0],spherePos[1],spherePos[2]);

	glPrintf("a%f b%f c%f result%f\n",a,b,c,result);
*/

	float du,du_b;
	float sqrtRes;
	if (result<0) //this shouldn't happen, but check for it anyway
	{
		return false;
	}
	if (result==0) //one intersection
	{
		du=-b/(2-a);
	}
	else
	{
		sqrtRes=sqrt(result);

		//2 intersections
		du=  (-b + sqrtRes)/(2*a);

		du_b=(-b - sqrtRes)/(2*a);

		//take the lowest value. Only interested in first intersection point
		if (du_b<du) du=du_b;
	}

	if (du<0 || du>1) return false;
	u_out=du;
	return true;
}

void PhysicsSystem::doPhysics()
{
	

	if (enabled)
	{
		for (int a=0;a<numObjects;++a)
		{
			objects[a].calcOldPos();
		}
		for (a=0;a<numObjects;++a)
		{
			objects[a].doVelocity();
		}

		updateObjectChildren();

		calcInsides();
		clearObjectCollisions();

		buildMovingStaticLists();
		buildCScoreList();

		for (a=0;a<2;++a)
		{
			if (!doCollisions()) break;
			convertCollsionResponses();
			buildMovingStaticLists();
			buildCScoreList();
		}
		//moveObjects(1);
		for (a=0;a<numObjects;++a)
		{
			objects[a].moveToRealPos();
			objects[a].cr.enabled=false;
		}
		updateObjectChildren();
	}
}

void PhysicsSystem::clearObjectCollisions()
{
	for (int a=0;a<numObjects;++a)
	{
		objects[a].clearCollision();
	}
}

void PhysicsSystem::convertCollsionResponses()
{
	for (int a=0;a<numObjects;++a)
	{
		if (objects[a].cr.enabled)
		{
			objects[a].moving=true;
			for (int b=0;b<3;++b)
			{
				objects[a].vect[b]=objects[a].cr.vect[b];
				objects[a].pos[b]=objects[a].realPos[b]+objects[a].vect[b];
			}
			if (objects[a].numChildren>0)
			{
				objects[a].updateChildrenPos();
			}
		}
	}
}

bool PhysicsSystem::doCollisions()
{
	//get the lowest score, and do the collision
	//(see the doc for details)
	float collisionTime;
	float lowestTime=9999999;
	int lowestCNum=0;
	float * cVect;
	float time=0;
	int a;
	bool allCollisionsDone=false;

	//glPrintf("numCScores %d",numCScores);
	int numCScoresTested=0;

	bool firstCollision=true;

	int collisionCount=0;
	int lc[255];

	bool collisionResponseb=false;

//	if (clearScreen) glCls();

//	glPrintf("-");
	while (!allCollisionsDone)
	{
		//find a collision
		lowestTime=9999;
		lowestCNum=-1; //if this remains at -1 then no score was found
		//glCls();
		//glPrintf("numCScores %d",numCScores);
		for (a=0;a<numCScores;++a)
		{
			if (cScoreList[a].time<lowestTime)
			{
				lowestTime=cScoreList[a].time;
				lowestCNum=a;
			}
		}


		if (lowestCNum!=-1)
		{
			CScore & collision=cScoreList[lowestCNum];
			collisionTime=collision.time;
			cVect=collision.cVect;
			float remainingTime=1.0-time;
			float timeTaken=collisionTime*remainingTime;

			if (collisionTime>0.01)
				moveObjects(collisionTime-0.01);

			time+=timeTaken; //keep track of the time
			collisionResponseb=true;
			collisionResponse(cScoreList[lowestCNum].a,cScoreList[lowestCNum].b,cVect);

			
			//things have moved, so rebuild the lists
			buildMovingStaticLists();
			buildCScoreList(); 

			lc[collisionCount]=lowestCNum;
			++collisionCount;
		}
		else
		{
			allCollisionsDone=true;
		}
	}
	

	//if (collisionCount>1) messageBox("%d %d %d",collisionCount,lc[0],lc[1]);
	//if (lowestCNum==-1) //there are no more collisions to test
	//{
		moveObjects(1.0-time);
		//moveObjects(1);
//	}
		return collisionResponseb;
}

void PhysicsSystem::moveObjects(float time)
{
	int a,b;
	float vect[3];


	for (a=0;a<numMoving;++a)
	{
		for (b=0;b<3;++b)
		{
			vect[b]=time*movingList[a]->vect[b];
			movingList[a]->realPos[b]+=vect[b];
			movingList[a]->vect[b]-=vect[b];
		}
	}
}


void PhysicsSystem::buildMovingStaticLists()
{
	numMoving=0;
	numStatic=0;
	for (int a=0;a<numObjects;++a)
	{
		objects[a].calcVect();
		if (objects[a].moving)
		{
			movingList[numMoving++]=&objects[a];
		}
		else
		{
			staticList[numStatic++]=&objects[a];
		}
		objects[a].calcSphereOfInfluence();
	}
	//glPrintf("%d",numMoving);

}


//requires movingList and staticList to have been built
void PhysicsSystem::buildCScoreList()
{
	int b;

	numCScores=0;
	for (int a=0;a<numMoving;++a)
	{	
		for (b=a+1;b<numMoving;++b)
		{
			buildCScore(movingList[a],movingList[b]);

		}
		for (b=0;b<numStatic;++b)
		{

			buildCScore(movingList[a],staticList[b]);

		}
	}
}

void PhysicsSystem::buildCScore(PhysicsObject *obA, PhysicsObject *obB)
{
	float time;
	float cVect[3];
	bool checkChildren=true;

	if (simpleObCheck(obA,obB))
	{
		if (detectCollision(obA,obB,&time,cVect))
		{
			if (obA->numChildren==0 && obB->numChildren==0)
			{
				addCScore(obA,obB,cVect,time);
				checkChildren=false;
			}
			else
			{
				insideList[obA->index][obB->index]=true;
				insideList[obB->index][obA->index]=true;
				checkChildren=true;
			}
		}
		else
		{
			if (insideList[obA->index][obB->index])
			{
				if (obA->numChildren>0 || obB->numChildren>0)
				{
					checkChildren=true;
				}
			}
		}
		if (checkChildren)
		{
			int a,b;
			if (obA->numChildren==0) //obb has children
			{
				for (b=0;b<obB->numChildren;++b)
				{
					if (detectCollision(obA,&obB->child[b],&time,cVect))
					{
						addCScore(obA,&obB->child[b],cVect,time);
					}
				}
			}
			else if (obB->numChildren==0) //oba has children
			{
				for (a=0;a<obA->numChildren;++a)
				{
					if (detectCollision(&obA->child[a],obB,&time,cVect))
					{
						addCScore(&obA->child[a],obB,cVect,time);
					}
				}
			}
			else //both objects have children
			{
				for (a=0;a<obA->numChildren;++a)
				{
					for (b=0;b<obB->numChildren;++b)
					{
						if (detectCollision(&obA->child[a],&obB->child[b],&time,cVect))
						{
							addCScore(&obA->child[a],&obB->child[b],cVect,time);
						}
					}
				}
			}
		}
	}
}

void PhysicsSystem::addCScore(PhysicsObject * obA, PhysicsObject * obB, float cVect[3],float time)
{
	cScoreList[numCScores].cVect[0]=cVect[0];
	cScoreList[numCScores].cVect[1]=cVect[1];
	cScoreList[numCScores].cVect[2]=cVect[2];
	cScoreList[numCScores].time=time;//dist/speed;
	cScoreList[numCScores].a=obA;
	cScoreList[numCScores].b=obB;
	++numCScores;
}

inline bool PhysicsSystem::simpleObCheck(PhysicsObject *obA, PhysicsObject *obB)
{
	float *posa=obA->infSphere.pos;
	float *posb=obB->infSphere.pos;

	float vect[3];
	vect[0]=posb[0]-posa[0];
	vect[1]=posb[1]-posa[1];
	vect[2]=posb[2]-posa[2];
	float dist=sqrt(vect[0]*vect[0]+vect[1]*vect[1]+vect[2]*vect[2]);

	//float dist=fabs(posb[0]-posa[0])+fabs(posb[1]-posa[1])+fabs(posb[2]-posa[2]);
	if (dist<(obA->infSphere.radius+obB->infSphere.radius))
	{
		return true;
	}
	return false;
}


//handles a possible collision
//returns true if there was a collision
bool PhysicsSystem::detectCollision(PhysicsObject *obA,
									PhysicsObject *obB, 
									float *time_out, 
									float cVect_out[3])
{
	float sphereRad=obB->radius+obA->radius;
	float spherePos[3];
	float linea[3];
	float lineb[3];
	
	for (int a=0;a<3;++a)
	{
		spherePos[a]=obB->realPos[a];
		linea[a]=obA->realPos[a];
		cVect_out[a]=obA->vect[a] - obB->vect[a];
		lineb[a]=linea[a]+cVect_out[a];
	}

	float t;

	bool res=lineSphereIntersection_U(linea,lineb,spherePos,sphereRad,t);

	//subtract the distance moved from cVect
	float cNorm[3];
	cNorm[0]=cVect_out[0];
	cNorm[1]=cVect_out[1];
	cNorm[2]=cVect_out[2];
	normalize(cNorm);
	
	normalize(cVect_out);
	for (a=0;a<3;++a)
	{
		cVect_out[a]=cNorm[a]*(1.0-t);
	}

//	res=false;
	*time_out=t;

	return res;	
}

//the object should now be on the edge of the sphere
void PhysicsSystem::collisionResponse(PhysicsObject * obA, PhysicsObject * obB, const float cVect[3])
{
	int a;
	//FIND P
	//obA is the point, obB is the sphere
	//A the intersection point with the sphere
	//B is the projected point end pos (were there no collision)
	//C is the centre of the sphere 
	//P is the adjusted point pos on the tangent plane of the sphere

	//get the tangent plane of the sphere 
	//normal is CA / |CA|
	//USE CVECT
	
	float radius=obB->radius+obA->radius;
	float tanPlane[4];
	float C[3];



	for (a=0;a<3;++a)
	{
		tanPlane[a]=obA->realPos[a]-obB->realPos[a];
		C[a]=obB->realPos[a]-obA->realPos[a];
	}
	normalize(tanPlane);
	tanPlane[3]=0;//radius; //plane is in the intersection point's local coordinates
	
	//put posB into intersection point's local coordinates
	
	const float * posB=&cVect[0];

	//line BP has plane's normal
	//U = k - org.norm
	float u=tanPlane[3]-(tanPlane[0]*posB[0] + tanPlane[1]*posB[1] + tanPlane[2]*posB[2]);

	float posP[3];
	//P = norm*U+org
	posP[0]=u*tanPlane[0]+posB[0]; 	
	posP[1]=u*tanPlane[1]+posB[1];
	posP[2]=u*tanPlane[2]+posB[2];

	//when responding to a collision involving child objects, the trajectory must be taken 
	//from the child object but the parent object must be moved
	//hence:
	PhysicsObject *moveObA,*moveObB;

	bool aParent;
	bool bParent;
	if (obA->parent==NULL) 
	{
		aParent=false;
		moveObA=obA;
	}
	else 
	{
		aParent=true;
		moveObA=obA->parent;
	}

	if (obB->parent==NULL) 
	{
		bParent=false;
		moveObB=obB;
	}
	else 
	{
		bParent=true;
		moveObB=obB->parent;
	}


	//set the collisionID in the parent level objects
	moveObA->setCollision(moveObB->getID());
	moveObB->setCollision(moveObA->getID());

	//posP is now the vector from A to P
	//in reality, both A and B could be moving, so posP needs to translated for both spheres

	//get Aweight and Bweight. These represent the proportion of the movement for each object
	float Aweight=lengthOf(moveObA->vect);
	float Bweight=lengthOf(moveObB->vect);

	float totalWeight=Aweight+Bweight;
	Aweight=Aweight/totalWeight;
	Bweight=Bweight/totalWeight;

	Aweight*=0.5; //friction
	Bweight*=0.5;


	//A's col response vector is its weight * posP
	//B's is bweight * -posp
	//float aVect[3];

	//enable the collision response step for both objects
	if (Aweight>=0)
	{
		moveObA->cr.enabled=true;
		for (a=0;a<3;++a)
		{
			moveObA->cr.vect[a]=Aweight*posP[a]*speedmod;
		}
	}

	if (Bweight>=0)
	{
		moveObB->cr.enabled=true;
		for (a=0;a<3;++a)
		{
			moveObB->cr.vect[a]=Bweight*-posP[a]*speedmod;
		}
	}

	//stop the objects
	for (a=0;a<3;++a)
	{
		moveObA->vect[a]=0;
		moveObB->vect[a]=0;
		//moveObA->vel[a]=moveObA->vel[a]*0.5;;
		//moveObB->vel[a]=moveObB->vel[a]*0.5;
		moveObA->vel[a]=0;
		moveObB->vel[a]=0;
		moveObA->pos[a]=moveObA->realPos[a];
		moveObB->pos[a]=moveObB->realPos[a];
	}

/*	
	glPrintf("  Collision response:\n");
	glPrintf("    C:              %f %f %f\n",C[0],C[1],C[2]);
	glPrintf("    cvect:          %f %f %f\n",cVect[0],cVect[1],cVect[2]);
	glPrintf("    tangent plane:  %f %f %f %f\n",tanPlane[0],tanPlane[1],tanPlane[2],tanPlane[3]);
	glPrintf("    obA vect:       %f %f %f\n",obA->vect[0],obA->vect[1],obA->vect[2]);
	glPrintf("    U :             %f\n",u);
	glPrintf("    moveObA:          %f %f %f\n",posB[0],posB[1],posB[2]);
*/	
	//if (aParent) moveObA->updateChildrenPos();
	//if (bParent) moveObB->updateChildrenPos();
/*	if (sphereSphereIntersection(obA->pos,obA->radius,obB->pos,obB->radius))
		glPrintf("Collision error");*/
	//cVect=Av - Bv
	//Av=cVect+Bv

}

bool PhysicsSystem::sphereSphereIntersection(float posA[3], float radA, float posB[3], float radB)
{
	float vect[3];
	for (int a=0;a<3;++a)
	{
		vect[a]=posB[a]-posA[a];
	}

	float len=sqrt(vect[0]*vect[0]+vect[1]*vect[1]+vect[2]*vect[2]);

	if (len<=radA+radB) return true;
	else return false;
}


void PhysicsSystem::debugDisplay()
{
	bool displayChildren=true;

	for (int a=0;a<numObjects;++a)
	{
		if (objects[a].numChildren>0 && displayChildren)
		{
			for (int b=0;b<objects[a].numChildren;++b)
			{
				float * cpos=objects[a].child[b].realPos;
				glPushMatrix();
					glTranslatef(cpos[0],cpos[1],cpos[2]);
					drawSphere(objects[a].child[b].radius,10,10);
				glPopMatrix();
			}
		}
		else
		{
			float * realPos=objects[a].realPos;
			glPushMatrix();
				glTranslatef(realPos[0],realPos[1],realPos[2]);
				drawSphere(objects[a].radius,10,10);
			glPopMatrix();
		}
	}

}

void PhysicsSystem::updateObjectChildren()
{
	for (int a=0;a<numObjects;++a)
	{
		if (objects[a].numChildren>0)
		{
			objects[a].updateChildrenPos();
		}
	}
}


int PhysicsSystem::genID()
{
	for (int a=0;a<MAX_PHYSICS_OBJECTS;++a)
	{
		if (indexMap[a]==-1) return a;
	}
	return -1; //this can't ever happen if there are checks on the number of physics objects
				//created, which there are 
}

void PhysicsSystem::clearInsides()
{
	int a,b;

	for (a=0;a<MAX_PHYSICS_OBJECTS;++a)
	{
		for (b=0;b<MAX_PHYSICS_OBJECTS;++b)
		{
			insideList[a][b]=false;
		}
	}
}

void PhysicsSystem::calcInsides()
{
	PhysicsObject *obA,*obB;
	int a,b;
	for (a=0;a<numObjects;++a)
	{
		obA=&objects[a];
		for (b=a+1;b<numObjects;++b)
		{
			obB=&objects[b];
			if (sphereSphereIntersection(obA->realPos,obA->radius,obB->realPos,obB->radius))
			{
				insideList[a][b]=true;
				insideList[b][a]=true;
			}
			else
			{
				insideList[a][b]=false;
				insideList[b][a]=false;
			}
		}
	}
}


//////////////////////////////////////////////////////////////////////
// PhysPtr
//////////////////////////////////////////////////////////////////////
const PhysPtr& PhysPtr::operator= (const PhysPtr& src)
{
	id=src.id;
	physSys=src.physSys;

	return *this;
}




///////////////////////////////////////////////////////////////////////
