// PhysicsSystem.h: interface for the PhysicsSystem class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_PHYSICSSYSTEM_H__E9BB66EE_03A0_499D_AB35_BED22DC33F88__INCLUDED_)
#define AFX_PHYSICSSYSTEM_H__E9BB66EE_03A0_499D_AB35_BED22DC33F88__INCLUDED_

#define MAX_PHYSICS_OBJECTS 64

#include "PhysicsObject.h"

struct CScore
{
	float time;
	float cVect[3];
	PhysicsObject * a;
	PhysicsObject * b;
};


class PhysPtr;


class PhysicsSystem  
{
public:
	PhysicsSystem();
	virtual ~PhysicsSystem();
	void reset();

	PhysPtr createObject(float posx,float posy, float posz, float radius);
	void deleteObject(PhysicsObject * obj);

	void clearObjects();

	void doPhysics();

	//returns true if there is an intersection
	//first intersection point is returned in iPos_out
	//Pre: assumes line does not start in the sphere
	bool lineSphereIntersection(float linea[3], 
							   float lineb[3], 
							   float spherePos[3], 
							   float radius,
							   float iPos_out[3]);

	//returns true if there is an intersection
	//first intersection point is returned in iPos_out
	//Pre: assumes line does not start in the sphere
	bool lineSphereIntersection_U( float linea[3], 
								   float lineb[3], 
								   float spherePos[3], 
								   float radius,
								   float & u_out);

	bool sphereSphereIntersection(float posA[3], float radA, float posB[3], float radB);
	void debugDisplay();

	bool toggle(){
		enabled=!enabled;
		return enabled;
	}
	void enable(){enabled=true;}
	void disable(){enabled=false;}
	void updateObjectChildren();
	bool isEnabled(){return enabled;}
	void clearInsides();
	void calcInsides();
	PhysicsObject * getObjectByOID(int oid){return &objects[indexMap[oid]];}

protected:
	inline bool simpleObCheck(PhysicsObject *obA, PhysicsObject *obB);
	bool detectCollision(PhysicsObject *obA,PhysicsObject *obB, float *time_out, float cVect_out[3]);

	void clearObjectCollisions();
	void collisionResponse(PhysicsObject * a, PhysicsObject * b, const float cVect[3]); 
	void moveObjects(float time);
	void buildMovingStaticLists();
	void buildCScoreList();
	void buildCScore(PhysicsObject *obA, PhysicsObject *obB);
	void addCScore(PhysicsObject * obA, PhysicsObject * obB, float cVect[3],float time);

	bool doCollisions(); //returns true if there are collision responses
	void convertCollsionResponses();
	int genID();

	PhysicsObject objects[MAX_PHYSICS_OBJECTS];
	int numObjects;

	//used once per frame
	PhysicsObject *movingList[MAX_PHYSICS_OBJECTS];
	int numMoving;
	PhysicsObject *staticList[MAX_PHYSICS_OBJECTS];
	int numStatic;
	CScore cScoreList[MAX_PHYSICS_OBJECTS];
	int numCScores;

	bool insideList[MAX_PHYSICS_OBJECTS][MAX_PHYSICS_OBJECTS];
	int currentID;

	int indexMap[MAX_PHYSICS_OBJECTS]; //this maps IDs to indices, the latter of which can change
	bool enabled;
};


//a cunning PhysicsObject pointer, which always points to an object, even after it's been moved
//to free the object, use free()
class PhysPtr
{
public:
	PhysPtr(){id=-1; physSys=NULL;}
	void reset(){id=-1; physSys=NULL;}
	PhysPtr(PhysicsSystem * sys, int oid){id=oid; physSys=sys;}
	PhysicsObject * operator->(){return physSys->getObjectByOID(id);}
	void free(){physSys->deleteObject(physSys->getObjectByOID(id));}
	const PhysPtr& operator=(const PhysPtr& src);
protected:
	int id;
	PhysicsSystem * physSys;
};

#endif // !defined(AFX_PHYSICSSYSTEM_H__E9BB66EE_03A0_499D_AB35_BED22DC33F88__INCLUDED_)

