/*
(c)2002 Ben Woodhouse

ben@elf-stone.com
www.elf-stone.com

This code may not be used in a commercial product without my permission.
If you redistribute it, this message must remain intact. 
*/
// Sea.cpp: implementation of the CSea class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Sea.h"
#include "fastMaths.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSea::CSea(int res)
{
	//	sphere.set(10,50,10,10);
//	if (sphere.checkFrustum(viewFrustum)==COL_PART)

	//messageBox("HI");

	float sxz=((6.25*64.0)/81.0)*3;

	setScale(sxz,0.235294117);
	set(res,res,128);

	generateNormTable();
	windFactor=1;


	setupVisSpheres();

	clearScanLines();

	sky=NULL;
	//waveBeat=0;
	//messageBox("HI again");
} 

CSea::~CSea()
{

}

void CSea::setupVisSpheres()
{

	bigSphereSquares=27;
	lilSphereSquares=9;

	int squareRes=height-1;

	//setup the visibility spheres
	float bigTileSize=scalexz*bigSphereSquares;
	float lilTileSize=scalexz*lilSphereSquares;

	float halfBigTileSize=bigTileSize*0.5;
	float halfLilTileSize=lilTileSize*0.5;

	float start=-(float)squareRes*scalexz*0.5;

	//messageBox("start %f\nheight %f",start,height);
	float starty=128*scaley;


	float bigRadius=sqrt(halfBigTileSize*halfBigTileSize*2)+20;

	float lilRadius=bigRadius/3.0;

	for (int z=0;z<3;++z)
	{
		float zpos=start+z*bigTileSize;
		for (int x=0;x<3;++x)
		{
			float xpos=start+x*bigTileSize;

			visSphere[x][z].set(xpos+halfBigTileSize,
								starty,
								zpos+halfBigTileSize,
								bigRadius);

			visSphere[x][z].createChildren(9);

			int ccount=0;
			for (int zb=0;zb<3;++zb)
			{
				for (int xb=0;xb<3;++xb)
				{
					visSphere[x][z].getChild(ccount)->set(	xpos+xb*lilTileSize+halfLilTileSize,
															starty,
															zpos+zb*lilTileSize+halfLilTileSize,
															lilRadius);

					//visSphere[x][z].getChild(ccount)->set(10,60,10,10);

					++ccount;
				}
			}
		}
	}
}


void CSea::doFrustumCull(ViewFrustum &frustum)
{
	for (int y=0;y<3;++y)
	{
		for (int x=0;x<3;++x)
		{
			visSphere[x][y].checkFrustum(frustum); //also checks children on partials
		}
	}

	int x=0;
	int scanyStart;
	int scanyFin;
	int scanxStart;
	int scanxFin;

	for (y=0;y<9;++y)
	{
		scanLines[y][0]=83;
		scanLines[y][1]=-1;
	}

	int xb,yb;
	int cc;
	int smallx;
	int smally;

	for (y=0;y<3;++y)
	{
		scanyStart=y*3;
		scanyFin=scanyStart+3;
		BoundingSphere * vs;

		for (x=0;x<3;++x)
		{
			scanxStart=x*3;
			scanxFin=scanxStart+2;
			vs=&visSphere[x][y];
			
			switch (vs->vis)
			{
				case 1:
					//write the scanlines
					for (yb=scanyStart;yb<scanyFin;++yb)
					{
						if (scanxStart<scanLines[yb][0]) scanLines[yb][0]=scanxStart;
						if (scanxFin>scanLines[yb][1]) scanLines[yb][1]=scanxFin;
					}
				break;

				case 2:
					cc=0;
					smally=scanyStart;
					for (yb=0;yb<3;++yb)
					{
						for (xb=0;xb<3;++xb)
						{
							if (vs->getChild(cc)->vis)
							{
								smallx=scanxStart+xb;

								if (smallx<scanLines[smally][0]) scanLines[smally][0]=smallx;
								if (smallx>scanLines[smally][1]) scanLines[smally][1]=smallx;
							}
							++cc;
						}
						smally++;
					}
				break;
			}
		}

	}

	for (y=0;y<9;++y)
	{
		if (scanLines[y][0]==83)
			scanLines[y][0]=-1;
		else scanLines[y][0]*=lilSphereSquares;

		if (scanLines[y][1]==-1)
			scanLines[y][1]=-1;
		else scanLines[y][1]*=lilSphereSquares;
	}

/*
	//display the information

	glCls();
	for (y=0;y<3;++y)
	{
		for (int x=0;x<3;++x)
		{
			//BoundingSphere sphere=&visSphere[x][y];
			BoundingSphere * sphere=visSphere[0][0].getChild(x+y*3);
		
			glPrintf("[%f %f %f]\n",sphere->getPos(0),sphere->getPos(1),sphere->getPos(2));
		}
		glPrintf("\n");
	}
*/
	/*
	glCls();
	
	for (y=0;y<9;++y)
	{
		for (x=0;x<9;++x)
		{
			if (x>=scanLines[y][0] && x<=scanLines[y][1])
			  glPrintf("x");
			else glPrintf("_");
		}
		glPrintf(" %d to %d\n",scanLines[y][0],scanLines[y][1]);
	}
	glPrintf("desiredDepth %f",desiredDepth);
	glPrintf("indexStrip [0].numIndices %d",indexStrip[0].numIndices );
	*/
}
 
void CSea::doStuff(ViewFrustum &frustum)
{
	doFrustumCull(frustum);
}

void CSea::clearScanLines()
{
	for (int y=0;y<9;++y)
	{
		scanLines[y][0]=-1;
		scanLines[y][1]=-1;
	}
}
  
void CSea::draw()
{
	//glEnable(GL_LIGHTING);

	// enable spherical environment mapping

	
	//draw the spheres
/*	
	for (int y=0;y<3;++y)
	{
		for (int x=0;x<3;++x)
		{
			BoundingSphere * vs=&visSphere[x][y];
			glPushMatrix();
				glTranslatef(vs->getPos(0),vs->getPos(1),vs->getPos(2));

				drawSphere(vs->getRadius(),20,20);
//				drawSphere(265,20,20);
			glPopMatrix();
	
			for (int b=0;b<vs->getNumChildren();++b)
			{
				glPushMatrix();
					BoundingSphere * child=vs->getChild(b);

					glTranslatef(child->getPos(0),child->getPos(1),child->getPos(2));

					drawSphere(child->getRadius(),10,10);
				glPopMatrix();
			}

			
		}
	}
*/
	glPushAttrib(GL_ENABLE_BIT);
	glDisable(GL_LIGHTING);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	
	if (sky!=NULL)
	{
		if (sky->useCubemap)
		{	
			//use cube environment mapping 
			glPushMatrix();
			sky->getCubemap()->bind();
			glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_NV);
			glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_NV);
			glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_NV);

			glLoadIdentity();
			

			screen.defaultVP.setViewMatrix();
			
			glMatrixMode(GL_TEXTURE);
			glPushMatrix();
				glScalef(-1,-1,-1);
				screen.defaultVP.rotateByInvRotMat();
				
				glEnable(GL_TEXTURE_GEN_S);
				glEnable(GL_TEXTURE_GEN_T);
				glEnable(GL_TEXTURE_GEN_R);

				
				glEnable(GL_NORMALIZE);
				//drawSphere(100,20,20);
				drawHeightmap();

				glPopMatrix();
				glMatrixMode(GL_MODELVIEW);

			glPopMatrix();
		}
		else
		{
			tex->bind();
			glMatrixMode(GL_TEXTURE);
			glPushMatrix();
				glScalef(0.7,0.7,0.7);

				//use sphere environment mapping
				glEnable(GL_TEXTURE_GEN_S);
				glEnable(GL_TEXTURE_GEN_T);
				//glColor4f(1,1,1,0.6);
				glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
				glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);

				drawHeightmap();

				glDisable(GL_TEXTURE_GEN_S);
				glDisable(GL_TEXTURE_GEN_T);
			glPopMatrix();
			glMatrixMode(GL_MODELVIEW);


		}
	}

	glPopAttrib();
}


void CSea::drawHeightmap()
{
	array->enable();

	tex->bind();

	int scanWidth;
	int scanStartOffset;
	int s;
	
	for (int a=0;a<height-1;++a)
	{
		s=a/lilSphereSquares;
		if (scanLines[s][0]!=-1 && scanLines[s][1]!=-1)
		{
			scanWidth=(scanLines[s][1]-scanLines[s][0]+lilSphereSquares)*2;
			scanStartOffset=scanLines[s][0]*2;

			glDrawElements(
				GL_TRIANGLE_STRIP,
				scanWidth,
				GL_UNSIGNED_INT,
				&indexStrip[a].indices[scanStartOffset]
				);
		}
	}
	
/*
	for (int a=0;a<height-1;++a)
	{
			glDrawElements(
				GL_TRIANGLE_STRIP,
				indexStrip[a].numIndices,
				GL_UNSIGNED_INT,
				&indexStrip[a].indices[0]
				);
	}
*/

	array->disable();
}


void CSea::setWindFactor(float amt)
{
	windFactor=amt;
}

void CSea::animate()
{
	
	//waveBeat+=speedmod*0.01;

	if (timeTaken<1)
	{
		float depthmoved;

		depthmoved=(1-cos(timeTaken*3.141)*0.5*distanceToGo);

		timeTaken+=0.005*speedmod;

		depthmoved=(1-cos(timeTaken*3.141)*0.5*distanceToGo)-depthmoved;

		depth+=depthmoved;

		updateColSpheres();

		if (timeTaken>=1) depth=desiredDepth;
	}
	//if (waveBeat>6.283) waveBeat-=6.283;

	//float waveHeight=sin(waveBeat);

	int p=1;
	int cp=3;
	float oneOverWind=1.0/windFactor;
	float texDepth;
	float opacity;
	float depthFactor;

	float totalWind=windFactor+0.2;
/*
	static timer_c tim;
	static bool fast=false;

	if (tim.getElapsed()>1)
	{
		glCls();
		fast=!fast;

		//fast=false;
		glPrintf(fast ? "fast\n" : "slow\n");
		tim.start();
	}
*/
	bool yCentre=false;
	float *syncMapp;
	float heightMod=speedmod*0.03*windFactor;

	int scanStart;
	int scanEnd;
	int s;
	int x;

	int unseenWidth;

	float windFactorSquared=windFactor*windFactor;

	for (int y=0;y<height;++y)
	{
		if (y>26 && y<56) yCentre=true;
		else yCentre=false;

		s=y/lilSphereSquares;
		if (s==9) s=8;

		//update the syncmap for the whole row
		syncMapp=&syncMap[y][0];
		for (x=0;x<width;++x)
		{
			*syncMapp+=heightMod;
			if (*syncMapp>6.26) *syncMapp-=6.26;
			++syncMapp;
		}
		

		//if the scanline exists
		if (scanLines[s][0]!=-1 && scanLines[s][1] != -1)
		{
			scanStart=scanLines[s][0];
			scanEnd=(scanLines[s][1]+lilSphereSquares);

			//increment the pointers for the left side
			cp+=scanStart*4;
			p+=scanStart*3;

			//do the visible stuff
			for (x=scanStart;x<=scanEnd;++x)
//			for (x=0;x<width;++x)
			{
				texDepth=depth-depthMap[x][y];

				//set the opacity for the point
				if (yCentre && x>26 && x<56)
				{
					opacity=texDepth*0.4;
					if (opacity>1) 
					{
						opacity=1;
					}
					else 
					{
						if (opacity<0) opacity=0.6;
						else
						{
							opacity*=0.6;
							opacity+=0.4;
						}



						//opacity=1;
						//opacity*=10;
					}
					array->colour[cp]=opacity;
				}
				
				depthFactor=array->colour[cp]*2;//texDepth*0.2;
				array->pos[p]=depth+fastSin_noCheck(syncMap[x][y])*depthFactor*totalWind;
			
				p+=3;
				cp+=4;
			}
			//increment the pointers for the right side
			
			unseenWidth=width-(scanEnd+1);

			cp+=unseenWidth*4;
			p+=unseenWidth*3;

		}
		//otherwise increment the pointers
		else
		{
			p+=width*3;
			cp+=width*4;
		}
	}

	recalculateNormals();
}

void CSea::recalculateNormals()
{
	int x,y;
	int yInc=width*3;
	int pos=1;
	float a,b,c;
	int s;

	int scanStart;
	int scanEnd;

	for (y=0;y<height-1;++y)
	{
		s=y/lilSphereSquares;

		if (scanLines[s][0]!=-1 && scanLines[s][1]!=-1)
		{
			scanStart=scanLines[s][0];
			scanEnd=scanLines[s][1]+lilSphereSquares;

			pos+=scanStart*3;

			for (x=scanStart;x<scanEnd;++x)
			{
				a=array->pos[pos];
				b=array->pos[pos+3];
				c=array->pos[pos+yInc];

				normTableLookup(a,b,c,&array->normal[pos-1]);
				pos+=3;
			}
			pos+=((width-1)-scanEnd)*3;
		}
		//increment pos the appropriate amount
		else
		{
			pos+=3*(height-1);
		}
		pos+=3;
	}
}
void CSea::updateColSpheres()
{
	for (int y=0;y<3;++y)
	{
		for (int x=0;x<3;++x)
		{
			visSphere[x][y].setDPos(1,depth);
		}
	}

}

void CSea::setDepth(float depth_in)
{
	
	depth=depth_in;
	setDesiredDepth(depth);

 
	updateColSpheres();

}
void CSea::setDesiredDepth(float depth_in)
{
	
	desiredDepth=depth_in;
	timeTaken=0;
	distanceToGo=desiredDepth-depth;
	
}

/*
void CSea::setDepthMap(BYTE ** landPoints, float landScale)
{
//	float seaSize=scalexz*width;
//	float landSize=landScale*width;
	float seaScale=scalexz/landScale;
	landScale=landScale/scalexz;

	for (int y=0;y<height;++y)
	{
		for (int x=0;x<width;++x)
		{
			depthMap[y][x]=0;
		}
	}

	int landWidth=(float)width*landScale;
	int offset=(65.0-(float)landWidth)/2.0;

	glPrintf("Landwidth %d %f",landWidth,(float)width*landScale);

	int lx=0,ly=0;
	seaScale=3;

	for (y=offset;y<offset+landWidth;++y)
	{
		lx=0;
		for (int x=offset;x<offset+landWidth;++x)
		{
			depthMap[y][x]=(float)landPoints[lx][ly]*scaley;
			lx+=seaScale;
			
		}
		ly+=seaScale;
	}

}
*/
void CSea::loadDepthMap(char * fileName)
{
	CImage image;
	image.loadBMP(fileName);



//	convert the image to a syncmap
	BYTE * dat=image.getData();

	int p=0;
	for (int x=0;x<width;++x)
	{
		for (int y=0;y<height;++y)
		{
			depthMap[y][x]=((float)dat[p++]*scaley);
		
		}
	}
	//messageBox("depthMap[0][0] = %f\ndepthmap[50][50]=%f",depthMap[0][0],depthMap[50][50]);
}

void CSea::loadAlphaMap(char * fileName)
{
	CImage image;
	image.loadBMP(fileName);

	BYTE * dat=image.getData();

	int p=0;
	for (int x=0;x<width;++x)
	{
		for (int y=0;y<height;++y)
		{
			array->colour[p*4+3]=dat[p]/255.0;
			++p;
		}
	}
}

void CSea::loadSyncMap(char * fileName)
{
	CImage syncImage;
	syncImage.loadBMP(fileName);
//	convert the image to a syncmap
	BYTE * syncDat=syncImage.getData();

	int p=0;
	for (int x=0;x<width;++x)
	{
		for (int y=0;y<height;++y)
		{
			syncMap[y][x]=((float)syncDat[p++]/255.0)*6.283;
		}
	}

/*
	//make some noise
	for (int y=0;y<height;++y)
	{
		for (int x=0;x<height;++x)
		{
			syncMap[x][y]=(float)random(255)/255.0;
		}
	}

	//blur it
	for (int i=0;i<5;++i)
	{
		for (int y=0;y<height;++y)
		{
			for (int x=0;x<height;++x)
			{
				syncMap[x][y]=;
			}
		}
	}
	*/
}

//a and b c are float heights. The distance from a to b and b to c is calculated
//then converted to ints in the range -256 to 256
//the range is clipped to -64 to 63 
//then this range is mapped to 0 to 127, suitable for the lookup table
inline void CSea::normTableLookup(float a, float b, float c, float * normOut)
{
	
	int bi = (b-a)*20+64;
	int ci = (c-a)*20+64;

	if (bi>127) bi=127;
	else if (bi<0) bi=0;

	if (ci>127) ci=127;
	else if (ci<0) ci=0;

//	bi+=64;
//	ci+=64;

	int normPos=(bi+ci*128)*3;

	normOut[0]=normTable[normPos++];
	normOut[1]=normTable[normPos++];
	normOut[2]=normTable[normPos];
	
	/*
	float pointa[3]={0,0,0};
	float pointb[3];
	float pointc[3];

	tableUnit=0.05;
	pointb[0]=0;
	pointb[2]=scalexz;

	pointa[1]=a;
	pointb[1]=b;
	pointc[1]=c;

	pointc[0]=scalexz;
	pointc[2]=0;

 	getTriNormal(normOut,pointa,pointb,pointc);
	*/
}
