/*
(c)2002 Ben Woodhouse

ben@elf-stone.com
www.elf-stone.com

This code may not be used in a commercial product without my permission.
If you redistribute it, this message must remain intact. 
*/
// TornadoSpell.cpp: implementation of the TornadoSpell class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TornadoSpell.h"
#include "dauntless.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TornadoSpell::TornadoSpell(float target[3],Wizard * caster, Wizard * enemy)
{
//	glPrintf("tornado created");
	//tornado=new ParticleTornado(target[0],target[1],target[2],70);
	tornado=new ParticleTornado(target[0],target[1],target[2],70);
	vect[0]=0;
	vect[1]=0;
	timer=0;
	decisionTimer=50;

	mainGM.sounds[3]->getDuplicate(&windSound);

	wizLock[0]=false;
	wizLock[1]=false;

	for (int a=0;a<3;++a)
	{
	
		pos[a]=target[a];
	}

	tornado->setColour(0.8,0.8,0.8);

	windSound.setVolume(0);
	windSound.playRepeat();
	exist=true;


	this->enemy=enemy;
	this->caster=caster;
}


TornadoSpell::~TornadoSpell()
{
	//unlock the wizards if necessary

	if (wizLock[0])
	{
		caster->unlock();
//		glPrintf("locked\n");
	}
	if (wizLock[1])
	{
		enemy->unlock();
	}
//	glPrintf("tornado deleted");
	windSound.stop();
	mainGM.setWind(1);
}

void TornadoSpell::doStuff()
{
	double oldTimer=timer;
	timer+=speedmod;
	

	//float tpos[3];
	float angle;
	float force;

//	

	decisionTimer-=speedmod;
	if (decisionTimer<=0)
	{
		decisionTimer=400;

		angle=caster->getYRot();

		vect[0]=cos(angle)*0.15;
		vect[1]=sin(angle)*0.15;
	}


	if (timer<800 && tornado!=NULL)
	{
		float * tpos=tornado->getPos();
		mainGM.AIMessage(AIS_TORNADO,tpos);

		pos[0]+=vect[0]*speedmod;
		pos[2]+=vect[1]*speedmod;
		
		float oldHeight=pos[1];
		
		pos[1]=mainGM.getLand()->getYAt(pos[0],pos[2]);
		float seaHeight=mainGM.getSea()->getYAt(pos[0],pos[2]);
		tpos[0]=pos[0];
		tpos[1]=pos[1];
		tpos[2]=pos[2];

		if (timer>150) 
		{
			if (timer<350)
			{
				force=(timer-150)/200.0;
			}
			else force=1;
		}
		moveWizards(caster,enemy,force);

		if (timer<50)
		{
			windSound.setVolume(timer/60.0);
		}

		if (timer>50 && oldTimer<=50)
		{
			mainGM.setWind(1.8);
		}

		if (tpos[1]<seaHeight)
		{
			if (tpos[1]<seaHeight-8) 
			{
				//kill the tornado - it went in the sea
//				glPrintf("tornado in the sea");

				timer=2801; 		
			}
		}

	}

	if (timer>=800)
	{
		if (tornado!=NULL) 
		{
			tornado->stop();
			tornado=NULL;
		}
		windSound.setVolume(0.8-((timer-800)/400.0));

		if (timer>1000 && oldTimer<=1000)
		{
			exist=false;
		}
	}
}


void TornadoSpell::moveWizards(Wizard * wizarda, Wizard * wizardb, float tforce)
{

	Wizard *wiz[2];

	wiz[0]=wizarda;
	wiz[1]=wizardb;

	float v[3];
	float force,dist;

	float len;
	float angle[3];

	float vect[3]={0,0,0};

	float chuckFactor=0;


	for (int a=0;a<2;++a)
	{
		

		v[0]=wiz[a]->phys->pos[0]-pos[0];
		v[1]=wiz[a]->phys->pos[1]-pos[1];
		v[2]=wiz[a]->phys->pos[2]-pos[2];

//		if (v[1]<40) v[1]=0;
//		else v[1]-=40;


		
		dist=lengthOf(v);
		float dist2d=sqrt(v[0]*v[0]+v[2]*v[2]);
		if (dist2d<1) dist2d=1;
		if (dist<1) dist=1;

		if (dist<100)
		{
			force=(tforce*tforce)/(dist*dist);

			if (force>0.05) 
			{
				force=0.05;
				//glPrintf("force is lots");
			}

			if (wiz[a]->downSpeed>-0.4)
			{
				wiz[a]->downSpeed-=25*force*speedmod;
			}
			else
			{
				wiz[a]->downSpeed=-0.4;
			}

			
			if (wiz[a]->isAlive())
			{
				wiz[a]->rise(0.1*speedmod); //rise out of any quicksand
				//(but not if you're a corpse)
			}

			//v[1]=wiz[a]->phys->pos[1]-pos[1];
			//v[1]=0;

			coordsToAngles(v,&len,angle);

			
			float rotBy=(3000*tforce*tforce*tforce)/((dist2d*dist2d));


			if (rotBy>10) rotBy=10;
			if (rotBy<0.01) rotBy=0.01;


			//angle[0]=0;
			angle[1]+=rotBy;
			//angle[2]=0;

			if (angle[1]>=180) angle[1]=angle[1]-360;


			wiz[a]->rotate(rotBy);
			/*
			if (angle[0]==HUGE_VAL || angle[1]==HUGE_VAL)
			{
				
				messageBox("QNAN rotby %f\nwpos%f %f %f\nv %f %f %f\npos %f %f %f",rotBy,
										wiz[a]->phys->pos[0],wiz[a]->phys->pos[1],wiz[a]->phys->pos[2],
										v[0],v[1],v[2],
										pos[0],pos[1],pos[2]);
										
			}*/


			//suction
			len-=dist2d*dist2d*force*speedmod;
			if (len<2) len=2;


			anglesToCoord(angle,len,v);
			
//			wiz[a]->phys->velChange(v,100);


			

/*
			vectorAdd(pos,v,vect); //vect=pos+v
			vectorSubtract(vect,wiz[a]->phys->pos,vect);//vect=vect-wpos
			*/

			for (int b=0;b<3;++b)
			{
				vect[b]=pos[b]+v[b];
				vect[b]=vect[b]-wiz[a]->phys->pos[b];
			}
			
			vect[1]=0;
			if (fabs(vect[0])<10 && fabs(vect[1])<10 && fabs(vect[2])<10)
			{
				wiz[a]->phys->velChange(vect);
			}
			//wiz[a]->phys->vel[1]=force*10;

		}

	}

}