/*
(c)2002 Ben Woodhouse

ben@elf-stone.com
www.elf-stone.com

This code may not be used in a commercial product without my permission.
If you redistribute it, this message must remain intact. 
*/
// Wizard.cpp: implementation of the Wizard class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Wizard.h"
#include "dauntless.h"
#include "spell.h"
#include "fireballSpell.h"
#include "tornadoSpell.h"
#include "quicksandSpell.h"
#include "floodSpell.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


Wizard::Wizard() 
{
	resetFlags();
	
}

Wizard::~Wizard()
{
}

void Wizard::respawn(const float startPos[3])
{
	resetFlags();

	phys->setRealPos(startPos);
	phys->vel[0]=0;
	phys->vel[1]=0;
	phys->vel[2]=0;



	model.reset();
}

void Wizard::reset(const float startPos[3])
{
	resetFlags();
	phys.reset();
	setupPhysics(startPos);
	model.reset();
}
void Wizard::resetFlags()
{
	altitude=0;
	locked=0;
	yrot=0;
	health=1;
	spellSelect=WATER;
	nextDestAction=0;
	alive=true;
	downSpeed=0;

	for (int a=0;a<4;++a)
	{
		spellPower[a]=0.5;
	}
	casting=false;
	sinkDist=0;
	sinking=false;
	onFire=false;
	sunkThisFrame=false;
	netWiz=false;
	drowned=false;
//	justDied=false;
}

void Wizard::setPos(float x, float y, float z)
{
	phys->pos[0]=x;
	phys->pos[1]=y;
	phys->pos[2]=z;
}
void Wizard::init(int texNum, float startPos[3])
{
	/*
	Schematic view

			     Ca
			      ^
			      |
	Drd  <--Dr---St --> SW --> Wa
			      | <-- WS <--
			      |
			      V
			      Di
			      |
			      V
			      De
	*/

	setupPhysics(startPos);
	/*
	if (wizNum==0)
	{
		
		phys=mainGM.physics.createObject(-20,100,0,3.3);
		phys->createChildren(3);
		phys->getChild(0)->setup(-20,28,0,3.2);
		phys->getChild(1)->setup(-20,32,0,3.5);
		phys->getChild(2)->setup(-20,36,0,1);
		phys->getDimensionsFromChildren();
	//	phys->pos[1]=mainGM.getLand()->getYAt(phys->pos[0],phys->pos[2])-getFeetDist();
	}
	else
	{
		phys=mainGM.physics.createObject(-100,30,0,3.3);
		phys->createChildren(3);
		phys->getChild(0)->setup(-100,28,0,3.2);
		phys->getChild(1)->setup(-100,32,0,3.3);
		phys->getChild(2)->setup(-100,36,0,1);
		phys->getDimensionsFromChildren();
	//	phys->pos[1]=mainGM.getLand()->getYAt(phys->pos[0],phys->pos[2])-getFeetDist();
	}*/


	//setup the animation
	model.init();
	model.setFPS(30);
	model.assignAction("stand",0,0,0,0);
	model.linkAction(0,0);
	model.linkAction(0,1);
	model.linkAction(0,4);
	model.linkAction(0,5);
	model.linkAction(0,7);

	model.assignAction("standToWalk",1,15,1,0);
	model.linkAction(1,2);
	model.linkAction(1,3);
	model.linkAction(1,9);

	model.assignAction("walk",16,30,2,0);
	model.linkAction(2,2);
	model.linkAction(2,3);
	model.linkAction(2,5);
	model.linkAction(2,9);

	model.assignAction("walkToStand",31,38,1,0);
	model.linkAction(3,0);

	model.assignAction("cast",39,59,0,1);
	model.linkAction(4,0);
	model.linkAction(4,4);

	model.assignAction("die",60,78,0,-1);
	model.linkAction(5,6);

	model.assignAction("dead",78,78,0,-2);
	model.linkAction(6,6);

	model.assignAction("drown",79,98,-1,0);
	//model.linkAction(7,7);
	model.linkAction(7,8);

	model.assignAction("drowned",98,98,-2,0);
	model.linkAction(8,8);

	model.assignAction("walkCast",100,115,1,1);
	model.linkAction(9,2);
	model.linkAction(9,3);
	model.linkAction(9,9);
	
	model.setDestAction(0);
	model.setAction(0);

	findCastPoint();
	
	this->texNum=texNum;
}

void Wizard::setupPhysics(const float pos_in[3])
{
	phys=mainGM.physics.createObject(pos_in[0],pos_in[1],pos_in[2],3.3);
	phys->createChildren(3);
	phys->getChild(0)->setup(pos_in[0],pos_in[1]-2,pos_in[2],3.2);
	phys->getChild(1)->setup(pos_in[0],pos_in[1]+2,pos_in[2],3.5);
	phys->getChild(2)->setup(pos_in[0],pos_in[1]+6,pos_in[2],1);
	phys->getDimensionsFromChildren();
}

void Wizard::draw()
{
	model.getB3D()->swapTextures(0,texNum);

	float white[4]={0.4,0.4,0.4,1};

	glPushAttrib(GL_LIGHTING_BIT);
	//glMaterialfv(GL_FRONT,GL_DIFFUSE,white);
	glMaterialfv(GL_FRONT,GL_AMBIENT,white);
	//glMaterialfv(GL_FRONT,GL_SPECULAR,white);

	/*
	glPushMatrix();
		glTranslatef(phys->realPos[0],phys->realPos[1],phys->realPos[2]);
		drawSphere(5,5,5);
	glPopMatrix();
*/
	glPushMatrix();
		glTranslatef(phys->realPos[0],getFeetY(),phys->realPos[2]);
		glRotatef(yrot,0,1,0);

		model.draw();
	glPopMatrix();
	glPopAttrib();

	model.getB3D()->swapTextures(0,texNum);
}


void Wizard::doStuff()
{
	if (alive) 
	{
		vx=sin(yrot/rade);
		vz=cos(yrot/rade);

		move();
	}

	//gravity even applies if you're dead
	applyGravity();	

	
	recharge();

	if (onFire) doFire();
}

void Wizard::applyGravity()
{
	//landheight=mainGM.getLand()->getYAt(this->pos[0]+vx,this->pos[2]+vz);
	
	float landheightCenter=mainGM.getLand()->getYAt(phys->pos[0],phys->pos[2]);
	float landheightInFront=mainGM.getLand()->getYAt(phys->pos[0]+vx,phys->pos[2]+vz);

	float landY;
	//pos[0],pos[2]);

	if (landheightCenter>landheightInFront)
	{
		feetY=landheightCenter;
	}
	else
	{
		feetY=landheightInFront;
	}
	landY=feetY+getFeetDist();


	landY-=sinkDist*7;
	float seaHeight=mainGM.getSea()->getYAt(phys->pos[0],phys->pos[2]);

	//drowning
	if ((phys->pos[1])<=seaHeight)
	{
		hit(0.003*speedmod,false);
	}

	if (alive)
	{
		//rise if not in quicksand 
		if (!sinking) rise(0.1*speedmod);
		sinking=false;
	}

	
	//gravity
	downSpeed+=0.02*speedmod;
	phys->pos[1]-=downSpeed*speedmod;

	altitude=phys->pos[1]-landY;

//	if (phys->id==0) glPrint

	if (altitude<2) phys->setFriction(0.97);
	else phys->setFriction(0.983);

	if (altitude<=0)
	{
		phys->pos[1]=landY;
		downSpeed=0;
		altitude=0;
	}

	sunkThisFrame=false;
}




void Wizard::move()
{
	int modelAction=model.getCurrentActionNo();
	int modelFrame=model.getCurrentFrame();

	//distance to move affected by sinkFactor
	float sinkFactor=1-sqrt(sinkDist);

	//turning
	fixAngle(&yrot);
	fixAngle(&desiredRot);
	float toTurn=angleSubtract(desiredRot,yrot);
	float turnAmt=speedmod*5;

	oldToTurn=toTurn;

	if (toTurn>turnAmt)
	{
		yrot+=turnAmt;
	}
	else if (toTurn<-turnAmt)
	{
		yrot-=turnAmt;
	}
	else 
	{
		yrot=desiredRot;
	}

	switch (modelAction)
	{
		case WIZ_START:
			if (fabs(toTurn)<60)
			{
				moveForwards(speedmod*0.27*sinkFactor);
			}
		break;
		case WIZ_WALK:
			if (fabs(toTurn)<60 && altitude<5)
			{
				moveForwards(speedmod*0.42*sinkFactor);
			}
		break;
		case WIZ_STOP:
			if (fabs(toTurn)<60)
			{
				moveForwards(speedmod*0.15*sinkFactor);
			}
		break;
		case WIZ_CAST:
			if (model.getCurrentFrame()>=5 && casting)
			{
				launchSpell();
				casting=false;
				unlock();
				model.setDestAction(nextDestAction);
			}
		break;
		case WIZ_WALKCAST:
			if (fabs(toTurn)<60)
			{
				moveForwards(speedmod*0.3);
			}

			if (model.getCurrentFrame()>=7 && casting)
			{
				launchSpell();
				unlock();
				casting=false;
				model.setDestAction(nextDestAction);
			}
		break;	
	}
}


void Wizard::recharge()
{
	float recharge[4]={0.003,0.0005,0.04,0.00021};
	
	/*
	//cheating 
	if (phys->getID()==0)
	{
		for (int a=0;a<4;++a)
			spellPower[a]=1;

	}*/
	for (int a=0;a<4;++a)
	{
		spellPower[a]+=recharge[a]*speedmod;
		if (spellPower[a]>1) 
		{
			spellPower[a]=1;
		}
	}
}



float Wizard::getFeetDist()
{
	return -phys->getBottomOffset();
}

float Wizard::getAltitude()
{
	return altitude;
}
float Wizard::getFeetY()
{
	return phys->pos[1]+phys->getBottomOffset();
}

bool Wizard::castSpell(float target[3], bool forceSuccess)
{
	if (!alive) 
	{
		//glPrintf("wiz not alive. Failing\n");
		return false;

	}
	bool success=false;
	int act=model.getCurrentActionNo();

	float pos[3];
	getLaunchPos(pos);	


	if (act!=WIZ_CAST && act!=WIZ_WALKCAST)
	{
		if (!casting)
		{
			if (spellPower[spellSelect]==1 || forceSuccess)
			{
				float v[3];
				v[0]=target[0]-pos[0];
				v[1]=target[1]-pos[1];
				v[2]=target[2]-pos[2];

				float angles[3];
				float len;
				coordsToAngles(v,&len,angles);
				desiredRot=angles[1];
				//yRot=angles[1];

				spellTarget[0]=target[0];
				spellTarget[1]=target[1];
				spellTarget[2]=target[2];
				success=true;

//				launchSpell();

				//model.syncDestAction(
				casting=true;
				lock();
			}

			//act like you're casting a spell (even if you're not)
			if (act==WIZ_WALK || act==WIZ_START)
			{
//				model.speedToAction(WIZ_WALKCAST);
				model.setAction(WIZ_CAST);
	
				nextDestAction=WIZ_WALK;
			}
			else
			{
				model.setAction(WIZ_CAST);
				nextDestAction=WIZ_STAND;
//				model.speedToAction(WIZ_CAST);
			}
		}
	}
//	glPrintf("spell result %d\n",success);
	return success;
}


void Wizard::walk()
{
	if (!locked)
	{
		model.setDestAction(WIZ_WALK);
	}
	else
	{
		nextDestAction=WIZ_WALK;
	}
}
void Wizard::stop()
{
	if (!locked) 
	{
		model.setDestAction(WIZ_STAND);
	}
	else 
	{
		nextDestAction=WIZ_STAND;
	}
}
void Wizard::turn(int angle)
{
	if (!locked) 
	{
		if (!casting)
		{
			int modelAction=model.getCurrentActionNo();
			if (modelAction!=WIZ_CAST && modelAction!=WIZ_WALKCAST)
			{

				desiredRot=angle;
			}
		}
	}
}

void Wizard::die()
{
	if (!locked)
	{
		model.setDestAction(WIZ_DEAD);
	}
	else
	{
		nextDestAction=WIZ_DEAD;
	}
}

void Wizard::drown()
{
	if (!locked)
	{
		model.setDestAction(WIZ_DROWN);
		model.setAction(WIZ_DROWN);
	}
	else
	{
		nextDestAction=WIZ_DROWN;
	}
} 




void Wizard::moveForwards(float dist)
{
	/*
	float gradient=mainGM.getLand()->getYAt(phys->pos[0]+vx,phys->pos[2]+vz)-feetY;

glCls();
glPrintf("%f",gradient);
	
	if (gradient>0)
	{
		if (gradient>1) gradient=1;
		dist-=dist*gradient;
	}
*/
	phys->pos[0]+=vx*dist;
	phys->pos[2]+=vz*dist;



	if (sinkDist>0) rise(dist*0.0015); //movement causes rising from quicksand
}

void Wizard::getLaunchPos(float posOut[3])
{
	int frame=model.getRealCurrentFrame();
	float * pos=castPointMesh->frame[frame].ver[0].pos;



	float yrad=yrot/rade;
	posOut[0]=(sin(yrad)*pos[2]+cos(yrad)*pos[0])+phys->pos[0];
	posOut[1]=pos[1]+phys->pos[1]-6.5;
	posOut[2]=(cos(yrad)*pos[2]-sin(yrad)*pos[0])+phys->pos[2];
	
}

void Wizard::findCastPoint()
{
	castPointMesh=NULL;
	castPointMesh=model.getB3D()->findAnimMesh("CastPoint");
	if (castPointMesh==NULL)
	{
		messageBox("castpoint not found");
		return;
	}
	//messageBox("numvers %d",mesh->numVers);
	//castPoint=mesh->ver[0].pos;
}

void Wizard::launchSpell()
{
	Spell * spell=NULL;
	float launchPos[3];
	getLaunchPos(launchPos);
	switch (spellSelect)
	{
		case EARTH:
			spell=new QuicksandSpell(spellTarget,this,enemy);
		break;
		case AIR:
			spell=new TornadoSpell(spellTarget,this,enemy);
		break;
		case FIRE:
			spell=new FireballSpell(spellTarget,launchPos,this,enemy);
		break;
		case WATER:
			spell=new FloodSpell();
		break;
	}
	if (spell!=NULL)
	{
		mainGM.getSpellManager()->registerSpell(spell);
		if (spell->exists()) spellPower[spellSelect]=0;
	}
}

void Wizard::rise(float value)
{
	sinkDist-=value;
	if (sinkDist<=0)
	{
		sinkDist=0;
	}
}

void Wizard::sink(float value)
{
	if (sunkThisFrame) return;

	sunkThisFrame=true;
	sinkDist+=value;
	sinking=true;

	hit(0.002*sinkDist*speedmod,true);
	if (sinkDist>=1) 
	{
		hit(10,true); //drown
	}
	if (sinkDist>=1.62)
	{
		sinkDist=1.62;
	}
}

void Wizard::heal(float amt)
{
	if (health<=0 || health==1) return;
	health+=amt;

	if (health>1)
	{
		health=1;
	}

}

void Wizard::hit(float amt, bool drowning)
{
	if (health<=0) return;
	if (netWiz) 
	{
//		glPrintf("can't hit me! I'm a netwiz\n");
		return; //clients handle their own damage
	}

	health-=amt;

	if (health<=0)
	{
		kill(drowning);
//		model.setDestAction(WIZ_DEAD); //dead
//		nextDestAction=WIZ_DEAD;
	}
}

void Wizard::kill(bool drowning)
{
	if (drowning) drown();
	else die();
	alive=false;
	lock();
	health=0;
	drowned=drowning;
}

void Wizard::doFire()
{
	int firesDead=0;
	//make the fire follow the vertex
	//(be careful with particlesystems, as they delete themselves)
	for (int a=0;a<3;++a)
	{
		if (fire[a].exists)
		{
			if (fire[a].timer>0)
			{
				float firePos[3];
				getVertexPos(fire[a].aMeshNo,fire[a].vertexNo,firePos);
				fire[a].pfire->setPos(firePos);
				fire[a].timer-=speedmod;
			}
			else
			{
				fire[a].exists=false;
				fire[a].pfire->stop();
			}
		}
		else
		{
			++firesDead;
		}
		
	}

	if (firesDead==3) 
	{
		onFire=false;
	}
}

void Wizard::catchFire()
{
	//glPrintf("on fire!");
	if (!onFire)
	{

		onFire=true;

		fire[0].aMeshNo=0;
		fire[1].aMeshNo=0;
		fire[2].aMeshNo=1;
		for (int a=0;a<3;++a)
		{
			fire[a].vertexNo=random( 
				model.getB3D()->getAnim(fire[a].aMeshNo)->getNumVers() 
				);

			fire[a].exists=true;

			float firePos[3];
			getVertexPos(fire[a].aMeshNo,fire[a].vertexNo,firePos);
			
			fire[a].pfire=new ParticleFire(firePos[0],firePos[1],firePos[2],100,true);

			fire[a].pfire->setAcceleration(0.015);
			fire[a].timer=400+random(100)*7;
		}

		doFire();
	}
}

void Wizard::getVertexPos(int animMesh, int v, float * posOut)
{
	AnimMesh * aMesh=model.getB3D()->getAnim(animMesh);
	int frame=model.getRealCurrentFrame();

	float * pos=aMesh->frame[frame].ver[v].pos;

	float rot=yrot;

	float p[3]={10,0,0};


	float yrad=yrot/rade;
	posOut[0]=(sin(yrad)*pos[2]+cos(yrad)*pos[0])+phys->pos[0];
	posOut[1]=pos[1]+phys->pos[1]-6.0;
	posOut[2]=(cos(yrad)*pos[2]-sin(yrad)*pos[0])+phys->pos[2];
}