// Wizard.h: interface for the Wizard class.
//
//////////////////////////////////////////////////////////////////////


#if !defined(AFX_WIZARD_H__B22D40CB_9C7A_4F71_BF3C_EE66A6DA286E__INCLUDED_)
#define AFX_WIZARD_H__B22D40CB_9C7A_4F71_BF3C_EE66A6DA286E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define WIZ_STAND 0
#define WIZ_START 1
#define WIZ_WALK 2
#define WIZ_STOP 3
#define WIZ_CAST 4
#define WIZ_DIE 5
#define WIZ_DEAD 6
#define WIZ_DROWN 7
#define WIZ_DROWNED 8
#define WIZ_WALKCAST 9

#define EARTH 0
#define AIR 1
#define FIRE 2
#define WATER 3

#include "ActionManager.h"
#include "PhysicsSystem.h"
#include "ParticleFire.h"


struct WizFire
{
	WizFire(){timer=0;}
	int vertexNo;
	int aMeshNo;
	ParticleFire * pfire;
	double timer;
	bool exists;
};

class NetPlayer;
class UserPlayer;
class Wizard 
{
public:
	friend NetPlayer;
	friend UserPlayer;
	Wizard();
	virtual ~Wizard();
	//bool setB3D(ActionManager * model){this->model=model;}
	void reset(const float startPos[3]); 
	void respawn(const float startPos[3]);
	void init(int texNum, float startPos[3]); //only init once!
	
	void draw();
	void doStuff();

	void drawSpells();


	void setB3D(B3D * b3d){this->model.setB3D(b3d);}
	void setPos(float x, float y, float z);
	void setDestAction(int aID){model.setDestAction(aID);}
	void setAction(int aID){model.setAction(aID);}
	void setupPhysics(const float pos_in[3]);

	bool castSpell(float target[3],bool forceSuccess=false); //returns true if successful
	void walk();
	void stop();
	void turn(int angle);
	void kill(bool drowning);

	void die();
	void drown();
	int getCurrentSpell(){return spellSelect;}
	void setCurrentSpell(int spellNo){spellSelect=spellNo;}

	float getYRot(){return yrot;}

	int getCurrentAction(){return model.getAction();}

	void moveForwards(float dist);
	void getLaunchPos(float posOut[3]);
	void destroySpells();
	PhysPtr phys;

	float * getPos(){return phys->getRealPos();}
	float getFeetDist();
	float getFeetY();

	float getSpellPower(int spell){return spellPower[spell];}
	
	float getHealth(){return health;}
	//float setHealth(float health){health=health_in; if (health<=0) hit(0.1,false);}
	void hit(float amt, bool drowning);
	void heal(float amt);


	void setEnemy(Wizard * enemy){this->enemy=enemy;}
	int getPhysID(){return phys->getID();}
	void lock(){++locked;}
	void unlock()
	{
		--locked;
		if (locked<0) locked=0;
	}
	void sink(float value);
	void rise(float value);


	float downSpeed;
	void rotate(float amt){desiredRot+=amt;}

	void doFire();
	void catchFire();

	Wizard * getEnemy(){return enemy;}

	float getAltitude();
	bool isAlive(){return alive;}
	void setNetWiz(bool val){netWiz=val;}

protected:
	//bool justDied;
	bool netWiz;
	void resetFlags();
	void launchSpell();
	void findCastPoint();
	void recharge();
	void move();
	void applyGravity();
	void getVertexPos(int animMesh, int v, float * posOut);


	int nextDestAction;

	float feetY;
	//float land
	float sinkDist;
	bool sinking;
	bool drowned;


	float yrot;
	float oldToTurn;
	float desiredRot;
	float vx,vz;
	bool sunkThisFrame;

	bool onFire;
	WizFire fire[3];

	float health;
	int spellSelect;
	 
	ActionManager model; 
	int texNum;

	float * castPoint;
	AnimMesh * castPointMesh;

	float spellPower[4];
	float spellTarget[3];

	bool casting;
	bool alive;
	int locked;

	float altitude;
	Wizard * enemy;
};

#endif // !defined(AFX_WIZARD_H__B22D40CB_9C7A_4F71_BF3C_EE66A6DA286E__INCLUDED_)
