/*
Camera.cpp

Version 1.0

(c)2002 Ben Woodhouse

ben@elf-stone.com
www.elf-stone.com

This code may not be used in a commercial product without my permission.
If you redistribute it, this message must remain intact. 
*/

//Memory leak detection
/*
#ifdef _DEBUG
#include <crtdbg.h>
#define new new(_NORMAL_BLOCK,__FILE__, __LINE__)
#endif  
*/
#include "stdafx.h"

Viewport::Viewport()
{
	aspect=3.0/2.0;
	viewangle=VIEWANGLE;
 	nearclip=NEARCLIP;
	farclip=FARCLIP;
}

void Viewport::unproject(int x,int y, 
						double * outx, double * outy, double * outz)
{
	double modelview[16];
	double projection[16];
	int viewport[4];
 
	float z;
	glReadPixels(x,y,1,1,GL_DEPTH_COMPONENT,GL_FLOAT,&z);

	glGetDoublev(GL_MODELVIEW_MATRIX,modelview);
	glGetDoublev(GL_PROJECTION_MATRIX,projection),
	glGetIntegerv(GL_VIEWPORT,viewport);

	gluUnProject(x,y,(double)z,modelview,projection,
						viewport,outx,outy,outz);
}
 
void ThreeDegreesCamera::setViewMatrix()
{ 
	
	TMatrix4 translateMat2;
	loadIdentity(translateMat2);

	translateMat2[12]=-pos[0];
	translateMat2[13]=-pos[1];
	translateMat2[14]=-pos[2];

	glLoadIdentity();
	multMatrixTo(rotateMat,translateMat2,modelView);

  glLoadMatrixf(modelView);
  
}

ThreeDegreesCamera::ThreeDegreesCamera()
{
	for (int a=0;a<2;++a) 
	{
		
		rot[0] = initRot0;
		rot[1] = initRot1;
		rot[2] = initRot2;  

		loadIdentity(modelView);

		pos[0]=initPos0;
		pos[1]=initPos1;
		pos[2]=initPos2;
	}
	currentVP=this;
}	

void ThreeDegreesCamera::getCameraPos(float * posOut)
{
	posOut=pos;
}

void ThreeDegreesCamera::rotateByRotMat()
{
	glLoadMatrixf(rotateMat);
}

void ThreeDegreesCamera::rotateByInvRotMat()
{
	glMultMatrixf(invRotateMat);
}




OrbitalCamera::OrbitalCamera()
{
	pos[0]=0;
	pos[1]=25;
	pos[2]=0;

	rot[0]=45;
	rot[1]=45;
	rot[2]=0;
	radius=100;
}

void OrbitalCamera::getCameraNorm(float *normOut)
{
	TMatrix4 mv;
	glGetFloatv(GL_MODELVIEW_MATRIX, mv);
	
	normOut[0]=-mv[2];
	normOut[1]=-mv[6];
	normOut[2]=-mv[10];
}

void OrbitalCamera::setViewMatrix()
{
	glLoadIdentity();
	glTranslatef(0,0,-radius);

	glRotatef(rot[0],1,0,0);
	glRotatef(rot[1],0,1,0);
	glRotatef(rot[2],0,0,1);

	glTranslatef(-pos[0],-pos[1],-pos[2]);
//	glTranslatef(0,0,0);

/*
	glCls();
	glPrintf("radius %f\n",radius);
	glPrintf("rot %f %f %f\n",rot[0],rot[1],rot[2]);
	glPrintf("pos %f %f %f\n",pos[0],pos[1],pos[2]);
	*/
	//glTranslatef(-pos[0],-pos[1],-pos[2]);

}

void OrbitalCamera::fixAngles()
{
	int a;
	for (a=0;a<2;++a)
	{
		while (rot[1]>180)   rot[1]=rot[1]-360;
		while (rot[1]<=-180) rot[1]=rot[1]+360;
		while (rot[0]>180)   rot[0]=rot[0]-360;
		while (rot[0]<=-180) rot[0]=rot[0]+360;
	}
}

void OrbitalCamera::getCameraPos(float * posOut)
{
	float vector[3];

	float negRot[3];

	negRot[0]=-rot[0];
	negRot[1]=-rot[1];
	negRot[2]=-rot[2];
	anglesToCoord(negRot,radius,vector);
	
	posOut[0]=vector[0]+pos[0];
	posOut[1]=vector[1]+pos[1];
	posOut[2]=vector[2]+pos[2]; 

}
/*
void OrbitalCamera::translateByPos()
{
}*/
void OrbitalCamera::rotateByRotMat()
{
	glRotatef(rot[0],1,0,0);
	glRotatef(rot[1],0,1,0);
	glRotatef(rot[2],0,0,1);
}

void OrbitalCamera::rotateByInvRotMat()
{
	glRotatef(-rot[2],0,0,1);
	glRotatef(-rot[1],0,1,0);
	glRotatef(-rot[0],1,0,0);
}