/*
LinkedList.h

V1.4
(c)2001 Ben Woodhouse


//some array functionality added to CLinkedList. CLinkedListP still at 1.1 spec
*/

#ifndef LinkedListH
#define LinkedListH


#include <exception>
#include <iostream.h>


template <class type>
class CNode
{
	public:
        CNode();
		CNode(type * datp_in);
		~CNode();
		void deleteNext();
//        void insertAfter(type * datp_in);

		type dat;

		CNode <type> * next;
};



//standard linked lists
template <class type>
class CLinkedList
{
	public:
		CLinkedList();

		~CLinkedList();

		void clear();
		void deleteElement(int e);
		void deleteNode(CNode<type> * nodeDel,CNode<type> * nodePrev);

		int getMemSize();


		void insert(type * datp_in);
		void insertAtEnd(type * datp_in); //call with NULL to create an empty node at end

		void create();
		void createAtEnd();
		CNode <type> * getElement(int e);

		type * getElementDat(int e);

		void loadData(type * array_in, int elements);//adds a load of data to the list

		void resetCurrent();
		type * getCurrentInc();
		type * getCurrent();

		bool incCurrent();


		CNode <type> * head;
		CNode <type> * tail;
		CNode <type> * current;
		int numNodes;

};



/*
//pointer linked lists
template <class type> 
class CLinkedListP
{
	public:
		CLinkedListP()
		{
			head=NULL;
			//memset
			numNodes=0;
		}

		~CLinkedListP()
		{
			if (head!=NULL)
				delete head;
		}

		void deleteNode(CNode<type *> * nodeDel,CNode<type *> * nodePrev)
		{
			CNode <type *> * temp;
			if (nodeDel==head)
			{
				temp=head->next;
				head->next=NULL;
				delete head;
				head=temp;
			}
			else
			{
				nodePrev->deleteNext();
			}
			--numNodes;
		}

		//this function will only work if you have allocated the elements of an array in order
		//for instance, if you used loadPointers
		inline CNode <type *> * getNode(int element) 		
		{
			int elementReverse=(numNodes-1)-element;

			printf("%d\n",numNodes);
			//return &head[elementReverse];
			CNode <type *> * current=head;

			for (int a=0;a<elementReverse;++a)
			{
				current=head->next;
			}
			return current;
			//return &head[];
		}



		int getMemSize()
		{
			return numNodes*sizeof(*head);
		}

		//adds a load of pointers to the list
		void insert(type * datp_in)
		{
			CNode <type *> * newhead=new CNode<type *>(&datp_in);
			if (head!=NULL)
			{
				newhead->next=head;
			}
			head=newhead;
			++numNodes;

		}


		void loadPointers(type * array_in, int elements)
		{
			//build the linked lists and write the IDs to the tri and ver arrays
			for (int a=0;a<elements;++a)
			{
				insert(&array_in[a]);
			}
		}
		int numNodes;
		CNode <type *> * head;
};







*/

//---------------------------------------------------------------------
//
//  CNode
//
//---------------------------------------------------------------------
template <class type>
CNode <type>::    CNode()
{
    next=NULL;
}

template <class type>
CNode <type>::    CNode(type * datp_in)
{
    next=NULL;
//    dat=*datp_in;
	 memcpy(&dat,datp_in,sizeof(type));
}

template <class type>
CNode <type>::  ~CNode()
{
//          messageBox("Deleting node");
    if (next!=NULL) delete next;
}

template <class type>
void CNode <type>:: deleteNext()
{
    CNode <type> * nextNext=next->next;
    next->next=NULL;
    delete next;
    next=nextNext;
}
/*
template <class type>
void CNode <type>::insertAfter(type * datp_in)
{
    CNode <type> * nextNext=next;
    next=new CNode <type>;
    next->next=nextNext;
    next->dat=datp_in;
}
*/











//---------------------------------------------------------------------
//
//  CLinkedList
//
//---------------------------------------------------------------------
template <class type>
CLinkedList <type>::CLinkedList()
{
    head=NULL;               
    tail=NULL;
    current=NULL;
    numNodes=0;
}

template <class type> CLinkedList <type>::~CLinkedList()
{
//            messageBox("Deleting list");
    if (head!=NULL)
        delete head;
}
 
template <class type>
void CLinkedList <type>::   clear()
{
    if (head!=NULL)
        delete head;

    head=NULL;
    tail=NULL;
    current=NULL;
    numNodes=0;
}

template <class type>
void CLinkedList <type>::   deleteElement(int e)
{
    CNode <type> * pre;

    if (e>0)
    {
        pre=getElement(e-1);
//               messageBox(pre->dat.name);
//               messageBox(pre->next->dat.name);

        deleteNode(pre->next,pre);
    }
    else
    {
        deleteNode(head,NULL);
    }
}



template <class type>
void CLinkedList <type>::   deleteNode(CNode<type> * nodeDel,CNode<type> * nodePrev)
{
    CNode <type> * temp;
    if (nodeDel==current)
    {
        current=current->next;
    }
    if (nodeDel==head)
    {
        temp=head->next;
        head->next=NULL;
        delete head;
        head=temp;
        if (nodeDel==tail)
        {
            //find a new tail
            tail=temp;
        }

    }
    else
    {
        if (nodeDel==tail)
        {
            //find a new tail
            tail=nodePrev;
        }


        nodePrev->deleteNext();

    }


    --numNodes;
}

template <class type>
int CLinkedList <type>::    getMemSize()
{
    return numNodes*sizeof(*head);
}


template <class type>
void CLinkedList <type>::   insert(type * datp_in)
{
	CNode <type> * newhead;

	//create a new head node
	if (datp_in==NULL)
	{
		//if datp_in is null then create an empty node
		newhead=new CNode<type>;
	}
   else newhead=new CNode<type>(datp_in);


	 //if there is a head already, make newhead's next pointer point to it
    if (head!=NULL)
    {

        newhead->next=head;
    }
	 //otherwise, make the tail of the list point to the newhead
    else
    {
        tail=newhead;

		  //a new list has been started, so make the current pointer point to the
		  //first element
	     current=newhead;
    }
	 //make the head of the list point to the newhead
    head=newhead;

    ++numNodes;

}

template <class type>
void CLinkedList <type>::   insertAtEnd(type * datp_in) //call with NULL to create an empty node at end
{
    CNode <type> * newtail;

    if (datp_in!=NULL)
        newtail=new CNode<type>(datp_in);
    else
        newtail=new CNode<type>;

    if (tail!=NULL)
    {
        tail->next=newtail;
    }
    else
    {
        head=newtail;
        current=newtail;
    }
    tail=newtail;
    ++numNodes;
}

template <class type>
void CLinkedList <type>::   create()
{
	insert(NULL);
}

template <class type>
void CLinkedList <type>::   createAtEnd()
{
    insertAtEnd(NULL);
}

template <class type>
CNode <type> * CLinkedList <type>:: getElement(int e)
{
    CNode <type> * p=head;

    for (int a=0;a<e;++a)
    {
        p=p->next;
    }
    return p;
}

template <class type>
type * CLinkedList <type>:: getElementDat(int e)
{
    CNode <type> * p=head;

    for (int a=0;a<e;++a)
    {
        p=p->next;
    }
    return &p->dat;
}

//adds a load of data to the list
template <class type>
void CLinkedList <type>::   loadData(type * array_in, int elements)
{
    //build the linked lists and write the IDs to the tri and ver arrays
    for (int a=0;a<elements;++a)
    {
        insert(&array_in[a]);
    }
}

template <class type>
void CLinkedList <type>::   resetCurrent()
{
    current=head;
}

template <class type>
type * CLinkedList <type>:: getCurrentInc()
{
    type * rv=NULL;
    if (current!=NULL)
    {
        rv=&current->dat;
        current=current->next;
    }

    return rv;
}

template <class type>
type * CLinkedList <type>::getCurrent()
{
    if (current!=NULL)
        return &current->dat;

    return NULL;
}

template <class type>
bool CLinkedList <type>::  incCurrent()
{
    if (current!=NULL)
    {
        current=current->next;
        return true;
    }
    return false;
}
/*
bool CLinkedList::incCurrent();
{
    if (current!=NULL)
    {
        current=current->next;
        return true;
    }
    return false;
}
*/



// Example of the template keyword


#endif
