/*
Matrix.cpp
Quaternion and matrix operations
Version 1.0


(c)2002 Ben Woodhouse

ben@elf-stone.com
www.elf-stone.com

This code may not be used in a commercial product without my permission.
If you redistribute it, this message must remain intact. 



The matrices used are compatible with openGL matrices and therefore are turned on their sides
*/

#include "stdafx.h"

TMatrix4 Identity4={	1,0,0,0,
							0,1,0,0,
							0,0,1,0,
							0,0,0,1
							};

TMatrix3 Identity3={	1,0,0,
							0,1,0,
							0,0,1
							};

void copyMatrix(TMatrix4 src, TMatrix4 dst)
{
	for (int a=0;a<16;++a)
	{
		dst[a]=src[a];
	}

}

void transposeMatrix(TMatrix4 m)
{
	TMatrix4 temp;
	for (int a=0;a<16;++a)
	{
		temp[a]=m[a];
	}
	m[0]=temp[0];
	m[1]=temp[4];
	m[2]=temp[8];
	m[3]=temp[12];
	
	m[4]=temp[1];
	m[5]=temp[5];
	m[6]=temp[9];
	m[7]=temp[13];

	m[8]= temp[2];
	m[9]= temp[6];
	m[10]=temp[10];
	m[11]=temp[14];

	m[12]=temp[3];
	m[13]=temp[7];
	m[14]=temp[11];
	m[15]=temp[15];
}

void loadIdentity(TMatrix4 m)
{
	TMatrix4 lIdentity4={	1,0,0,0,
								0,1,0,0,
								0,0,1,0,
								0,0,0,1
								};

	for (int a=0;a<16;++a)
		m[a]=lIdentity4[a];
}

void translateMatrix(TMatrix4 m,float trans[3])
{
//	int a,b;

	TMatrix4 transMat={
							1,0,0,0,
							0,1,0,0,
							0,0,1,0,
							0,0,0,1
						};


	transMat[12]=trans[0];
	transMat[13]=trans[1];
	transMat[14]=trans[2];


	multMatrixBy(m,transMat);
}

//multiplies matrixa by matrixb. Stores result in matrixDest
void multMatrixTo(TMatrix4 matrixa, TMatrix4 matrixb, TMatrix4 matrixDest)
{
	int x,y;
	int mpos;

	for (y=0;y<4;++y)
	{
		for (x=0;x<4;++x)
		{
			mpos=x+y*4;

			//opengl matrices are backwards
			matrixDest[mpos]=
							matrixb[y*4]*matrixa[x]+
							matrixb[y*4+1]*matrixa[x+4]+
							matrixb[y*4+2]*matrixa[x+8]+
							matrixb[y*4+3]*matrixa[x+12];
		}
	}


}

//multiplies matrixa by matrixb. Stores result in matrixa
void multMatrixBy(float *matrixa, float matrixb[16])
{
	int x,y;
	int mpos;
	float tempMat[16];

	for (y=0;y<4;++y)
	{
		for (x=0;x<4;++x)
		{
			mpos=x+y*4;

			//opengl matrices are backwards
			tempMat[mpos]=
							matrixb[y*4]*matrixa[x]+
							matrixb[y*4+1]*matrixa[x+4]+
							matrixb[y*4+2]*matrixa[x+8]+
							matrixb[y*4+3]*matrixa[x+12];
		}
	}

	for (x=0;x<16;++x)
	{
		matrixa[x]=tempMat[x];
	}

}

void displayMatrix4(TMatrix4 m)
{
	for (int a=0;a<16;++a)
	{
		if (a>0 && ((a%4)==0))
			glPrintf("\n");

		glPrintf("%5f ",m[a]);
	}
	glPrintf("\n");
}

void axisAngleToRotMatrix(float angle, float axis[3], TMatrix4 rotMat)
{
	TQuaternion quat;

	quat_convertFromAxisAngle(angle,axis,quat);
	quat_convertToRotMatrix(quat,rotMat);	
}

void quat_multiplyBy(TQuaternion quata, TQuaternion quatb)
{
	TQuaternion temp;


	/*
   (A*B):h = A:h*B:h - A:i*B:i - A:j*B:j - A:k*B:k 
   (A*B):i = A:h*B:i + A:i*B:h + A:j*B:k - A:k*B:j 
   (A*B):j = A:h*B:j - A:i*B:k + A:j*B:h + A:k*B:i 
   (A*B):k = A:h*B:k + A:i*B:j - A:j*B:i + A:k*B:h 

	w=w1w2 - x1x2 - y1y2 - z1z2
	x = w1x2 + x1w2 + y1z2 - z1y2
	y = w1y2 + y1w2 + z1x2 - x1z2
	z = w1z2 + z1w2 + x1y2 - y1x2 
	*/
	temp[0] = quata[0]*quatb[0] - quata[1]*quatb[1] - quata[2]*quatb[2] - quata[3]*quatb[3];
	temp[1] = quata[0]*quatb[1] + quata[1]*quatb[0] + quata[2]*quatb[3] - quata[3]*quatb[2];
	temp[2] = quata[0]*quatb[2] - quata[1]*quatb[3] + quata[2]*quatb[0] + quata[3]*quatb[1];
	temp[3] = quata[0]*quatb[3] + quata[1]*quatb[2] - quata[2]*quatb[1] + quata[3]*quatb[0];
	
	memcpy((void *)quata,(void *)temp,sizeof(float)*4);

}
void quat_convertFromAxisAngle(float angle, float axis[3], TQuaternion quat)
{
	float sin_a=sin(angle/2);
	float cos_a=cos(angle/2);

	quat[0]=cos_a;
	quat[1]=sin_a*axis[0];
	quat[2]=sin_a*axis[1];
	quat[3]=sin_a*axis[2];

	if (axis[0]==0 && axis[1]==0 && axis[2]==0)
	{
		quat[0]=1;
		quat[1]=0;
		quat[2]=0;
		quat[3]=0;
	}

	quat_normalize(quat);

}

void quat_normalize(TQuaternion quat)
{
	float length=quat_getLength(quat);
	
	for (int a=0;a<4;++a)
	{
		quat[a]/=length;
	}
}

float quat_getLength(TQuaternion quat)
{
	return sqrt(quat[0]*quat[0] + quat[1]*quat[1] + quat[2]*quat[2] + quat[3]*quat[3]);
}

void quat_convertToRotMatrix(TQuaternion quat, TMatrix4 rotMat)
{
	float xx=quat[1]*quat[1];
	float xy=quat[1]*quat[2];
	float xz=quat[1]*quat[3];
	float xw=quat[1]*quat[0];

	float yy=quat[2]*quat[2];
	float yz=quat[2]*quat[3];
	float yw=quat[2]*quat[0];

	float zz=quat[3]*quat[3];
	float zw=quat[3]*quat[0];

	/*
Matrix =  [ 1 - 2yy - 2zz    2xy - 2wz      2xz + 2wy
              2xy + 2wz    1 - 2xx - 2zz    2yz - 2wx
              2xz - 2wy      2yz + 2wx    1 - 2xx - 2yy ]

	*/
	rotMat[0] =1 - 2*(yy+zz);
	rotMat[1] =    2*(xy-zw);
	rotMat[2] =    2*(xz+yw);

	rotMat[4] =    2*(xy+zw);
	rotMat[5] =1 - 2*(xx+zz);
	rotMat[6] =    2*(yz-xw);

	rotMat[8] =    2*(xz-yw);
	rotMat[9] =    2*(yz+xw);
	rotMat[10]=1 - 2*(xx+yy);

	rotMat[3]=rotMat[7]=rotMat[11]=rotMat[12]=rotMat[13]=rotMat[14]=0;
	rotMat[15]=1;
	/*
	rotMat[0] =1 - 2*(yy+zz);
	rotMat[1] =    2*(xy+zw);
	rotMat[2] =    2*(xz-yw);

	rotMat[4] =    2*(xy-zw);
	rotMat[5] =1 - 2*(xx+zz);
	rotMat[6] =    2*(yz+xw);

	rotMat[8] =    2*(xz+yw);
	rotMat[9] =    2*(yz-xw);
	rotMat[10]=1 - 2*(xx+yy);

	rotMat[3]=rotMat[7]=rotMat[11]=rotMat[12]=rotMat[13]=rotMat[14]=0;
	rotMat[15]=1;
	*/
}

void rotateByRotMatrix(float point[3],TMatrix4 rotMat)
{
	TMatrix4 m;
	
	m[0]=rotMat[0]*point[0];
	m[1]=rotMat[1]*point[0];
	m[2]=rotMat[2]*point[0];

	m[4]=rotMat[4]*point[1];
	m[5]=rotMat[5]*point[1];
	m[6]=rotMat[6]*point[1];

	m[8]=rotMat[8]*point[2];
	m[9]=rotMat[9]*point[2];
	m[10]=rotMat[10]*point[2];

	point[0]=m[0]+m[4]+m[8];
	point[1]=m[1]+m[5]+m[9];
	point[2]=m[2]+m[6]+m[10];
}

