/*
menu.h

Mirage Menus

(c)2000 Ben Woodhouse

Version 1.11
*/

#if     _MSC_VER > 1000
#pragma once
#endif

#ifndef _INC_MENU
#define _INC_MENU

#define MENU_MENU 0
#define MENU_EXIT 1
#define MENU_SLIDER 2
#define MENU_SWITCH 3
#define MENU_THING 4
#define MENU_DIVIDER 5

typedef void(*changeFunc_t)(int=NULL);


	
class miMenu_c
{
	public:
		void setTitle(char * nameIn);
		miMenu_c (char * namein);
		miMenu_c ();
		~miMenu_c(void);


		void execute();

		void init();
		void display(void);					//call with parent menu 
											//pointer or NULL if 
											//called from the game
		void exitMenus();
		virtual void postInit(){}

		void addItem(	char * name_in, 
						int type_in,
						changeFunc_t onUpdate_in=NULL,
						int initValue=NULL,
						char * text0=NULL, 
						char * text1=NULL,  
						char * text2=NULL,
						char * text3=NULL);  //adds an item to a menu

		void KeysIn();

		void escapeFunc();



 		class item_c
		{
			public:
//				changeFunc_t hi;
				item_c(){active=true;}
	//			item_c(char * name_in);
				virtual void display(){}
				virtual void slide(bool right){}  
				virtual int getValue(){return NULL;}
				virtual int getType(){return NULL;}
				void setValue(int val){value=val;}
				virtual void reset(){changed=false;}
				void setFunc(changeFunc_t onUpdate_inn);
				void onUpdatef()
				{
					if (hi!=NULL) 
						(*hi)(value);
				}
				bool getChanged(){return changed;}
			
			protected:
				char name[20];
				int type;
				bool changed;
				bool active;
				int value;
				changeFunc_t hi;


		};
		class itemSwitch_c: public item_c
		{
			public: 
				itemSwitch_c(char * name_in,int initValue_in) 
				{
					sprintf(name,"%s",name_in);
					value=initValue_in;
					numOptions=0;
				} 
				int getType(){return MENU_SWITCH;}
				void display()
				{
 					glPrintf("%-16s %s\n",name, optionText[value]);
				}
				void setOptionText(char * text_in)
				{
					sprintf(optionText[numOptions],"%s",text_in);
					++numOptions;
				}
				void slide(bool right=true)
				{
					changed=true;

					if (right)
					{
						value++;
						if (value>=numOptions) value=0;
					}
					else
					{
						value--;
						if (value<0) value=numOptions-1;
					}
					onUpdatef();
				}
				int getValue()
				{
					return value;
				}
			private:
				int numOptions;
				char optionText[4][20]; //use if necessary or leave blank
		};

		class itemThing_c: public item_c
		{
			public: 
				itemThing_c(char * name_in) 
				{
					sprintf(name,"%s",name_in);
					changed=false;
				} 
				int getType(){return MENU_THING;}
				void display()
				{
 					glPrintf("%-16s\n",name);
				}

				void slide(bool right=true)
				{
					changed=true;
					onUpdatef();
				}
			private:
		};

		class itemSlider_c: public item_c
		{
			public: 
				itemSlider_c(char * name_in,int initValue_in) 
				{
					sprintf(name,"%s",name_in);
					changed=false;
					value=initValue_in;
				} 
				int getType(){return MENU_SLIDER;}
				void display()  
				{
					int a,b;
 					glPrintf("%-16s ",name);
					for (a=0;a<value;++a)
					{
						glPrintSymbol(2);					
					}
					glPrintSymbol(3);
					for (b=a+1;b<7;++b)
					{
						glPrintSymbol(2);					
					}
					glPrintf("\n");
				}

				void slide(bool right)
				{
					changed=true;
					if (right)
					{
						value++;
						if (value>6) value=6;
					}
					else
					{
						value--;
						if (value<0) value=0;
					}
					onUpdatef();
				}

				int getValue()
				{
					return value;
				}
			private:
		};

		class itemDivider_c: public item_c
		{
		public:
			itemDivider_c(char * name_in)
			{
				sprintf(name,"%s",name_in);
			}
			int getType(){return MENU_DIVIDER;}
			void display()
			{
 					glPrintf("%-16s\n",name);
			}
		private:
		};
		item_c * item_p[20];

	private:
		char title[50];
		int numItems;
		int selector;
		miMenu_c * parent;
		changeFunc_t escFunc;
};

extern miMenu_c * currentMenu;

#endif