/*
sound.cpp


(c)2002 Ben Woodhouse

ben@elf-stone.com
www.elf-stone.com

This code may not be used in a commercial product without my permission.
If you redistribute it, this message must remain intact. 

Version 1.6

updated for Direct sound 8, and cleaned up (slightly)
*/

#include "stdafx.h"




//globals


LPDIRECTSOUND lpDirectSound=0;	//holds global direct sound object
LPDIRECTSOUNDBUFFER primebuf=0;	//primary buffer
UINT numSounds=0;
BOOL quality=LOW_QUALITY;
Sound soundst[MAX_SOUNDS];
float masterVolume=1.0;




Sound * loadNewSound(const char * filename, int numChannels)
{
	Sound * sound=getNewSound();

	if (sound==NULL) return NULL;

	if (sound->load(filename,numChannels))
	{
		return sound;
	}

	return NULL;
}



Sound * getNewSound()
{
	for (int a=0;a<MAX_SOUNDS;++a)
	{
		if (!soundst[a].exists())
		{
			//soundst[a].setExists();
			return &soundst[a];
		}
	}

	return NULL;
}


//functions



BOOL sound_initialize(void)
{
	
	if (!music.init(&lpDirectSound))
		return FALSE;

	//set cooperative level
	if (FAILED(lpDirectSound->SetCooperativeLevel(ghWnd, DSSCL_EXCLUSIVE)))
	{
		glPrintf("Direct Sound cooperative level ERROR\n");

		lpDirectSound->Release();
		return FALSE;
	}

	//set up the primary buffer

    DSBUFFERDESC dsbdesc; //buffer description 

    ZeroMemory(&dsbdesc, sizeof(DSBUFFERDESC));
    dsbdesc.dwSize = sizeof(DSBUFFERDESC);
    dsbdesc.dwFlags = DSBCAPS_PRIMARYBUFFER;

    if FAILED(lpDirectSound->CreateSoundBuffer(&dsbdesc, &primebuf, NULL))
	{
		glPrintf("DS: failed to set up primary buffer\n");
        return FALSE;
	}

	int freq=22050;
	int numChannels=2;
	int numBits=16;
	int bytesPerSec=freq*numChannels*(numBits>>3);
	int blockAlign=numChannels*numBits/8;
	WAVEFORMATEX wfx ={WAVE_FORMAT_PCM, numChannels, freq, bytesPerSec, blockAlign, numBits, 0};

	if (FAILED( primebuf->SetFormat(&wfx) )) 
	{
		return FALSE;
	}

//	primebuf->SetVolume(-10000);

 
	return TRUE;
}

BOOL sound_uninitialize(void)
{
	for (int a=0;a<numSounds;++a)
	{
		soundst[a].release();
	}
	if (lpDirectSound)
	{
		lpDirectSound->Release();
		UO.sound=0;
	}
	else
	{
		glPrintf("Direct Sound release ERROR\n");
		return FALSE;
	}
	return TRUE;
	
}

/////////////////////////////////////////////////////////
//							SOUND
/////////////////////////////////////////////////////////
Sound::Sound()
{
	exist=FALSE;
	playing=false;
	buf=NULL;
	volume=1.0;
}

void Sound::updateVolume()
{
	LONG attenuation=-10000*(float)(1.0-masterVolume*volume);
	buf->SetVolume(attenuation);
}

BOOL Sound::load(const char * soundFN, int channels)
{
	//loads an 8-bit 22KHZ mono wave sound file, creates a buffer for it
	FILE *wavein;
	int a;

	WAVEFORMATEX wfx;// ={WAVE_FORMAT_PCM, 1, 22050, 44100, 2, 16, 0};
    // wFormatTag, nChannels, nSamplesPerSec, nAvgBytesPerSec,
    // nBlockAlign, wBitsPerSample, cbSize

	ZeroMemory(&wfx,sizeof(WAVEFORMATEX));
	wfx.wFormatTag=WAVE_FORMAT_PCM;
	wfx.nChannels=channels;
	wfx.nSamplesPerSec=22050;
	wfx.wBitsPerSample=16;
	wfx.nBlockAlign=channels*wfx.wBitsPerSample/8;
	wfx.nAvgBytesPerSec=wfx.nBlockAlign*wfx.nSamplesPerSec;
	wfx.cbSize=0;
	


	if ( (wavein=fopen(soundFN,"rb"))==NULL ) 
	{
		glPrintf("Error opening sound file\n");
		return FALSE;
	}

	//skip past the header
	if (!stream_search(wavein,"data"))
	{
		glPrintf("%s: FILE IS NOT WAVE FILE\n",soundFN);
		fclose(wavein);
		return FALSE;
	}


    DSBUFFERDESC dsbdesc; //buffer description 

	//set up buffer
    ZeroMemory(&dsbdesc, sizeof(DSBUFFERDESC));
    dsbdesc.dwSize = sizeof( DSBUFFERDESC ); 
    dsbdesc.dwFlags = DSBCAPS_STATIC | DSBCAPS_CTRLVOLUME ; 
    dsbdesc.dwBufferBytes = _getw(wavein); //reads size of data chunk and advances stream
    dsbdesc.lpwfxFormat = &wfx;

   //create the buffer 
    if ( FAILED( lpDirectSound->CreateSoundBuffer( 
            &dsbdesc, &buf, NULL ) ) )   
	{
		glPrintf("DS: Failed to create buffer");
		fclose(wavein);
		return FALSE;

	}

	//lock the buffer
    LPVOID soundInStart; 
    DWORD  soundBytes; 
	

	if (FAILED(buf->Lock( 
								0,              // Offset of lock start. 
								0,              // Size of lock; ignored in this case.
								&soundInStart,	// Address of lock start (OUT)
								&soundBytes,    // Number of bytes locked (OUT). 
								NULL,           // Address of wraparound start. 
								NULL,           // Number of wraparound bytes. 
								DSBLOCK_ENTIREBUFFER 
	) ) )
	{
		glPrintf("DS: Failed to lock buffer\n");
		fclose(wavein);
		return FALSE;
	}

	INT numwritten=0;

	for (a=0;a<dsbdesc.dwBufferBytes;++a)
	{
		*((BYTE*)soundInStart+a)=fgetc(wavein);

		++numwritten;
		if (feof(wavein)) 
		{
			glPrintf("ERROR with buffer write at byte %d\n",a);
			break;
		}
	}
	


	//copy wave data to buffer
	
	//unlock the buffer (load it into memory)
    if (FAILED( buf->Unlock( soundInStart, soundBytes, NULL, 0 ) ))
	{
		glPrintf("DS: Failed to unlock buffer\n");
		fclose(wavein);
		return FALSE;
	}

	fclose(wavein);

	
//	pSegment=music.loadNewSound(soundFN);



	++numSounds;

	exist=TRUE;

	return TRUE;
}


BOOL Sound::play(void)
{
	if (!UO.sound || !exist) 
	{
		return FALSE;
	}
	
	buf->SetCurrentPosition( 0 );  
	if (FAILED(buf->Play(0,0,0)) )
	{
		return FALSE;
	}
	
	/*
	debugPrint("playing sound");
	music.getPerformance()->PlaySegmentEx(
											pSegment,  
											NULL,    //song
											NULL,    // transitions. 
											DMUS_SEGF_SECONDARY ,     // Flags.
											0,     // Start time; 0 is immediate.
											NULL,    // Pointer that receives segment state.
											NULL,    // Object to stop.
											NULL   // Audiopath, if not default.
										  );  */
	

	return TRUE;

}

void Sound::setVolume(float amt)
{
	volume=amt;
	LONG attenuation=-10000*(float)(1.0-amt);
	buf->SetVolume(attenuation);
}


BOOL Sound::playRepeat(void)
{
	if (!UO.sound || !exist) 
	{
		return FALSE;
	}

	buf->SetCurrentPosition( 0 );  
	if (FAILED(buf->Play(0,0,DSBPLAY_LOOPING)) )
	{
		return FALSE;
	}
	/*
	music.getPerformance()->PlaySegmentEx(
											pSegment,  
											NULL,    //song
											NULL,    // transitions. 
											0,     // Flags.
											0,     // Start time; 0 is immediate.
											NULL,    // Pointer that receives segment state.
											NULL,    // Object to stop.
											NULL   // Audiopath, if not default.
										  );  
*/
	playing=true;
	return TRUE;

}

BOOL Sound::stop(void)
{
	if (!UO.sound || !exist) 
	{
		return FALSE;
	}
	if (!playing)
	{
		return FALSE;
	}

	playing=false;
	if (FAILED(buf->Stop()))
	{
		return FALSE;
	}

	return TRUE;
}


BOOL stream_search(FILE * stream,char * text)
{
	
	int textPos=0;

	//search through stream until either the end is reached or the text is found
	while (feof(stream)==0 && text[textPos]!=NULL)      
	{
		if (fgetc(stream)==text[textPos]) ++textPos;
		else textPos=0;
	}

	//if end of text was not reached, text was not found, so return FALSE;
	if (text[textPos]!=NULL) return FALSE;   

	return TRUE;
}

void Sound::getDuplicate(Sound * soundOut)
{
	//lpDirectSound->DuplicateSoundBuffer(buf, &soundOut->buf);
	soundOut->buf=buf;
	soundOut->exist=true;
}




//////////////////////////////////////////////////////////////////













void sound_setVolume(float vol)
{
	//doesn't work for some obscure reason
	//	LONG attenuation=-10000*(float)(1.0-vol);
	
	//primebuf->SetVolume(attenuation);

	masterVolume=vol;
	//set the volumes of the secondary buffers instead
	for (int a=0;a<MAX_SOUNDS;++a)
	{
		if (soundst[a].exists())
		{
			soundst[a].updateVolume();
		}
	}

}


float sound_getVolume()
{
	return masterVolume;
}