/*
Texture.cpp

Version 2.1

Texture class

(c)2002 Ben Woodhouse

ben@elf-stone.com
www.elf-stone.com

This code may not be used in a commercial product without my permission.
If you redistribute it, this message must remain intact. 
*/

#include "stdafx.h"

CTexture Tex[MAX_TEXTURES];
GLsizei numTextures;

int TEX_currentTexGLType[2]={0,0};
int TEX_activeChannel=0;
int *TEX_currentChannelGLTypeP=&TEX_currentTexGLType[0];

void readyAllTextures(void)
{
	int a;
	//int progress=0;

		
	glPrintf("Initializing textures. Please wait.\n");
	screen.update();	

	screen.push2D();

	//prioritize textures doesn't work, so use this instead
	for (a=0;a<numTextures;++a)
	{
/*
		glCls();
		glPrintf("Initializing textures. Please wait.\n");
		glPrintf("Progress: %d%%\n",progress);
		glText();
		SwapBuffers(hDC);*/

		Tex[a].bind();
//		glBindTexture(GL_TEXTURE_2D,texNames[a]);
		glBegin(GL_TRIANGLES);
			glTexCoord2i(0,0); 
			glVertex2i(0,0);
			glTexCoord2i(1,0);
			glVertex2i(256,0);
			glTexCoord2i(1,1);
			glVertex2i(256,256);
		glEnd();
		//progress+=100.0/numTextures;
	}
	screen.pop2D(); 
//	glCls();
}

void glDeleteTexture(GLuint tex)
{
	GLuint texarray[1];

	texarray[0]=tex;

	glDeleteTextures(1,texarray);
}


int getFileType(const char * fileName)
{
	char  ftype[3][4]={"RAW","BMP","JPG"};
	int count;
	//{{'R','A','W',NULL},{'B','M','P',M},{'J','P','G'}}


	//find the end of the string
	for (int a=0;fileName[a]!=NULL;++a);

	//fileName[a] is now the NULL char

	if (a-4<0) return UNKNOWN;
	
	//if 4 chars back from the NULL is not a '.' then return
	if (fileName[a-4]!='.') 
	{
		return UNKNOWN;
	}


	//check each type to see if its that one
	for (int t=0;t<3;++t)
	{
		count=0;


		for (int b=0;b<3;++b)
		{
			if (toupper(fileName[a-3+b])==ftype[t][b]) ++count;
			else break;
		}
		if (count==3) return t;
	}

	return UNKNOWN; 


}



CTexture * loadNewCubemap(			const char * basefn,
									GLenum magFilter, 
									GLenum minFilter,
									int fileType,
									int additional,
									FILE * openFile)
{
	CImage img[6];

	CTexture * texp=getNextTexture();
	std::string fn[6];
	int a;

	if (!extensions.exists("GL_EXT_texture_cube_map"))
	{
		messageBox("extension not supported");
		return texp;
	}



	if (fileType==UNKNOWN)
	{
		fileType=getFileType(basefn);
		if (fileType==UNKNOWN)
		{
			messageBox("can't find filetype");
			return texp;
		}
	}


	int numStars=0,star;
	//find which character is the *

	for (a=0;basefn[a]!=NULL;++a)
	{
		if (basefn[a]=='*') 
		{
			star=a;
			++numStars; //should be 1
//			break;
		}

	}

	if (numStars!=1)
	{
		messageBox("wrong number of stars %d",numStars);
		return texp;
	}

	//copy the filename into 6 other filenames with the * replaced
	for (a=0;a<6;++a)
	{
		fn[a]=basefn;

		fn[a][star]=48+a;
	}
	

	switch (fileType)
	{
		case RAW:
			if (openFile==NULL)
			{
				for (a=0;a<6;++a)
				{
					if (openFile==NULL) img[a].loadRAW(fn[a].c_str());
				}
			}
			else
			{
				for (a=0;a<6;++a)
				{
					img[a].loadRAWEmbedded(openFile);
				}
			}
		break;

		case BMP:
			if (openFile==NULL)
			{
				for (a=0;a<6;++a)
				{
					if (openFile==NULL) img[a].loadBMP(fn[a].c_str());
				}
			}
			else
			{
				for (a=0;a<6;++a)
				{
					img[a].loadBMPEmbedded(openFile);
				}
			}
		break;

		case JPG:
			if (openFile==NULL)
			{
				for (a=0;a<6;++a)
				{
					if (openFile==NULL) img[a].loadJPG(fn[a].c_str());
				}
			}
			else messageBox("embedded JPG loading not yet supported");
		break;
	}

	switch (additional)
	{
		case LOADTEX_ALPHAMAP:
			for (a=0;a<6;++a)
				img[a].duplicateChannel(4);
			break;
	}
		
	texp->loadCubemapFromImages(img,
						magFilter, 
						minFilter);

	return texp;

}


CTexture * loadNewTexture(			const char * texfn,
									GLenum magFilter, 
									GLenum minFilter,
									int fileType,
									int additional,
									FILE * openFile)
{
	CImage img;
	CTexture * texp=getNextTexture();

	if (fileType==UNKNOWN)
	{
		fileType=getFileType(texfn);
		if (fileType==UNKNOWN)
		{
			return texp;
		}
	}
	switch (fileType)
	{
		case RAW:
			if (openFile==NULL) img.loadRAW(texfn);
			else img.loadRAWEmbedded(openFile);
		break;

		case BMP:
			if (openFile==NULL) img.loadBMP(texfn);
			else img.loadBMPEmbedded(openFile);
		break;

		case JPG:
			if (openFile==NULL) img.loadJPG(texfn);
			else messageBox("embedded JPG loading not yet supported");
		break;
	}

	switch (additional)
	{
		case LOADTEX_ALPHAMAP:
				img.duplicateChannel(4);
			break;
	}

		
	texp->loadFromImage(img,
						magFilter, 
						minFilter);

	return texp;
}



CTexture * loadTexFromImage(CImage & image,
							GLenum magFilter,
							GLenum minFilter)
{
	CTexture * texp=getNextTexture();
	texp->loadFromImage(image,magFilter,minFilter);
	return texp;
	 
}

void initTextures(void) 
{ 
	
	GLuint texNames[MAX_TEXTURES];
	glGenTextures(MAX_TEXTURES, texNames);

	for (int a=0;a<MAX_TEXTURES;++a)
	{
		Tex[a].setTexName(texNames[a]);    
	}
}

void uninitTextures(void)
{
	GLuint texNames[MAX_TEXTURES];
	for (int a=0;a<MAX_TEXTURES;++a)
	{
		texNames[a]=Tex[a].getTexName();
	}
	glDeleteTextures(MAX_TEXTURES, texNames);
}



//CTEXTURE
void CTexture::bind()
{
	if (enabled)
	{
		setTextureLevel(GLType);
  
		glBindTexture(GLType,texName);
	}
}

void CTexture::setGLType(int GLType_in)
{
	GLType=GLType_in;
}

int CTexture::getGLType()
{
	return GLType;
}


CTexture::CTexture()
{
	texName=NULL;
	wrap_s=GL_REPEAT;
	wrap_t=GL_REPEAT;
	magFilter=TF_LML;
	minFilter=TF_LML;
	envMode=GL_MODULATE;
	GLType=GL_TEXTURE_2D;
	enabled=false;
	size=0;
}


void CTexture::setParams(int wrap_s_in, 
				int wrap_t_in,
				int magFilter_in,
				int minFilter_in,
				int envMode_in)
{
	if (wrap_s_in!=NULL)
	{
		wrap_s=wrap_s_in;
		glTexParameteri(GLType, GL_TEXTURE_WRAP_S, wrap_s);

	}
	if (wrap_t_in!=NULL)
	{
		wrap_t=wrap_t_in;
		glTexParameteri(GLType, GL_TEXTURE_WRAP_T, wrap_t);

	}
	if (magFilter_in!=NULL)
	{
		magFilter=magFilter_in;
		glTexParameteri(GLType, GL_TEXTURE_MAG_FILTER, magFilter);
	}

	if (minFilter_in!=NULL)
	{
		minFilter=minFilter_in;
		glTexParameteri(GLType, GL_TEXTURE_MIN_FILTER, minFilter);
	}
	if (envMode_in!=NULL)
	{
		envMode=envMode_in;
		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, envMode);
	}
}





bool CTexture::loadFromImage(
					 CImage & image,
					 GLenum magFilter, 
					 GLenum minFilter)
{
	bool mip=false;
	bool rv=false;

	int size=image.getWidth();

	if (size!=image.getHeight()) return false;

	if (minFilter!=GL_NEAREST && minFilter!=GL_LINEAR) mip=true;
	else if (magFilter!=GL_NEAREST && magFilter!=GL_LINEAR) mip=true;

	//enable the texture before binding it
	enable();
	bind();

	setParams(GL_REPEAT,GL_REPEAT,magFilter,minFilter,GL_MODULATE);

	if (mip)
	{
		gluBuild2DMipmaps(GL_TEXTURE_2D,image.getNumChannels(),size,size,image.getFormat(),
						GL_UNSIGNED_BYTE,image.getData());
	}
	else
	{
		glTexImage2D(GL_TEXTURE_2D,0,image.getNumChannels(),size,size,0,image.getFormat(),
						GL_UNSIGNED_BYTE,image.getData());
	}  

	rv=true;


	return rv;   
}

bool CTexture::loadCubemapFromImages(
							CImage image[6],
							GLenum magFilter,
							GLenum minFilter)
{

	bool mip=false;
	bool rv=false;

	int size=image[0].getWidth();

	if (size!=image[0].getHeight()) return false;

	if (minFilter!=GL_NEAREST && minFilter!=GL_LINEAR) mip=true;
	else if (magFilter!=GL_NEAREST && magFilter!=GL_LINEAR) mip=true;

	//enable the texture before binding it
	setGLType(GL_TEXTURE_CUBE_MAP_EXT);
	enable();

	bind();

	setParams(GL_REPEAT,GL_REPEAT,magFilter,minFilter,GL_DECAL);
/*
	glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_WRAP_T, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
*/
	int a;

	if (mip)
	{
		for (a=0;a<6;++a)
		{
			gluBuild2DMipmaps(	GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT+a,
								image[a].getNumChannels(),
								size,
								size,
								image[a].getFormat(),							
								GL_UNSIGNED_BYTE,
								image[a].getData());	
		}
	}
	else
	{
		for (a=0;a<6;++a)
		{
			glTexImage2D(	GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT+a,
							0,
							image[a].getNumChannels(),
							size,
							size,
							0,
							image[a].getFormat(),
							GL_UNSIGNED_BYTE,
							image[a].getData());
		}
	}  

	rv=true;


	return rv;   
}

CTexture * getNextTexture()
{
	//glPrintf("Texture %d loaded %d\n",numTextures,Tex[numTextures].getTexName());
	++numTextures;
	return &Tex[numTextures-1];
}

/* Initialize a cube map texture object that generates RGB values
 * that when expanded to a [-1,1] range in the register combiners
 * form a normalized vector matching the per-pixel vector used to
 * access the cube map.
 */
CTexture * makeNormalVectorCubeMap(int size)
{
	float vect[3];
	int f;
	int x, y;

	CTexture * texp=getNextTexture(); //get the next texture
	BYTE *pixels;

	if (!extensions.exists("GL_EXT_texture_cube_map"))
	{
		messageBox("GL_EXT_texture_cube_map extension not supported");
		return texp;
	}
	

	pixels = new BYTE[size*size*3];

	glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_WRAP_T, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

/*
  glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_EXT);
  glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_EXT);
  glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_EXT);
  */
 
	int datpos;
	texp->enable();
	texp->bind();
	for (f = 0; f < 6; f++)  //face
	{
		datpos=0;
		for (y = 0; y < size; y++) 
		{
			for (x = 0; x < size; x++) 
			{
				//getCubeVector(i, size, x, y, vector);
				float s=((float)x+0.5)/(float)size;
				float t=((float)y+0.5)/(float)size;

				//make a vector in the 0..1 range
				//scale it and bias it to the -1...1 range

				s=s*2.0-1.0;
				t=t*2.0-1.0;

				switch (f)
				{
				case 0:
					vect[0] = 1.0;
					vect[1] = -t;
					vect[2] = -s;
					break;
				case 1:
					vect[0] = -1.0;
					vect[1] = -t;
					vect[2] = s;
					break;
				case 2:
					vect[0] = s;
					vect[1] = 1.0;
					vect[2] = t;
					break;
				case 3:
					vect[0] = s;
					vect[1] = -1.0;
					vect[2] = -t;
					break;
				case 4:
					vect[0] = s;
					vect[1] = -t;
					vect[2] = 1.0;
					break;
				case 5:
					vect[0] = -s;
					vect[1] = -t;
					vect[2] = -1.0;
					break;
				}

				normalize(vect);

				pixels[datpos++]=128+127*vect[0];
				pixels[datpos++]=128+127*vect[1];
				pixels[datpos++]=128+127*vect[2];
			}
		}
		glTexImage2D(	GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT+f, 0, GL_RGB8,	size, size, 
						0, GL_RGB, GL_UNSIGNED_BYTE, pixels);
	}

	delete [] pixels;

	texp->setGLType(GL_TEXTURE_CUBE_MAP_EXT);


	return texp;
}

