/*
Texture.h

Version 2.1

Texture class

*/

#include "Image.h" 

#define LOADTEX_NORMAL 0 
#define LOADTEX_ALPHAMAP 1
#define LOADTEX_ADDALPHA 2
#define LOADTEX_LOADMIPMAPS 3 

//texture filters 
#define TF_N GL_NEAREST
#define TF_L GL_LINEAR
#define TF_NMN GL_NEAREST_MIPMAP_NEAREST
#define TF_LMN GL_LINEAR_MIPMAP_NEAREST
#define TF_NML GL_NEAREST_MIPMAP_LINEAR
#define TF_LML GL_LINEAR_MIPMAP_LINEAR 

#define UNKNOWN -1
#define RAW 0
#define BMP 1
#define JPG 2


extern int TEX_currentTexGLType[2];
extern int TEX_activeChannel;
extern int *TEX_currentChannelGLTypeP;



//TEXTURES
#define MAX_TEXTURES 256


//texture array
extern GLsizei numTextures;

class CTexture
{
	public:
		CTexture();

		bool loadFromImage(
					 CImage & image,
					 GLenum magFilter, 
					 GLenum minFilter);

		bool loadCubemapFromImages(
					CImage image[6],
					GLenum magFilter,
					GLenum minFilter);

		//call to set params for texture. -1 indicates fields to be left unchanged
		void setParams(int wrap_s_in, 
						int wrap_t_in,
						int magFilter_in,
						int minFilter_in,
						int envMode_in);

		void bind();
		void enable(){enabled=true;}
		void disable(){enabled=false;}
		bool isEnabled(){return enabled;}
		void setGLType(int GLType_in);
		void setTexName(GLuint texName_in){texName=texName_in;}
		GLuint getTexName(){return texName;}
		int getGLType();

	private:
		GLuint texName;
		int wrap_s;
		int wrap_t;
		int magFilter;
		int minFilter;
		int envMode;
		int GLType;
		bool enabled;

		int size;
};


extern CTexture Tex[MAX_TEXTURES];

//guesses a file's type from its extension
int getFileType(const char * fileName);

CTexture * getNextTexture();



//loads a texture and returns a pointer
CTexture * loadNewTexture(			const char * texfn,
									GLenum magFilter, 
									GLenum minFilter,
									int fileType=UNKNOWN,
									int additional=LOADTEX_NORMAL,
									FILE * openFile=NULL);

CTexture * loadTexFromImage(CImage & image,
							GLenum magFilter,
							GLenum minFilter);

CTexture * loadNewCubemap(			const char * basefn,
									GLenum magFilter, 
									GLenum minFilter,
									int fileType=UNKNOWN,
									int additional=LOADTEX_NORMAL,
									FILE * openFile=NULL);



CTexture * makeNormalVectorCubeMap(int size);
void readyAllTextures(void);
void initTextures(void);
void uninitTextures(void);
void glDeleteTexture(GLuint tex);


inline void setTextureLevel(int lev)
{
	if (*TEX_currentChannelGLTypeP!=lev)
	{
		glDisable(*TEX_currentChannelGLTypeP);

/*
		if (lev==GL_TEXTURE_CUBE_MAP_EXT) 
			messageBox("gotOne %d %d",GL_TEXTURE_CUBE_MAP_EXT,lev);
*/
		*TEX_currentChannelGLTypeP=lev;
		if (lev!=0) glEnable(lev);
	}
}

inline void setTexChannel(int channel)
{
	TEX_activeChannel=channel;
	TEX_currentChannelGLTypeP=&TEX_currentTexGLType[channel];
	switch(channel)
	{
		case 0:
			glActiveTextureARB(GL_TEXTURE0_ARB);
		break;
		case 1:
			glActiveTextureARB(GL_TEXTURE1_ARB); 
		break;
	}
}
